"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EngagementEngineConnector = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _types = require("../types");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

'';
/* eslint-enable */

/**
 * An analytics Connector for Mobify's analytics backend, the Engagement Engine.
 *
 * @implements {module:progressive-web-sdk/dist/analytics-integrations/interface.AnalyticsConnector}
 */

class EngagementEngineConnector {
  /**
   * @constructor
   *
   * @param {Object} options Engagement Engine options.
   * @param {String} options.projectSlug (required) The project slug.
   */
  constructor({
    projectSlug
  }) {
    if (!projectSlug) throw new Error(`options.projectSlug must be defined`);
    this.projectSlug = projectSlug;
  }
  /**
   * @inheritDoc
   */


  load() {
    return Promise.resolve().then(() => _interopRequireWildcard(require('sandy-tracking-pixel-client'))).then(Sandy => {
      Sandy.init(window);
      Sandy.create(this.projectSlug, 'auto'); // eslint-disable-line no-undef
      // The act of running Sandy.init() blows away the binding of
      // window.sandy.instance in the pixel client. We are restoring it
      // here for now and will revisit this when we rewrite the Sandy
      // tracking pixel client

      window.sandy.instance = Sandy;
      this.sandy = window.sandy.instance;
      this.tracker = Sandy.default.trackers[Sandy.DEFAULT_TRACKER_NAME];
      this.tracker.set('mobify_adapted', 'true');
      this.tracker.set('referrer', Sandy._global.document.referrer);
      this.tracker.set('platform', 'PWA');
      return Promise.resolve();
    });
  }
  /**
   * @inheritDoc
   */


  track(type, data) {
    this.tracker.set({
      page: this.sandy._global.location.pathname,
      title: this.sandy._global.document.title,
      location: this.sandy._global.location.href,
      referrer: this.sandy._global.document.referrer
    });

    switch (type) {
      case _types.PAGEVIEW:
        data = this.transformPageviewData(data);
        break;

      case _types.PERFORMANCE:
        data = this.transformPerformanceData(data);
        break;

      case _types.UIINTERACTION:
        data = this.transformUIInteractionData(data);
        break;

      case _types.OFFLINE:
        data = this.transformOfflineData(data);
        break;

      case _types.PURCHASE:
        data = this.trackPurchaseData(data);
        return data;

      case _types.ADDTOCART:
      case _types.REMOVEFROMCART:
      case _types.ADDTOWISHLIST:
      case _types.REMOVEFROMWISHLIST:
        data = this.transformShoppingListData(type, data);
        break;

      case _types.APPLEPAYOPTIONDISPLAYED:
      case _types.APPLEPAYBUTTONDISPLAYED:
      case _types.APPLEPAYBUTTONCLICKED:
        data = this.transformApplePayAction(type);
        break;

      case _types.ERROR:
        data = this.transformAppError(data);
        break;

      case _types.LOCALE:
        this.tracker.set('site_locale', data.locale);
        return null;
      // site locale is set on the tracker, but nothing sent to EE.

      default:
        return null;
    }

    if (data) this.tracker.sendEvent(data);
    return data;
  }

  transformPageviewData(data) {
    return {
      data: {
        action: _types.PAGEVIEW,
        category: 'pageview'
      },
      dimensions: {
        templateName: data.templateName
      }
    };
  }

  transformPerformanceData(metrics) {
    return {
      data: {
        action: _types.PERFORMANCE,
        category: 'timing'
      },
      dimensions: metrics
    };
  }

  transformUIInteractionData(data) {
    const {
      subject,
      action,
      object,
      name,
      content
    } = data;
    return {
      data: {
        action: `${subject}${action}${object}`,
        category: 'timing'
      },
      dimensions: {
        container_name: name,
        content: content
      }
    };
  }

  transformOfflineData(data) {
    if (!data.startTime) return null;
    return {
      data: {
        action: _types.OFFLINE,
        category: 'timing'
      },
      dimensions: {
        value: data.startTime
      }
    };
  }

  trackPurchaseData(data) {
    let dataSent = [];
    (0, _types.renameKey)(data.transaction, 'id', 'transaction_id');
    data.products.forEach(product => {
      (0, _types.renameKey)(product, 'id', 'product_id');
      const productData = {
        data: {
          action: 'purchase.product',
          category: 'ecommerce'
        },
        dimensions: _objectSpread({
          transaction_id: data.transaction.transaction_id
        }, product)
      };
      dataSent.push(productData);
      this.tracker.sendEvent(productData);
    });
    const transactionData = {
      data: {
        action: 'purchase',
        category: 'ecommerce'
      },
      dimensions: data.transaction
    };
    dataSent.push(transactionData);
    this.tracker.sendEvent(transactionData);
    return dataSent;
  }

  transformShoppingListData(type, data) {
    const {
      id,
      name,
      category
    } = data.product;
    const {
      count,
      subtotal
    } = data;
    return {
      data: {
        action: type,
        category: 'timing'
      },
      dimensions: {
        product_id: id,
        product_name: name,
        product_category: category,
        cart_items: count,
        value: subtotal
      }
    };
  }

  transformApplePayAction(type) {
    const applePayAction = {
      [_types.APPLEPAYOPTIONDISPLAYED]: 'apple pay payment option displayed',
      [_types.APPLEPAYBUTTONDISPLAYED]: 'apple pay button displayed',
      [_types.APPLEPAYBUTTONCLICKED]: 'apple pay clicked'
    };
    const action = applePayAction[type];
    if (!action) throw new Error('Apple pay action not supported');
    return {
      data: {
        action,
        category: 'ecommerce'
      },
      dimensions: {}
    };
  }

  transformAppError(data) {
    const {
      name,
      content
    } = data;
    return {
      data: {
        action: `appRecieveError`,
        category: 'timing'
      },
      dimensions: {
        container_name: name,
        content: content
      }
    };
  }

}

exports.EngagementEngineConnector = EngagementEngineConnector;