"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.GoogleTagManagerConnector = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _types = require("../types");

var _utils = require("../utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

'';
/* eslint-enable */

/**
 * A Connector for the Analytics Provider: Google Tag Manager.
 *
 * @implements {module:progressive-web-sdk/dist/analytics-integrations/interface.AnalyticsConnector}
 */

class GoogleTagManagerConnector {
  /**
   * @constructor
   *
   * @param {Object} options Google Tag Manager options.
   * @param {String} options.containerId (required) Google Tag Manager Container id.
   */
  constructor({
    containerId
  }) {
    if (!containerId) throw new Error(`containerId cannot be undefined`);
    this.containerId = containerId;
    this.src = `https://www.googletagmanager.com/gtm.js?id=${containerId}`;
  }
  /**
   * @inheritDoc
   */


  load() {
    window.dataLayer = window.dataLayer || [];
    window.dataLayer.push({
      'gtm.start': new Date().getTime(),
      event: 'gtm.js'
    });
    return (0, _utils.loadScript)(this.src);
  }
  /**
   * @inheritDoc
   */


  track(type, data) {
    switch (type) {
      case _types.PAGEVIEW:
        data = this.transformPageviewData(data);
        break;

      case _types.PURCHASE:
        data = this.transformPurchaseData(data);
        break;

      default:
        return null;
    }

    window.dataLayer.push(data);
    return data;
  }

  transformPageviewData(data) {
    return _objectSpread({
      event: 'Pageview'
    }, data);
  }

  transformPurchaseData(data) {
    return {
      event: 'Ecommerce',
      ec: {
        purchase: {
          actionField: data.transaction,
          products: data.products
        }
      }
    };
  }

}

exports.GoogleTagManagerConnector = GoogleTagManagerConnector;