"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related components:
 *
 * * [AccordionItem](#!/AccordionItem)
 * * [Accordion](#!/Accordion)
 *
 * This component is for internal use only within the AccordionItem.
 *
 * @example ./DESIGN.md
 */
class AccordionItemContent extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      fullHeight: 0,
      style: {
        maxHeight: '0px',
        transition: 'none'
      }
    };
    this.animate = this.animate.bind(this);
    this.getFullHeight = this.getFullHeight.bind(this);
  }

  componentDidAppear() {
    this.triggerAnimate();
  }

  componentDidEnter() {
    this.triggerAnimate();
  }

  componentWillLeave(callback) {
    const complete = () => {
      this.props.onAnimationComplete();
      callback();
    };

    this.animate(this.getFullHeight(), 0, complete);
  }

  triggerAnimate() {
    const complete = () => {
      // After the content has fully opened,
      // remove the set height so the content can grow as needed
      this.setState({
        style: {
          maxHeight: 'initial',
          transition: 'none'
        }
      });
      this.props.onAnimationComplete();
    };

    this.animate(0, this.getFullHeight(), complete);
  }

  animate(start, end, callback) {
    const {
      duration,
      easing
    } = this.props;

    const frame = () => {
      return new Promise(resolve => {
        requestAnimationFrame(resolve);
      });
    };

    const setStartState = () => {
      return new Promise(resolve => {
        const state = {
          style: {
            maxHeight: `${start}px`,
            transition: `none`
          }
        };
        this.setState(state, resolve);
      });
    };

    const setEndState = () => {
      return new Promise(resolve => {
        const state = {
          style: {
            maxHeight: `${end}px`,
            transition: `max-height ${duration / 1000}s ${easing}`
          }
        };
        this.setState(state, resolve);
      });
    };

    const waitForAnimation = () => {
      setTimeout(callback, duration);
    };

    frame().then(setStartState).then(frame).then(setEndState).then(waitForAnimation);
  }

  getFullHeight() {
    return this._content.children[0].offsetHeight;
  }

  render() {
    const {
      children
    } = this.props;
    return /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__content-wrapper",
      ref: el => {
        this._content = el;
      },
      style: this.state.style
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__content",
      role: "presentation"
    }, children));
  }

}

AccordionItemContent.propTypes = {
  /**
   * PROVIDED INTERNALLY. Whatever you'd like this AccordionItem to display.
   */
  children: _propTypes.default.node,

  /**
   * PROVIDED INTERNALLY. Duration of the animation in millis.
   */
  duration: _propTypes.default.number,

  /**
   * PROVIDED INTERNALLY. Easing function for the animation.
   */
  easing: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY. Triggered every time the content has finished animating.
   */
  onAnimationComplete: _propTypes.default.func
};
var _default = AccordionItemContent;
exports.default = _default;