"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _icon = _interopRequireDefault(require("../icon"));

var _accordionItemContent = _interopRequireDefault(require("./accordion-item-content"));

var _a11y = require("../../utils/a11y");

var _TransitionGroup = _interopRequireDefault(require("react-transition-group/TransitionGroup"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related components:
 *
 * * [AccordionItemContent](#!/AccordionItemContent)
 * * [Accordion](#!/Accordion)
 *
 * `AccordionItem` acts as expandable container for the accordion's content.
 *
 * @example ./DESIGN.md
 */
class AccordionItem extends _react.default.Component {
  constructor(props) {
    super(props);
    this.id = props.id;
    this.itemId = `accordion__item-${this.id}`;
    this.onClick = this.onClick.bind(this);
    this.onAnimationComplete = this.onAnimationComplete.bind(this);
  }

  onClick(e) {
    this.props.onHeaderClick(e);
  }

  componentWillReceiveProps(nextProps) {
    const {
      shown,
      onOpen,
      onClose
    } = this.props;

    if (nextProps.shown === shown) {
      return;
    }

    if (nextProps.shown) {
      onOpen(this.itemId);
    } else {
      onClose(this.itemId);
    }
  } // NOTE: JSDOM doesn't invoke animation complete events, so this never gets called
  // during tests.

  /* istanbul ignore next */


  onAnimationComplete() {
    const {
      shown,
      onOpened,
      onClosed
    } = this.props;

    if (shown) {
      onOpened(this.itemId);
    } else {
      onClosed(this.itemId);
    }
  }

  render() {
    const {
      children,
      className,
      shown,
      prerender,
      header,
      openIconName,
      closeIconName,
      iconSize,
      iconPosition,
      duration,
      easing
    } = this.props;
    const classes = (0, _classnames.default)('pw-accordion__item', {
      'pw-accordion--is-open': shown,
      'pw-accordion--is-prerender': prerender && !shown
    }, className);
    const headerId = `accordion__header-${this.id}`;
    const contentId = `accordion__content-${this.id}`;
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      id: this.itemId,
      ref: el => {
        this._container = el;
      }
    }, /*#__PURE__*/_react.default.createElement("button", {
      className: "pw-accordion__header",
      onClick: this.onClick,
      onKeyUp: (0, _a11y.onKeyUpWrapper)(this.onClick),
      tabIndex: "0",
      role: "tab",
      "aria-expanded": shown,
      "aria-selected": shown,
      "aria-controls": contentId,
      id: headerId,
      type: "button",
      "data-analytics-name": `${shown ? 'collapse' : 'expand'}_${'accordion'}`,
      "data-analytics-content": this.itemId
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: `pw-accordion__inner-header pw--icon-${iconPosition}`
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__icon",
      "aria-hidden": "true"
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__open-icon"
    }, /*#__PURE__*/_react.default.createElement(_icon.default, {
      className: "pw-accordion__glyph",
      size: iconSize,
      name: openIconName
    })), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__close-icon"
    }, /*#__PURE__*/_react.default.createElement(_icon.default, {
      className: "pw-accordion__glyph",
      size: iconSize,
      name: closeIconName
    }))), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-accordion__title"
    }, header))), !shown && prerender && /*#__PURE__*/_react.default.createElement(_accordionItemContent.default, null, children), /*#__PURE__*/_react.default.createElement(_TransitionGroup.default, {
      component: "div",
      role: "tabpanel",
      id: contentId,
      "aria-hidden": !shown,
      "aria-labelledby": headerId,
      tabIndex: shown ? 0 : -1
    }, shown && /*#__PURE__*/_react.default.createElement(_accordionItemContent.default, {
      onAnimationComplete: this.onAnimationComplete,
      duration: duration,
      easing: easing
    }, children)));
  }

}

AccordionItem.defaultProps = {
  closeIconName: 'minus',
  iconPosition: 'start',
  openIconName: 'plus'
};
AccordionItem.propTypes = {
  /**
   * Whatever you'd like this AccordionItem to display.
   * This can also include more Accordions.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * The name of the icon shown in the header
   * when the accordion can be closed.
   */
  closeIconName: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY. Duration of the animation in millis.
   */
  duration: _propTypes.default.number,

  /**
   * PROVIDED INTERNALLY. Easing function for the animation.
   */
  easing: _propTypes.default.string,

  /**
   * The content that should be displayed as the header.
   */
  header: _propTypes.default.node,

  /**
   * Determines whether the icons should be placed before or after the title.
   */
  iconPosition: _propTypes.default.oneOf(['start', 'end']),

  /**
   * Passes a custom className to the Accordion Item's icon.
   */
  iconSize: _propTypes.default.string,

  /**
   * A unique identifier for this AccordionItem.
   * It is highly recommended that you provide an id for each AccordionItem.
   */
  id: _propTypes.default.string,

  /**
   * The name of the icon shown in the header
   * when the accordion can be opened.
   */
  openIconName: _propTypes.default.string,

  /**
   * This is used by Accordion to prerender the content before opening it.
   */
  prerender: _propTypes.default.bool,

  /**
   * PROVIDED INTERNALLY. This is used by Accordion to open and close the items
   */
  shown: _propTypes.default.bool,

  /**
   * Triggered every time an accordion item is
   * starting to close.  This function is passed the id of the
   * accordion item which is closing.
   */
  onClose: _propTypes.default.func,

  /**
   * Triggered every time an accordion item is
   * finished closing.  This function is passed the id of the
   * accordion item which closed.
   */
  onClosed: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY. This is the callback used by Accordion to
   * open and close the items.
   */
  onHeaderClick: _propTypes.default.func,

  /**
   * Triggered every time an accordion item is
   * starting to open.  This function is passed the id of the
   * accordion item which is opening.
   */
  onOpen: _propTypes.default.func,

  /**
   * Triggered every time an accordion item has
   * finished opening.  This function is passed the id of the
   * accordion item which opened.
   */
  onOpened: _propTypes.default.func
};
var _default = AccordionItem;
exports.default = _default;