"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _bazaarvoiceWrapper = _interopRequireDefault(require("../bazaarvoice-wrapper"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * This component makes it easier to integrate Bazaarvoice reviews into your PWA.
 *  This component allows you to render the reviews for a product based on a product ID.
 */
class BazaarvoiceReview extends _react.default.Component {
  componentDidMount() {
    const {
      apiVersion,
      productId
    } = this.props;

    if (apiVersion === '1') {
      this.apiVersionOneLoadReviews(productId);
    }
  }

  apiVersionOneLoadReviews(productId) {
    const $BV = window.$BV; // We can't really test that a function that doesn't exist wasn't called

    /* istanbul ignore else */

    if ($BV) {
      $BV.ui('rr', 'show_reviews', {
        productId
      });
    }
  }

  render() {
    const {
      apiVersion,
      className,
      containerId,
      productId
    } = this.props;
    const classes = (0, _classnames.default)('pw-bazaarvoice-review', className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, apiVersion === '1' && /*#__PURE__*/_react.default.createElement("div", {
      id: containerId
    }), apiVersion === '2' && /*#__PURE__*/_react.default.createElement("div", {
      "data-bv-show": "reviews",
      "data-bv-product-id": productId
    }));
  }

}

BazaarvoiceReview.defaultProps = {
  apiVersion: '1',
  containerId: 'BVRRContainer'
};
BazaarvoiceReview.propTypes = {
  /**
   * The src for the Bazaarvoice API script
   */
  apiSrc: _propTypes.default.string.isRequired,

  /**
   * The product ID the reviews should be shown for
   */
  productId: _propTypes.default.string.isRequired,

  /**
   * Version of the BazaarVoice API defaults to '1'
   */
  apiVersion: _propTypes.default.string,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * This id is added to the element Bazaarvoice should render the
   * review content in. The containerId will only be used for version 1 of the BazaarVoice API.
   */
  containerId: _propTypes.default.string
};

var _default = (0, _bazaarvoiceWrapper.default)(BazaarvoiceReview);

exports.default = _default;