"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-env jest */
const removeScripts = src => {
  const scripts = global.document.querySelectorAll(`[src="${src}"]`);
  const scriptsAsArray = [...scripts];
  scriptsAsArray.forEach(script => script.remove());
};

const addScript = (src, loading = false, loaded = true) => {
  const script = global.document.createElement('script');
  script.src = src;
  script.classList.add('js-bvapi');

  if (loading) {
    script.classList.add('pw--loading');
  }

  if (loaded) {
    script.classList.add('pw--loaded');
  }

  global.document.body.appendChild(script);
};

describe('bazaarvoiceWrapper higher order component', () => {
  // This HOC uses the AppError, which means that window.Progressive.analytics.send
  // must be defined for this test
  window.Progressive = {
    analytics: {
      send: () => {}
    }
  };

  const mockHtml = /*#__PURE__*/_react.default.createElement("div", {
    className: "mock"
  }, "Fake component");

  const mockComponent = () => mockHtml;

  mockComponent.displayName = 'Foo';
  const WrapperComponent = (0, _index.default)(mockComponent);
  const src = 'http://www.test.com/test.js';
  beforeAll(() => {
    removeScripts(src);
  });
  afterEach(() => {
    removeScripts(src);
  });
  test('loads the Bazaarvoice API', () => {
    (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    const script = document.body.children[0];
    expect(script.src).toBe(src);
  });
  test('loads the Bazaarvoice API only once', () => {
    (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }), /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }), /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    })));
    const scripts = document.querySelectorAll(`[src="${src}"]`);
    expect(scripts.length).toBe(1);
  });
  test('apiLoaded is true if existing script has loaded', () => {
    addScript(src, false, true);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    expect(wrapper.state('apiLoaded')).toBeTruthy();
    const scripts = document.querySelectorAll(`[src="${src}"]`);
    expect(scripts.length).toBe(1);
  });
  test('existing script has loading class but has loaded', () => {
    global.BV = true;
    addScript(src, true, false);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    expect(wrapper.state('apiLoaded')).toBeTruthy();
    const scripts = document.querySelectorAll(`[src="${src}"]`);
    expect(scripts.length).toBe(1);
  });
  test('mounts wrapped component after the script has loaded', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    wrapper.setState({
      apiLoaded: true
    });
    expect(wrapper.find('.mock').length).toBe(1);
  });
  test('calls $BV.configure after the script has loaded', () => {
    window.$BV = {
      configure: jest.fn()
    };
    const configurationOptions = {};
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src,
      configurationOptions: configurationOptions
    }));
    const script = document.querySelector(`[src="${src}"]`);
    script.dispatchEvent(new Event('load', {
      cancelable: false,
      bubbles: false
    }));
    expect(window.$BV.configure).toBeCalledWith('global', configurationOptions);
    expect(wrapper.state('apiLoaded')).toBe(true);
  });
  test('$BV.configure not called with API v2', () => {
    window.$BV = {
      configure: jest.fn()
    };
    const configurationOptions = {};
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src,
      apiVersion: "2",
      configurationOptions: configurationOptions
    }));
    const script = document.querySelector(`[src="${src}"]`);
    script.dispatchEvent(new Event('load', {
      cancelable: false,
      bubbles: false
    }));
    expect(window.$BV.configure).not.toHaveBeenCalled();
    expect(wrapper.state('apiLoaded')).toBe(true);
  });
  test('calls onError when the script fails to load', () => {
    const onError = jest.fn();
    (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src,
      onError: onError
    }));
    const script = document.querySelector(`[src="${src}"]`);
    script.dispatchEvent(new Event('error', {
      cancelable: false,
      bubbles: false
    }));
    expect(onError).toBeCalled();
  });
  test('sets apiLoaded to true if another instance has already loaded it', () => {
    window.$BV = {
      configure: () => {}
    };
    (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    const script = document.querySelector(`[src="${src}"]`);
    script.dispatchEvent(new Event('load', {
      cancelable: false,
      bubbles: false
    }));
    const secondWrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src
    }));
    expect(secondWrapper.state('apiLoaded')).toBe(true);
  });
  test('remove event listeners on component unmount', () => {
    window.$BV = {
      configure: jest.fn()
    };
    const onError = jest.fn();
    const configurationOptions = {};
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      apiSrc: src,
      onError: onError,
      configurationOptions: configurationOptions
    }));
    const script = document.querySelector(`[src="${src}"]`);
    wrapper.unmount();
    script.dispatchEvent(new Event('load', {
      cancelable: false,
      bubbles: false
    }));
    script.dispatchEvent(new Event('error', {
      cancelable: false,
      bubbles: false
    }));
    expect(onError).not.toHaveBeenCalled();
    expect(window.$BV.configure).not.toHaveBeenCalled();
  });
});