"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _link = _interopRequireDefault(require("../link"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * `Breadcrumbs` is used to provide users a navigational path they took to get to the current page.
 *
 * @example ./DESIGN.md
 */
const Breadcrumbs = ({
  className,
  items,
  youAreHereMessage,
  includeMicroData
}) => {
  if (!items.length) {
    return false;
  }

  let breadcrumbListMicroData = {};
  let listMicroData = {};
  let thingMicroData = {};

  if (includeMicroData) {
    breadcrumbListMicroData = {
      itemScope: true,
      itemType: 'http://schema.org/BreadcrumbList'
    };
    listMicroData = {
      itemProp: 'itemListElement',
      itemScope: true,
      itemType: 'http://schema.org/ListItem'
    };
    thingMicroData = {
      itemScope: true,
      itemProp: 'item',
      itemType: 'http://schema.org/Thing'
    };
  }

  return /*#__PURE__*/_react.default.createElement("nav", {
    role: "navigation",
    className: (0, _classnames.default)('pw-breadcrumbs', className)
  }, /*#__PURE__*/_react.default.createElement("p", {
    id: "breadcrumb__label",
    className: "pw-breadcrumbs__label u-visually-hidden"
  }, youAreHereMessage, ": ", items[items.length - 1].text), /*#__PURE__*/_react.default.createElement("ol", (0, _extends2.default)({
    "aria-labelledby": "breadcrumb__label",
    className: "pw-breadcrumbs__list"
  }, breadcrumbListMicroData), items.map(({
    href,
    text,
    onClick
  }, index) => {
    const itemContent = includeMicroData ? /*#__PURE__*/_react.default.createElement("span", {
      itemProp: "name"
    }, text, href && /*#__PURE__*/_react.default.createElement("meta", {
      itemProp: "url",
      content: href
    })) : text;
    return /*#__PURE__*/_react.default.createElement("li", (0, _extends2.default)({
      className: "pw-breadcrumbs__item",
      key: index
    }, listMicroData), href || onClick ? /*#__PURE__*/_react.default.createElement(_link.default, (0, _extends2.default)({
      href: href,
      className: "pw-breadcrumbs__link",
      onClick: onClick
    }, thingMicroData), itemContent) : /*#__PURE__*/_react.default.createElement("span", (0, _extends2.default)({
      className: "pw-breadcrumbs__non-link"
    }, thingMicroData), itemContent), includeMicroData && /*#__PURE__*/_react.default.createElement("meta", {
      itemProp: "position",
      content: index + 1
    }));
  })));
};

Breadcrumbs.defaultProps = {
  items: [],
  youAreHereMessage: 'You are here'
};
Breadcrumbs.propTypes = {
  /**
   * The list of breadcrumbs.
   * Each breadcrumb item should be an object (More info on items prop below).
   */
  items: _propTypes.default.arrayOf(_propTypes.default.shape({
    text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]).required,
    href: _propTypes.default.string,
    onClick: _propTypes.default.func
  })).isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Indicates if SEO microdata should be included in the breadcrumb element
   * https://developers.google.com/search/docs/data-types/breadcrumbs
   * https://www.w3.org/TR/microdata/
   */
  includeMicroData: _propTypes.default.bool,

  /**
   * This component has a visually hidden label to make it more accessible to screen readers.
   * You can change this property to change the label's content.
   */
  youAreHereMessage: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = Breadcrumbs;
exports.default = _default;