"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index"));

var _link = _interopRequireDefault(require("../link"));

var _icon = _interopRequireDefault(require("../icon"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-disable newline-per-chained-call */
test('Button renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.length).toBe(1);
});
test('returns a link if the href prop is passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    href: "http://test.com"
  }, "Test"));
  const links = wrapper.find(_link.default);
  expect(links.length).toBe(1);
  const link = links.first();
  expect(link.prop('href')).toBe('http://test.com');
  expect(link.childAt(0).text()).toBe('Test');
});
test('returns a button if the href prop is not passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    name: "test"
  }));
  const buttons = wrapper.find('button');
  expect(buttons.length).toBe(1);
  const button = buttons.first();
  expect(button.prop('name')).toBe('test');
});
test('Button renders with inner container', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.find('.pw-button__inner').length).toBe(1);
});
test('includes its children as the children of the button', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement("div", {
    className: "test"
  }, "Test")));
  expect(wrapper.contains( /*#__PURE__*/_react.default.createElement("div", {
    className: "test"
  }, "Test"))).toBe(true);
});
test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('pw-button')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name
    }));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
test('renders the contents of the innerClassName prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      innerClassName: name
    }));
    expect(wrapper.find('.pw-button__inner').hasClass(name)).toBe(true);
  });
});
test('renders an icon if the icon prop is passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    icon: "cart"
  }));
  expect(wrapper.find(_icon.default).length).toBe(1);
  expect(wrapper.find(_icon.default).prop('name')).toBe('cart');
});
test('renders an icon and a hidden title if the icon and title props are passed', () => {
  const title = 'Shopping Cart';
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    icon: "cart",
    title: title
  }));
  expect(wrapper.find(_icon.default).length).toBe(1);
  expect(wrapper.find('span').length).toBe(1);
  expect(wrapper.find('span').first().hasClass('u-visually-hidden')).toBe(true);
  expect(wrapper.find('span').first().text()).toBe(title);
});
test('renders a visible title if the icon and title props are passed with showIconText set to true', () => {
  const title = 'Shopping Cart';
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    icon: "cart",
    title: title,
    showIconText: true
  }));
  expect(wrapper.find('span').first().hasClass('pw-button__text')).toBe(true);
  expect(wrapper.find('span').first().text()).toBe(title);
});
test('renders a string child whole when an icon is passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    icon: "x"
  }, "Child String"));
  const inner = wrapper.find('.pw-button__inner');
  expect(inner.children().length).toBe(2);
  expect(inner.childAt(0).type()).toBe(_icon.default);
  expect(inner.childAt(1).text()).toBe('Child String');
});
test('renders its children alongside the icon', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    icon: "x"
  }, /*#__PURE__*/_react.default.createElement("hr", null), /*#__PURE__*/_react.default.createElement("hr", null)));
  const inner = wrapper.find('.pw-button__inner');
  expect(inner.children().length).toBe(3);
  expect(inner.children().map(child => child.type())).toEqual([_icon.default, 'hr', 'hr']);
});
test('adds aria attributes', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    role: "radio",
    "aria-checked": true
  }));
  const rawButtonProps = wrapper.find('button').props();
  expect(rawButtonProps['aria-checked']).toBe(true);
});
test('onMouseEnter fires on hover', () => {
  const mockOnMouseEnter = jest.fn();
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    onMouseEnter: mockOnMouseEnter
  }));
  wrapper.simulate('mouseEnter');
  expect(mockOnMouseEnter.mock.calls.length).toBe(1);
});