"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _link = _interopRequireDefault(require("../link"));

var _icon = _interopRequireDefault(require("../icon"));

var _a11y = require("../../utils/a11y");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * A styleable, accessible button component.
 *
 * @example ./DESIGN.md
 */
class Button extends _react.default.PureComponent {
  render() {
    const {
      // Values
      href,
      icon,
      iconSize,
      iconClassName,
      innerClassName,
      showIconText,
      text,
      title,
      type,
      // Attributes
      id,
      className,
      disabled,
      name,
      value,
      role,
      openInNewTab,
      // Handlers
      onBlur,
      onClick,
      onFocus,
      onMouseEnter,
      onMouseLeave
    } = this.props;
    const classes = (0, _classnames.default)('pw-button', {
      'pw--anchor': !!href,
      'pw--icon-only': !!icon && !this.props.children
    }, className);
    const innerClass = (0, _classnames.default)('pw-button__inner', innerClassName);
    const iconClass = (0, _classnames.default)('pw-button__icon', iconClassName, {
      'pw--has-siblings': this.props.children || title && showIconText
    });
    const textClass = (0, _classnames.default)('pw-button__text', {
      'u-visually-hidden': !showIconText
    });
    const attrs = {
      href,
      id,
      disabled,
      name,
      value,
      role,
      onBlur,
      onClick,
      onFocus,
      onMouseEnter,
      onMouseLeave,
      onKeyUp: (0, _a11y.onKeyUpWrapper)(onClick),
      className: classes
    };
    let children;

    if (icon) {
      children = [/*#__PURE__*/_react.default.createElement(_icon.default, {
        className: iconClass,
        size: iconSize,
        name: icon,
        key: "autoicon"
      }), title && /*#__PURE__*/_react.default.createElement("span", {
        className: textClass,
        key: "autotitle"
      }, title)];

      if (typeof this.props.children === 'string') {
        children.push(this.props.children);
      } else {
        children.push(...(this.props.children || []));
      }
    } else {
      children = text || this.props.children;
    } // Add all aria and data attributes


    Object.keys(this.props).forEach(key => {
      if (/^(aria|data)-/.test(key)) {
        attrs[key] = this.props[key];
      }
    });

    if (href) {
      return /*#__PURE__*/_react.default.createElement(_link.default, (0, _extends2.default)({}, attrs, {
        openInNewTab: openInNewTab
      }), /*#__PURE__*/_react.default.createElement("div", {
        className: innerClass
      }, children));
    } else {
      return /*#__PURE__*/_react.default.createElement("button", (0, _extends2.default)({}, attrs, {
        type: type
      }), /*#__PURE__*/_react.default.createElement("div", {
        className: innerClass
      }, children));
    }
  }

}

Button.defaultProps = {
  type: 'button'
};
Button.propTypes = {
  /**
   * Any children to be nested within this button.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Defines if button is disabled.
   */
  disabled: _propTypes.default.bool,

  /**
   * If specified, the component is rendered as a link, with this value set as the href.
   */
  href: _propTypes.default.string,

  /**
   * If specified, includes an icon of the given name in the button.
   * For more information about available icons, see the `Icon` component.
   */
  icon: _propTypes.default.string,

  /**
   * Adds values to the class attribute in the `Icon` component.
   */
  iconClassName: _propTypes.default.string,

  /**
   * If specified, will set the icon to the size of your choice.
   */
  iconSize: _propTypes.default.string,

  /**
   * Sets the `id` attribute of the root element.
   */
  id: _propTypes.default.string,

  /**
   * Adds values to the class attribute of the inner container.
   */
  innerClassName: _propTypes.default.string,

  /**
   * The button's `name` attribute.
   */
  name: _propTypes.default.string,

  /**
   * For use with Buttons with an href set.
   *
   * If true, target="_blank" will be added to the button.
   * Only use this property if you trust the link! https://mathiasbynens.github.io/rel-noopener
   */
  openInNewTab: _propTypes.default.bool,

  /**
   * The button's `role` attribute.
   */
  role: _propTypes.default.string,

  /**
   * For use when the icon and title attributes have been defined.
   * If false, `u-visually-hidden` class will be added to the container that wraps
   * the title attribute. If true, `u-visually-hidden` will be removed.
   *
   */
  showIconText: _propTypes.default.bool,

  /**
   * Text contents of the button.
   */
  text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The title to be used for accessibility (generally if `icon` is used).
   * If showIconText is set to true, this text will be shown with the icon
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Specifies button type, defaults to `button`.
   */
  type: _propTypes.default.oneOf(['button', 'submit']),

  /**
   * The button's `value` attribute.
   */
  value: _propTypes.default.string,

  /**
   * User-defined method for hooking into blur events. Use with
   * onMouseLeave as a keyboard substitute for hover events.
   */
  onBlur: _propTypes.default.func,

  /**
   * User-defined method for hooking into click events.
   */
  onClick: _propTypes.default.func,

  /**
   * User-defined method for hooking into focus events. Use with
   * onMouseEnter as a keyboard substitute for hover events.
   */
  onFocus: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseEnter events. Use with the onFocus prop for keyboard a11y purposes.
   */
  onMouseEnter: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseLeave events. Use with the onBlur prop for keyboard a11y purposes.
   */
  onMouseLeave: _propTypes.default.func
};
var _default = Button;
exports.default = _default;