"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Card component is used to show content in a card with header and footer
 */
const Card = ({
  className,
  header,
  children,
  footer,
  hasShadow,
  hasBorder,
  headerClassName,
  innerClassName,
  itemScope,
  itemType
}) => {
  const classes = (0, _classnames.default)('pw-card', {
    'c--shadow': hasShadow,
    'c--border': hasBorder
  }, className);
  const headerClasses = (0, _classnames.default)('pw-card__header', headerClassName);
  const innerClasses = (0, _classnames.default)('pw-card__inner', innerClassName);
  return /*#__PURE__*/_react.default.createElement("article", {
    className: classes,
    itemScope: itemScope,
    itemType: itemType
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: innerClasses
  }, header && /*#__PURE__*/_react.default.createElement("header", {
    className: headerClasses
  }, header), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-card__content"
  }, children), footer && /*#__PURE__*/_react.default.createElement("footer", {
    className: "pw-card__footer"
  }, footer)));
};

Card.propTypes = {
  /**
   * Main content of the card
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * Footer content of the card
   */
  footer: _propTypes.default.node,

  /**
   * Determines if card has border
   */
  hasBorder: _propTypes.default.bool,

  /**
   * Determines if card has box-shadow
   */
  hasShadow: _propTypes.default.bool,

  /**
   * Header content of the card
   */
  header: _propTypes.default.node,

  /**
   * Classes to add to the header
   */
  headerClassName: _propTypes.default.string,

  /**
   * Classes to add to the inner container of the component
   */
  innerClassName: _propTypes.default.string,

  /**
   * A value for the itemScope attribute
   * used to provide microdata to a page for SEO
   * https://www.w3.org/TR/microdata/
   */
  itemScope: _propTypes.default.bool,

  /**
   * A value for the itemType attribute
   * used to provide microdata to a page for SEO
   * https://www.w3.org/TR/microdata/
   */
  itemType: _propTypes.default.string
};
var _default = Card;
exports.default = _default;