"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _link = _interopRequireDefault(require("../link"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related component:
 *
 * * [Carousel](#!/Carousel)
 *
 * A single item to be displaced within the `Carousel` component.
 *
 * @example ./DESIGN.md
 */
const CarouselItem = ({
  active,
  isAnimating,
  isDragging,
  children,
  className,
  href,
  onClick,
  openInNewTab
}) => {
  const shouldRender = active || isAnimating || isDragging;
  const Item = href ? _link.default : 'div';
  const itemProps = {
    className: (0, _classnames.default)(`pw-carousel__item`, {
      'pw--active': active
    }, className),
    'aria-hidden': active ? 'false' : 'true',
    'aria-live': active ? 'polite' : '',
    onClick
  };
  const itemVisibility = {
    display: shouldRender ? 'block' : 'none'
  };

  if (href) {
    itemProps.href = href;
    itemProps.openInNewTab = openInNewTab;
  }

  return /*#__PURE__*/_react.default.createElement(Item, itemProps, /*#__PURE__*/_react.default.createElement("div", {
    style: itemVisibility
  }, children));
};

CarouselItem.propTypes = {
  /**
   * PROVIDED INTERNALLY: Defines if the item is active or not.
   */
  active: _propTypes.default.bool,

  /**
   * The contents of the carousel item, to be shown in the carousel.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * If specified, the component is rendered as a link, with this value set as the href.
   */
  href: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY: Defines if the item is animating or not.
   */
  isAnimating: _propTypes.default.bool,

  /**
   * PROVIDED INTERNALLY: Defines if the item is dragging or not.
   */
  isDragging: _propTypes.default.bool,

  /**
   * If rendered as a link, when the CarouselItem is clicked the corresponding link opens in a new tab.
   */
  openInNewTab: _propTypes.default.bool,

  /**
   * A callback to be called when the CarouselItem is clicked.
   */
  onClick: _propTypes.default.func
};
var _default = CarouselItem;
exports.default = _default;