"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _link = _interopRequireDefault(require("../link"));

var _index = _interopRequireDefault(require("./index.jsx"));

var _carouselItem = _interopRequireDefault(require("./carousel-item.jsx"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/* eslint-disable newline-per-chained-call, max-nested-callbacks */
describe('Carousel', () => {
  test('renders properly', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 1"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img1.jpg"
    })), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 2"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img2.jpg"
    })), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 3"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img3.jpg"
    }))));
    const carouselItems = wrapper.find('.pw-carousel__item');
    expect(carouselItems.length).toBe(3);
    const middleItem = carouselItems.at(1);
    const middleItemClassName = middleItem.prop('className');
    expect(middleItemClassName.includes('pw--active')).toBe(true);
    expect(middleItem.find('img').prop('src')).toBe('/img1.jpg');
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.hasClass('pw-carousel')).toBe(true);
  });
  test('only one carouselItems if there is only one item', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 1"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img1.jpg"
    }))));
    const carouselItems = wrapper.find(_carouselItem.default);
    expect(carouselItems.length).toBe(1);
  });
  test('three carouselItems if there are only two items', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 1"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img1.jpg"
    })), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      caption: "Item 2"
    }, /*#__PURE__*/_react.default.createElement("img", {
      src: "/img2.jpg"
    }))));
    const carouselItems = wrapper.find(_carouselItem.default);
    expect(carouselItems.length).toBe(3);
  });
  test('adjusts cur/prev/next indices when carouselItems length changes', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.state('currentIndex')).toBe(0);
    expect(wrapper.state('prevIndex')).toBe(0);
    expect(wrapper.state('nextIndex')).toBe(0);
    [2, 3, 4, 5].forEach(childCount => {
      const children = new Array(childCount).fill(0).map((_, idx) => /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
        key: idx
      }));
      wrapper.setProps({
        children
      });
      expect(wrapper.state('currentIndex')).toBe(0);
      expect(wrapper.state('prevIndex')).toBe(childCount - 1);
      expect(wrapper.state('nextIndex')).toBe(1);
    });
  });
  test('does not adjust current/prev/next indices when carouselItems length does not change', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.state('currentIndex')).toBe(0);
    expect(wrapper.state('prevIndex')).toBe(2);
    expect(wrapper.state('nextIndex')).toBe(1);
    wrapper.setProps({
      previousIcon: 'plus'
    });
    expect(wrapper.state('currentIndex')).toBe(0);
    expect(wrapper.state('prevIndex')).toBe(2);
    expect(wrapper.state('nextIndex')).toBe(1);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
        className: name
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('renders nothing with no children', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
    expect(wrapper.type()).toBe(null);
  });
  test('renders controls by default', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.find('.pw-carousel__controls').length).toBe(1);
  });
  test('does not render controls if showControls and showPips is false', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      showControls: false,
      showPips: false
    }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.find('.pw-carousel__controls').length).toBe(0);
  });
  test('renders pips if showPips is true', () => {
    ;
    [true, false].forEach(showControls => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
        showControls: showControls,
        showPips: true
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      expect(wrapper.find('.pw-carousel__pips').length).toBe(1);
    });
  });
  test('renders a pip for each child', () => {
    ;
    [2, 3, 4, 5, 6].forEach(childCount => {
      const children = new Array(childCount).fill(0).map((_, idx) => /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
        key: idx
      }));
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, children));
      expect(wrapper.find('CarouselPip').length).toBe(childCount);
    });
  });
  test('renders no caption by default', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.find('.pw-carousel__caption').length).toBe(0);
  });
  test('renders the caption if showCaption is true', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      showCaption: true
    }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.find('.pw-carousel__caption').length).toBe(1);
  });
  test('mousedown and touchstart event listeners are added to the inner wrapper on mount', () => {
    const mockInnerWrapper = {
      current: {
        addEventListener: jest.fn()
      }
    };
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)), {
      disableLifecycleMethods: true
    });
    const instance = wrapper.instance();
    instance._innerWrapper = mockInnerWrapper;
    instance.componentDidMount();
    expect(mockInnerWrapper.current.addEventListener).toHaveBeenCalledWith('mousedown', expect.any(Function));
    expect(mockInnerWrapper.current.addEventListener).toHaveBeenCalledWith('touchstart', expect.any(Function), {
      passive: false
    });
  });
  test('mousedown and touchstart event listeners are removed from the inner wrapper on unmount', () => {
    const mockInnerWrapper = {
      current: {
        removeEventListener: jest.fn()
      }
    };
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)), {
      disableLifecycleMethods: true
    });
    const instance = wrapper.instance();
    instance._innerWrapper = mockInnerWrapper;
    instance.componentWillUnmount();
    expect(mockInnerWrapper.current.removeEventListener).toHaveBeenCalledWith('mousedown', expect.any(Function));
    expect(mockInnerWrapper.current.removeEventListener).toHaveBeenCalledWith('touchstart', expect.any(Function), {
      passive: false
    });
  });
  test('onDownHandler does nothing if the animate flag is set', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const mockEvent = {
      preventDefault: jest.fn()
    };
    wrapper.setState({
      animate: true
    });

    const previousState = _objectSpread({}, wrapper.state());

    wrapper.instance().onDownHandler(mockEvent);
    expect(wrapper.state()).toEqual(previousState);
    expect(mockEvent.preventDefault).toBeCalled();
  });
  test('onDownHandler does not set the dragging flag and start position in the state', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const mockEvent = {
      clientX: 50,
      clientY: 100
    };
    wrapper.instance().onDownHandler(mockEvent);
    expect(wrapper.state('dragging')).toBe(false);
    expect(wrapper.state('dragStartX')).toBe(50);
  });
  test('onDownHandler calls addEventListeners', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const instance = wrapper.instance();
    instance.addEventListeners = jest.fn();
    instance.onDownHandler({});
    expect(instance.addEventListeners).toHaveBeenCalled();
  });
  test('onUpHandler clears the dragging flag if deltaX is zero or undefined', () => {
    ;
    [0, undefined].forEach(deltaX => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        deltaX,
        dragging: true
      });
      wrapper.instance().onUpHandler();
      expect(wrapper.state('dragging')).toBe(false);
    });
  });
  test('onUpHandler calls removeEventListeners', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const instance = wrapper.instance();
    instance.removeEventListeners = jest.fn();
    instance.onUpHandler();
    expect(instance.removeEventListeners).toHaveBeenCalled();
  });
  test('onUpHandler clears dragging and sets animate if abs(deltaX) is under the moveThreshold', () => {
    ;
    [-10, 10].forEach(deltaX => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        moveThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        deltaX,
        dragging: true
      });
      wrapper.instance().onUpHandler();
      expect(wrapper.state('dragging')).toBe(false);
      expect(wrapper.state('animate')).toBe(true);
      expect(wrapper.state('deltaX')).toBe(0);
    });
  });
  test('onUpHandler clears the dragging flag and animates when deltaX exceeds the moveThreshold', () => {
    ;
    [1, -1].forEach(sign => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        moveThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        deltaX: 30 * sign,
        dragging: true,
        itemWidth: 200
      });
      wrapper.instance().onUpHandler();
      expect(wrapper.state('dragging')).toBe(false);
      expect(wrapper.state('animate')).toBe(true);
      expect(wrapper.state('deltaX')).toBe(200 * sign);
    });
  });
  test('onUpHandler sends analytics data', () => {
    ;
    [1, -1].forEach(sign => {
      window.Progressive = {
        analytics: {
          send: jest.fn()
        }
      };
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        moveThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        deltaX: 30 * sign,
        dragging: true,
        itemWidth: 200
      });
      wrapper.instance().onUpHandler();
      expect(window.Progressive.analytics.send.mock.calls.length).toBe(1); // cleanup

      delete window.Progressive;
    });
  });
  test('onMoveHandler does nothing if the user is scrolling past (not dragging)', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.setState({
      dragging: false,
      isScrollingPast: true
    });

    const previousState = _objectSpread({}, wrapper.state());

    wrapper.instance().onMoveHandler({});
    expect(wrapper.state()).toEqual(previousState);
  });
  test('onMoveHandler does nothing if the deltaX is under threshold', () => {
    ;
    [-1, 1].forEach(sign => {
      const mockEvent = {
        clientX: 10 * sign,
        preventDefault: jest.fn()
      };
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        dragThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        dragging: true,
        dragStartX: 0
      });

      const previousState = _objectSpread({}, wrapper.state());

      wrapper.instance().onMoveHandler(mockEvent);
      expect(wrapper.state()).toEqual(previousState);
    });
  });
  test('onMoveHandler does nothing if the end of the carousel is reached', () => {
    ;
    [-1, 1].forEach(sign => {
      const mockEvent = {
        clientX: 30 * sign,
        preventDefault: jest.fn()
      };
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        dragThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      const instance = wrapper.instance(); // We need to calculate all indices, not just currentIndex
      // Or the carousel items won't have unique keys

      const indices = instance.calculateIndexes(sign > 0 ? 0 : 1);
      wrapper.setState(_objectSpread({
        dragging: true,
        dragStartX: 0
      }, indices));

      const previousState = _objectSpread({}, wrapper.state());

      instance.onMoveHandler(mockEvent);
      expect(wrapper.state()).toEqual(previousState);
    });
  });
  test('onMoveHandler sets the deltaX if the end of the carousel is not reached', () => {
    ;
    [-1, 1].forEach(sign => {
      const mockEvent = {
        clientX: 30 * sign,
        preventDefault: jest.fn()
      };
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        dragThreshold: 20
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      const instance = wrapper.instance();
      const indices = instance.calculateIndexes(sign < 0 ? 0 : 1);
      wrapper.setState(_objectSpread({
        dragging: true,
        dragStartX: 0
      }, indices));
      instance.onMoveHandler(mockEvent);
      expect(wrapper.state('deltaX')).toEqual(10 * sign);
    });
  });
  test('onMoveHandler calls preventDefault() on the event', () => {
    const mockEvent = {
      clientX: 30,
      preventDefault: jest.fn()
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      dragThreshold: 20
    }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.instance().onMoveHandler(mockEvent);
    expect(mockEvent.preventDefault).toHaveBeenCalled();
  });
  test('onLeaveHandler does nothing if no drag is in progress', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    expect(wrapper.state('dragging')).toBe(false);

    const previousState = _objectSpread({}, wrapper.state());

    wrapper.instance().onLeaveHandler();
    expect(wrapper.state()).toEqual(previousState);
  });
  test('onLeaveHandler generates a mouse up if a drag is in progress', () => {
    const mockHandler = jest.fn();
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.setState({
      dragging: true
    });
    wrapper.instance().onUpHandler = mockHandler;
    wrapper.instance().onLeaveHandler();
    expect(mockHandler).toBeCalled();
  });
  test('clicking on the previous button sets the animate flag and deltaX', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const instance = wrapper.instance();
    const indices = instance.calculateIndexes(1);
    wrapper.setState(_objectSpread({
      itemWidth: 100
    }, indices));
    wrapper.find('.pw-carousel__previous button').simulate('click');
    expect(wrapper.state('animate')).toBe(true);
    expect(wrapper.state('deltaX')).toBe(100);
  });
  test('clicking on the next button sets the animate flag and deltaX', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const instance = wrapper.instance();
    const indices = instance.calculateIndexes(0);
    wrapper.setState({
      itemWidth: 100,
      indices
    });
    wrapper.find('.pw-carousel__next button').simulate('click');
    expect(wrapper.state('animate')).toBe(true);
    expect(wrapper.state('deltaX')).toBe(-100);
  });
  test('moveComplete only clears the animate flag if deltaX is 0 or undefined', () => {
    ;
    [0, undefined].forEach(deltaX => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper.setState({
        deltaX,
        animate: true,
        currentIndex: 0,
        nextIndex: 1,
        prevIndex: 2
      });
      wrapper.instance().moveComplete();
      expect(wrapper.state('animate')).toBe(false);
      expect(wrapper.state('currentIndex')).toBe(0);
      expect(wrapper.state('nextIndex')).toBe(1);
      expect(wrapper.state('prevIndex')).toBe(2);
    });
  });
  test('moveComplete advances the indices if deltaX is negative', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.setState({
      deltaX: -100,
      animate: true,
      currentIndex: 0,
      nextIndex: 1,
      prevIndex: 2
    });
    wrapper.instance().moveComplete();
    expect(wrapper.state('animate')).toBe(false);
    expect(wrapper.state('currentIndex')).toBe(1);
    expect(wrapper.state('nextIndex')).toBe(2);
    expect(wrapper.state('prevIndex')).toBe(0);
    expect(wrapper.state('deltaX')).toBe(0);
  });
  test('moveComplete reduces the indices if deltaX is positive', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.setState({
      deltaX: 100,
      animate: true,
      currentIndex: 1,
      nextIndex: 2,
      prevIndex: 0
    });
    wrapper.instance().moveComplete();
    expect(wrapper.state('animate')).toBe(false);
    expect(wrapper.state('currentIndex')).toBe(0);
    expect(wrapper.state('nextIndex')).toBe(1);
    expect(wrapper.state('prevIndex')).toBe(2);
    expect(wrapper.state('deltaX')).toBe(0);
  });
  test('Pip click moves carousel positive direction', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 0
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 1
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 2
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 3
    })));
    expect(wrapper.find('CarouselPip').length).toBe(4);
    wrapper.setState({
      currentIndex: 0,
      nextIndex: 1,
      prevIndex: 3
    });
    expect(wrapper.find('.pw-carousel__pip.pw--active').length).toBe(1);
    wrapper.find('CarouselPip').last().simulate('click');
    expect(wrapper.state('currentIndex')).toBe(3);
    expect(wrapper.state('nextIndex')).toBe(0);
    expect(wrapper.state('prevIndex')).toBe(2);
    expect(wrapper.find('CarouselPip').last().prop('isCurrentPip')).toBe(true);
  });
  test('Pip click moves carousel negative direction', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 0
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 1
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 2
    }), /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      key: 3
    })));
    expect(wrapper.find('CarouselPip').length).toBe(4);
    wrapper.setState({
      currentIndex: 3,
      nextIndex: 2,
      prevIndex: 0
    });
    expect(wrapper.find('.pw-carousel__pip.pw--active').length).toBe(1);
    wrapper.find('CarouselPip').first().simulate('click');
    expect(wrapper.state('currentIndex')).toBe(0);
    expect(wrapper.state('nextIndex')).toBe(1);
    expect(wrapper.state('prevIndex')).toBe(3);
    expect(wrapper.find('CarouselPip').first().prop('isCurrentPip')).toBe(true);
  });
  test('Setting the value of currentSlide also changes currentIndex to the same value', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      currentSlide: 0
    }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.setProps({
      currentSlide: 2
    });
    wrapper.setState({
      deltaX: -200
    });
    wrapper.instance().moveComplete();
    expect(wrapper.state('currentIndex')).toBe(2);
  });
  test('moveComplete should call onSlideMove, passing it the the currentIndex', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      currentSlide: 0,
      onSlideMove: jest.fn()
    }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const targetSlide = 1;
    wrapper.setProps({
      currentSlide: targetSlide
    });
    wrapper.setState({
      deltaX: -200
    });
    expect(wrapper.prop('onSlideMove')).not.toBeCalled();
    wrapper.instance().moveComplete();
    expect(wrapper.prop('onSlideMove')).toBeCalledWith(targetSlide);
  });
  test('removes resize event listener on unmount', () => {
    const removeEventListener = window.removeEventListener;
    window.removeEventListener = jest.fn(removeEventListener);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const handler = wrapper.instance().updateItemWidth;
    expect(window.removeEventListener).not.toHaveBeenCalledWith('resize', handler);
    wrapper.unmount();
    expect(window.removeEventListener).toHaveBeenCalledWith('resize', handler);
    window.removeEventListener = removeEventListener;
  });
  test('calls removeEventListeners on unmount', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    const instance = wrapper.instance();
    instance._innerWrapper = {};
    instance.removeEventListeners = jest.fn();
    instance.componentWillUnmount();
    expect(instance.removeEventListeners).toHaveBeenCalled();
  });
  test('addEventListeners adds mouse and touch event listeners', () => {
    const addEventListener = window.addEventListener;
    window.addEventListener = jest.fn();
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.instance().addEventListeners();
    expect(window.addEventListener).toHaveBeenCalledWith('mousemove', expect.any(Function));
    expect(window.addEventListener).toHaveBeenCalledWith('mouseup', expect.any(Function));
    expect(window.addEventListener).toHaveBeenCalledWith('mouseleave', expect.any(Function));
    expect(window.addEventListener).toHaveBeenCalledWith('touchend', expect.any(Function));
    expect(window.addEventListener).toHaveBeenCalledWith('touchcancel', expect.any(Function));
    expect(window.addEventListener).toHaveBeenCalledWith('touchmove', expect.any(Function), {
      passive: false
    });
    window.addEventListener = addEventListener;
  });
  test('removeEventListeners removes mouse and touch event listeners', () => {
    const removeEventListener = window.removeEventListener;
    window.removeEventListener = jest.fn();
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
    wrapper.instance().removeEventListeners();
    expect(window.removeEventListener).toHaveBeenCalledWith('mousemove', expect.any(Function));
    expect(window.removeEventListener).toHaveBeenCalledWith('mouseup', expect.any(Function));
    expect(window.removeEventListener).toHaveBeenCalledWith('mouseleave', expect.any(Function));
    expect(window.removeEventListener).toHaveBeenCalledWith('touchend', expect.any(Function));
    expect(window.removeEventListener).toHaveBeenCalledWith('touchcancel', expect.any(Function));
    expect(window.removeEventListener).toHaveBeenCalledWith('touchmove', expect.any(Function), {
      passive: false
    });
    window.removeEventListener = removeEventListener;
  });
  describe('touch support', () => {
    test('onDownHandler reads the touch values rather than direct values', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      const mockEvent = {
        clientX: 50,
        clientY: 100,
        touches: [{
          clientX: 200,
          clientY: 150
        }]
      };
      wrapper.instance().onDownHandler(mockEvent);
      expect(wrapper.state('dragStartX')).toBe(200);
    });
    test('onMoveHandler sets isScrollingPast only if user scrolls down, not sideways', () => {
      const wrapper1 = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        dragThreshold: 0
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper1.instance().onMoveHandler({
        touches: [{
          clientX: 100,
          clientY: 0
        }],
        preventDefault: jest.fn()
      });
      expect(wrapper1.state('isScrollingPast')).toBeFalsy();
      const wrapper2 = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
        dragThreshold: 0
      }, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      wrapper2.instance().onMoveHandler({
        touches: [{
          clientX: 0,
          clientY: 100
        }],
        preventDefault: jest.fn()
      });
      expect(wrapper2.state('isScrollingPast')).toBeTruthy();
    });
  });
});
describe('CarouselItem', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, null));
    expect(wrapper.length).toBe(1);
  });
  test('renders a link if an href is passed', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      href: "http://google.com/"
    }, "Test"));
    expect(wrapper.type()).toBe(_link.default);
    expect(wrapper.prop('href')).toBe('http://google.com/');
  });
  test('renders a div if no href is passed', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, null, "Test"));
    expect(wrapper.type()).toBe('div');
  });
  test('onClick and openInNewTab passed through to child link', () => {
    const clickHandler = () => {};

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      openInNewTab: true,
      onClick: clickHandler,
      href: "http://google.com/"
    }, "Test"));
    expect(wrapper.prop('openInNewTab')).toBe(true);
    expect(wrapper.prop('onClick')).toBe(clickHandler);
  });
  test('onClick passed through to child div', () => {
    const clickHandler = () => {};

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
      onClick: clickHandler
    }, "Test"));
    expect(wrapper.prop('onClick')).toBe(clickHandler);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, null));
    expect(wrapper.hasClass('pw-carousel__item')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, null));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_carouselItem.default, {
        className: name
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  describe('moveTo', () => {
    test('calling moveTo with animate=false sets the correct indices', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      const instance = wrapper.instance();
      instance.moveTo(2, false);
      expect(wrapper.state('currentIndex')).toBe(2);
      expect(wrapper.state('nextIndex')).toBe(0);
      expect(wrapper.state('prevIndex')).toBe(1);
    });
    test('calling moveTo with a position outside of the carousel does nothing', () => {
      const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null), /*#__PURE__*/_react.default.createElement(_carouselItem.default, null)));
      const instance = wrapper.instance();
      instance.move = jest.fn();
      instance.moveTo(10);
      expect(wrapper.state('currentIndex')).toBe(0);
    });
  });
});