"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _icon = _interopRequireDefault(require("../icon"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Accessible feedback component that includes error and success styles
 *
 * @example ./DESIGN.md
 */
const Feedback = ({
  className,
  icon,
  text,
  title,
  isBlock,
  isError,
  isSuccess
}) => {
  const classes = (0, _classnames.default)('pw-feedback', {
    'pw--error': isError,
    'pw--success': isSuccess,
    'pw--block': isBlock
  }, className);
  let feedbackIcon;

  if (!icon) {
    // These are default icons for success and error if user doesn't provide any icon for them
    if (isSuccess) {
      feedbackIcon = 'check';
    } else if (isError) {
      feedbackIcon = 'warning';
    }
  } else {
    // Use user input icon
    feedbackIcon = icon;
  }

  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, feedbackIcon && /*#__PURE__*/_react.default.createElement(_icon.default, {
    className: "pw-feedback__icon",
    name: feedbackIcon,
    title: title
  }), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-feedback__content"
  }, text));
};

Feedback.propTypes = {
  /**
   * Text for the feedback.
   */
  text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]).isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Includes an icon of the given name in the feedback.
   * For more information about available icons, see the [Icon component](#!/Icon).
   */
  icon: _propTypes.default.string,

  /**
   * Defines if feedback is a block element.
   */
  isBlock: _propTypes.default.bool,

  /**
   * Defines if feedback is error.
   */
  isError: _propTypes.default.bool,

  /**
   * Defines if feedback is success.
   */
  isSuccess: _propTypes.default.bool,

  /**
   * The title to be used for accessibility.
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = Feedback;
exports.default = _default;