"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _icon = _interopRequireDefault(require("../icon/"));

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('IconLabel renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "person",
    label: "person"
  }));
  expect(wrapper.length).toBe(1);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "person",
    label: "person"
  }));
  expect(wrapper.children().hasClass('pw-icon-label')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "person",
    label: "person"
  }));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name,
      iconName: "person",
      label: "person"
    }));
    expect(wrapper.children().hasClass(name)).toBe(true);
  });
});
test('renders the icon and label with the correct values', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "tester",
    iconPrefix: "test",
    iconSize: "small",
    label: "Label"
  }));
  expect(wrapper.find(_icon.default).prop('name')).toBe('tester');
  expect(wrapper.find(_icon.default).prop('size')).toBe('small');
  expect(wrapper.find('.pw-icon-label__label').length).toBe(1);
  expect(wrapper.find('.pw-icon-label__label').text()).toBe('Label');
  expect(wrapper.find('use').html().indexOf('#test-') > 0).toBe(true);
});
test('renders IconLabel with button element', () => {
  const analyticsNameValue = 'analytics-name';
  const innerClassNameValue = 'inner-class-name';
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "tester",
    iconSize: "small",
    label: "Label",
    button: {
      analyticsName: analyticsNameValue,
      buttonInnerClassName: innerClassNameValue
    }
  }));
  expect(wrapper.find('Button').length).toBe(1);
  expect(wrapper.find('Button').prop('data-analytics-name')).toBe(analyticsNameValue);
  expect(wrapper.find('Button').prop('innerClassName')).toBe(`u-padding-0 ${innerClassNameValue}`);
});
test('If an onClick handler is passed, it is attached to the rendered component', () => {
  const onClick = jest.fn();
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    iconName: "tester",
    iconSize: "small",
    label: "Label",
    button: {
      onClick
    }
  }));
  expect(wrapper.children().is('Button')).toBe(true);
  expect(wrapper.children().prop('onClick')).toBe(onClick);
  expect(onClick).not.toBeCalled();
  wrapper.children().simulate('click');
  expect(onClick).toBeCalled();
});