"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _icon = _interopRequireDefault(require("../icon"));

var _button = _interopRequireDefault(require("../button"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * An icon and label combination. If no iconSize is passed in, the default will be medium
 *
 * @example ./DESIGN.md
 */
const IconLabel = ({
  label,
  iconName,
  iconPrefix,
  iconSize,
  button,
  className
}) => {
  const classes = (0, _classnames.default)('pw-icon-label', className);
  let iconLabel;

  if (button) {
    const buttonClasses = (0, _classnames.default)('pw--button', classes);
    const buttonInnerClasses = (0, _classnames.default)('u-padding-0', button.buttonInnerClassName);
    iconLabel = /*#__PURE__*/_react.default.createElement(_button.default, (0, _extends2.default)({
      onClick: button.onClick,
      className: buttonClasses,
      innerClassName: buttonInnerClasses,
      "data-analytics-name": button.analyticsName
    }, button.buttonProps), /*#__PURE__*/_react.default.createElement(IconLabelContent, {
      label: label,
      iconName: iconName,
      iconPrefix: iconPrefix,
      iconSize: iconSize
    }));
  } else {
    iconLabel = /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement(IconLabelContent, {
      label: label,
      iconName: iconName,
      iconPrefix: iconPrefix,
      iconSize: iconSize
    }));
  }

  return /*#__PURE__*/_react.default.createElement("div", null, iconLabel);
};

IconLabel.propTypes = {
  /**
   * The name of the icon to be used in the IconLabel.
   */
  iconName: _propTypes.default.string.isRequired,

  /**
   * The label text to be used in the IconLabel.
   */
  label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]).isRequired,

  /**
   * Render Button component
   */
  button: _propTypes.default.shape({
    /**
     * Analytics name for button event
     */
    analyticsName: _propTypes.default.string,

    /**
     * The properties for button
     */
    buttonProps: _propTypes.default.object,

    /**
     * Add classes to the inner div of button
     */
    buttonInnerClassName: _propTypes.default.string,

    /**
     * On click function for button
     */
    onClick: _propTypes.default.func
  }),

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * The icon prefix is the first part of the icon name/id. If you're using a
   * different icon set, the prefix may need to be modified.
   */
  iconPrefix: _propTypes.default.string,

  /**
   * Adds size of icon (normally, one of `small`, `medium`, or `large`).
   */
  iconSize: _propTypes.default.string
};
/**
 * Content in the IconLabel component
 */

const IconLabelContent = ({
  iconName,
  iconSize,
  label,
  iconPrefix
}) => {
  const iconProps = {
    name: iconName,
    size: iconSize
  };

  if (iconPrefix) {
    iconProps.prefix = iconPrefix;
  }

  return /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement(_icon.default, iconProps), /*#__PURE__*/_react.default.createElement("span", {
    className: "pw-icon-label__label"
  }, label));
};

IconLabelContent.propTypes = {
  iconName: _propTypes.default.string.isRequired,
  label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]).isRequired,

  /**
   * Adds a prefix to the icon name (defaults to 'pw').
   */
  iconPrefix: _propTypes.default.string,
  iconSize: _propTypes.default.string
};
IconLabelContent.defaultProps = {
  iconSize: 'medium'
};
var _default = IconLabel;
exports.default = _default;