"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.IconWithoutUniqueId = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _withUniqueId = _interopRequireDefault(require("../with-unique-id"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * `Icon` component renders an SVG that uses the chosen symbol from the SVG sprite present on the page.
 *
 * @example ./DESIGN.md
 */
class Icon extends _react.default.PureComponent {
  constructor(props) {
    super(props);
    this.id = `icon-${props.id}`;
  }

  render() {
    const {
      prefix,
      name,
      title,
      size,
      className,
      style
    } = this.props;
    const a11y = title.length ? {
      role: 'img'
    } : {
      'aria-hidden': 'true'
    };
    const sizeClass = `pw--${size}`;
    const classes = (0, _classnames.default)('pw-icon', {
      [sizeClass]: !!size
    }, className);
    return /*#__PURE__*/_react.default.createElement("svg", (0, _extends2.default)({}, a11y, {
      className: classes,
      title: title,
      "aria-labelledby": this.id,
      style: style
    }), /*#__PURE__*/_react.default.createElement("title", {
      id: this.id
    }, title), /*#__PURE__*/_react.default.createElement("use", {
      role: "presentation",
      xlinkHref: `#${prefix}-${name}`
    }));
  }

}

exports.IconWithoutUniqueId = Icon;
Icon.defaultProps = {
  name: '',
  title: '',
  prefix: 'pw',
  style: {}
};
Icon.propTypes = {
  /**
   * Identifier for the desired icon. Usually the filename, sans its prefix.
   * For example, if the target icon is `pw-chevron-up.svg`, the name
   * is `chevron-up`.
   */
  name: _propTypes.default.string.isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * ID for the component, `withUniqueId` will be responsible for generating
   * unique id to ensure that it's the same on the client and server-side.
   */
  id: _propTypes.default.string,

  /**
   * The prefix is the first part of the icon name/id. If you're using a
   * different icon set, the prefix may need to be modified.
   */
  prefix: _propTypes.default.string,

  /**
   * The size of the icon, controlled by CSS.
   */
  size: _propTypes.default.string,

  /**
   * Custom `style` attribute.
   */
  style: _propTypes.default.object,

  /**
   * **Beware** that ommitting a `title` will leave the icon **invisible** to
   * screen readers. If included, screen readers will read and treat the
   * icon like an image with alt text.
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};

var _default = (0, _withUniqueId.default)(Icon);

exports.default = _default;