"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * An inline loader component with accessible title for screen readers. Can customize
 * the number of loading dots and animation offset between dots.
 *
 * @example ./DESIGN.md
 */
const InlineLoader = ({
  animationDelayOffset,
  className,
  dotCount,
  title
}) => {
  const classes = (0, _classnames.default)('pw-inline-loader', className);
  let animationDelay = 0; // Generate loading dots

  const loadingDots = [];

  for (let i = 0; i < dotCount; i++) {
    loadingDots.push( /*#__PURE__*/_react.default.createElement("div", {
      key: i,
      className: `pw-inline-loader__loading-dot`,
      role: "presentation",
      style: {
        animationDelay: `${animationDelay}s`
      }
    })); // Calculate delay offset for next iteration

    animationDelay += animationDelayOffset;
  }

  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("span", {
    className: "u-visually-hidden"
  }, title), loadingDots);
};

InlineLoader.defaultProps = {
  animationDelayOffset: 0.2,
  dotCount: 3,
  title: 'Loading'
};
InlineLoader.propTypes = {
  /**
   * Amount of time before the next loading dot animates in. Time unit is in seconds.
   */
  animationDelayOffset: _propTypes.default.number,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * The amount of loading dots.
   */
  dotCount: _propTypes.default.number,

  /**
   * Screen reader text to indicate loading state.
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = InlineLoader;
exports.default = _default;