"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Lazy load image: content will render immediately
 * if its in view or will show when scrolled to it
 */
class LazyLoadContent extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      visible: false
    };
    this.handleScroll = (0, _lodash.default)(this.handleScroll.bind(this), this.props.scrollCheckInterval);
  }

  componentDidMount() {
    window.addEventListener('scroll', this.handleScroll);
    this.checkVisible();
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this.handleScroll);
  }

  checkVisible() {
    const topPosition = this.el.getBoundingClientRect().top; // Use pageYOffset here instead of scrollY because it has better browser compatibility (ex. IE11)

    if (topPosition <= window.innerHeight + window.pageYOffset - this.props.threshold) {
      this.setState({
        visible: true
      });
      window.removeEventListener('scroll', this.handleScroll);
    }
  }

  handleScroll() {
    if (!this.state.visible) {
      this.checkVisible();
    }
  }

  render() {
    const {
      className,
      children,
      placeholder
    } = this.props;
    const classes = (0, _classnames.default)('pw-lazy-load-content', className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      ref: el => {
        this.el = el;
      }
    }, this.state.visible ? children : placeholder);
  }

}

LazyLoadContent.defaultProps = {
  scrollCheckInterval: 200,
  threshold: 0
};
LazyLoadContent.propTypes = {
  /**
   * Content that will be revealed when scrolled to
   */
  children: _propTypes.default.node.isRequired,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * Placeholder content when actual content is not revealed
   */
  placeholder: _propTypes.default.node,

  /**
   * In milliseconds that delays invoking the function
   */
  scrollCheckInterval: _propTypes.default.number,

  /**
   * Number of pixels out the viewport before loading the content
   */
  threshold: _propTypes.default.number
};
var _default = LazyLoadContent;
exports.default = _default;