"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-env jest */
test('LazyLoadContent renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
  expect(wrapper.length).toBe(1);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
  expect(wrapper.children().hasClass('pw-lazy-load-content')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name
    }, "Test"));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
describe('Event listeners', () => {
  const addEventListener = window.addEventListener;
  const removeEventListener = window.removeEventListener;
  beforeEach(() => {
    window.addEventListener = jest.fn(addEventListener);
    window.removeEventListener = jest.fn(removeEventListener);
  });
  afterEach(() => {
    window.addEventListener = addEventListener;
    window.removeEventListener = removeEventListener;
  });
  test('registers a resize listener', () => {
    expect(window.addEventListener).not.toHaveBeenCalled();
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
    expect(window.addEventListener).toHaveBeenCalledWith('scroll', wrapper.instance().handleScroll);
  });
  test('removes the resize listener on unmount if not visible', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      threshold: 1000
    }, "test"));
    const handler = wrapper.instance().handleScroll;
    expect(window.removeEventListener).not.toHaveBeenCalledWith('scroll', handler);
    wrapper.unmount();
    expect(window.removeEventListener).toHaveBeenCalledWith('scroll', handler);
  });
  test('removes the resize listener immediately if visible', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
    const handler = wrapper.instance().handleScroll;
    expect(window.removeEventListener).toHaveBeenCalledWith('scroll', handler);
  });
});
describe('handleScroll', () => {
  test('calls checkVisible if not visible', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
    wrapper.setState({
      visible: false
    });
    wrapper.instance().checkVisible = jest.fn();
    wrapper.instance().handleScroll();
    expect(wrapper.instance().checkVisible).toBeCalled();
  });
  test('does nothing if visible', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "test"));
    wrapper.instance().checkVisible = jest.fn();
    expect(wrapper.state('visible')).toBeTruthy();
    wrapper.instance().handleScroll();
    expect(wrapper.instance().checkVisible).not.toBeCalled();
  });
});