"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related component:
 *
 * * [Ledger](#!/Ledger)
 *
 * This is used inside ledger component to display each row of the table.
 *
 * @example ./DESIGN.md
 */
const LedgerRow = ({
  className,
  isTotal,
  label,
  labelAction,
  labelDescription,
  value,
  valueAction
}) => {
  const classes = (0, _classnames.default)('pw-ledger__row', {
    'pw--total': isTotal
  }, className);
  return /*#__PURE__*/_react.default.createElement("tr", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("td", {
    className: "pw-ledger__item"
  }, label, (labelDescription || labelAction) && /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ledger__details"
  }, labelDescription, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ledger__action"
  }, labelAction))), /*#__PURE__*/_react.default.createElement("td", {
    className: "pw-ledger__value"
  }, value, valueAction && /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ledger__details"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ledger__action"
  }, valueAction))));
};

LedgerRow.displayName = 'LedgerRow';
LedgerRow.propTypes = {
  /**
   * Additional classes to add to the ledgerRow container.
   */
  className: _propTypes.default.string,

  /**
   * Indicates whether this ledger entry is the final total row.
   */
  isTotal: _propTypes.default.bool,

  /**
   * The label for the ledger entry.
   */
  label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Additional content that appears within the `pw-ledger__item` container.
   */
  labelAction: _propTypes.default.node,

  /**
   * Label description for ledger entry.
   */
  labelDescription: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The value for the ledger entry.
   */
  value: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Additional content that appears within the `pw-ledger__value` container.
   */
  valueAction: _propTypes.default.node
};
var _default = LedgerRow;
exports.default = _default;