"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _ledgerRow = _interopRequireDefault(require("./ledger-row"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related component:
 *
 * * [LedgerRow](#!/LedgerRow)
 *
 * A table/ledger component used to display order summaries.
 *
 * @example ./DESIGN.md
 */
const Ledger = ({
  children,
  className,
  rows
}) => {
  const classes = (0, _classnames.default)('pw-ledger', className);

  if (children) {
    return /*#__PURE__*/_react.default.createElement("table", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement("tbody", null, children));
  }

  return /*#__PURE__*/_react.default.createElement("table", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("tbody", null, rows.map((row, idx) => {
    return /*#__PURE__*/_react.default.createElement(_ledgerRow.default, (0, _extends2.default)({}, row, {
      key: idx
    }));
  })));
};

const ledgerRowPropType = (props, propName, componentName) => {
  let error;
  const prop = props[propName];

  _react.default.Children.forEach(prop, child => {
    if (!!child && child.type.displayName !== 'LedgerRow') {
      error = new Error(`${componentName} only accepts children of type "LedgerRow"`);
    }
  });

  return error;
};

Ledger.propTypes = {
  /**
   * Content for the ledger, must be of type LedgerRow.
   */
  children: ledgerRowPropType,

  /**
   * Additional classes to add to the ledger container.
   */
  className: _propTypes.default.string,

  /**
   * A collection of objects defining the content in each entry in the Ledger.
   * (See Details of rows prop below for more info)
   */
  rows: _propTypes.default.arrayOf(_propTypes.default.shape({
    isTotal: _propTypes.default.bool,
    label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),
    value: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
  }))
};
var _default = Ledger;
exports.default = _default;