"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = require("./index.js");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-disable newline-per-chained-call */
describe('LedgerRow', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement("table", null, /*#__PURE__*/_react.default.createElement("tbody", null, /*#__PURE__*/_react.default.createElement(_index.LedgerRow, null))));
    expect(wrapper.length).toBe(1);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, null));
    expect(wrapper.hasClass('pw-ledger__row')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, null));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
        className: name
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('sets the total class if isTotal is true', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      isTotal: true
    }));
    expect(wrapper.hasClass('pw--total')).toBe(true);
  });
  test('renders label, value, and no actions by default', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      label: "Subtotal",
      value: "15\u20AC20"
    }));
    expect(wrapper.find('.pw-ledger__action').length).toBe(0);
    expect(wrapper.find('.pw-ledger__item').text()).toBe('Subtotal');
    expect(wrapper.find('.pw-ledger__value').text()).toBe('15€20');
  });
  test('renders the labelAction if present', () => {
    const labelAction = /*#__PURE__*/_react.default.createElement("a", {
      href: "#test"
    }, "Test");

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      label: "LedgerRow",
      labelAction: labelAction
    }));
    expect(wrapper.find('.pw-ledger__item').childAt(0).text()).toBe('LedgerRow');
    expect(wrapper.find('.pw-ledger__item .pw-ledger__action').children().get(0)).toEqual(labelAction);
  });
  test('renders the valueAction if present', () => {
    const valueAction = /*#__PURE__*/_react.default.createElement("button", null, "X");

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      value: "\xA51700",
      valueAction: valueAction
    }));
    expect(wrapper.find('.pw-ledger__value').childAt(0).text()).toBe('¥1700');
    expect(wrapper.find('.pw-ledger__value .pw-ledger__action').children().get(0)).toEqual(valueAction);
  });
});
describe('Ledger', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Ledger, {
      rows: []
    }));
    expect(wrapper.length).toBe(1);
  });
  test('includes its children as the children of the Ledger', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Ledger, null, /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      className: "js-test"
    })));
    expect(wrapper.contains( /*#__PURE__*/_react.default.createElement(_index.LedgerRow, {
      className: "js-test"
    }))).toBe(true);
  });
  test('renders a row for each of the row props passed', () => {
    const rows = [{
      label: 'subtotal',
      value: '12.99'
    }, {
      label: 'tax',
      value: '0.00'
    }];
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Ledger, {
      rows: rows
    }));
    const ledgerRows = wrapper.find('.pw-ledger__row');
    expect(ledgerRows.length).toBe(rows.length);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Ledger, {
      rows: []
    }));
    expect(wrapper.hasClass('pw-ledger')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Ledger, {
      rows: []
    }));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Ledger, {
        className: name,
        rows: []
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('fails propType validation if non-LedgerRow children are passed', () => {
    expect(_index.Ledger.propTypes.children({
      children: /*#__PURE__*/_react.default.createElement("span", null, "Test")
    }, 'children', 'Ledger')).toEqual(new Error('Ledger only accepts children of type "LedgerRow"'));
  });
  test('No errors caused by propType validation if boolean children are passed', () => {
    expect(_index.Ledger.propTypes.children({
      children: false
    }, 'children', 'Ledger')).toEqual(undefined);
  });
});