"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _reactRouterDom = require("react-router-dom");

var _classnames = _interopRequireDefault(require("classnames"));

var _assets = require("../../utils/assets");

const _excluded = ["children", "className", "data", "href", "openInNewTab", "text", "onBlur", "onClick", "onFocus", "onMouseEnter", "onMouseLeave"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

/**
 * A link that converts to a react-router Link for paths within the React
 * site. It can take the link text/content either as child elements or as
 * a `text` prop.
 * If no href is passed, the link is rendered with a `#` href.
 * The list of routes from your project is automatically used to make decisions
 * whether the component renders a react-route link or an anchor tag'.
 */
const Link = _ref => {
  let {
    children,
    className,
    data,
    href,
    openInNewTab,
    text,
    onBlur,
    onClick,
    onFocus,
    onMouseEnter,
    onMouseLeave
  } = _ref,
      props = (0, _objectWithoutProperties2.default)(_ref, _excluded);
  let contents;

  if ((!children || !children.length) && text) {
    contents = text;
  } else {
    contents = children;
  }

  const attrs = _objectSpread(_objectSpread({
    className: (0, _classnames.default)('pw-link', className),
    onBlur,
    onClick,
    onFocus,
    onMouseEnter,
    onMouseLeave
  }, data), props);

  if (openInNewTab) {
    attrs.target = '_blank';
    attrs.rel = 'noopener';
  }

  if ((0, _assets.isInternalLink)(href)) {
    return /*#__PURE__*/_react.default.createElement(_reactRouterDom.Link, (0, _extends2.default)({
      to: href.replace(_assets.ORIGIN, '')
    }, attrs), contents);
  } else {
    return /*#__PURE__*/_react.default.createElement("a", (0, _extends2.default)({
      href: href
    }, attrs), contents);
  }
};

Link.defaultProps = {
  href: 'javascript:'
};
Link.propTypes = {
  /**
   * Any children for the Link component or anchor element.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * <p className="u-color-error">Deprecated</p> An object of data key/value pairs that are passed
   * through to the underlying DOM node.
   * Now that the Link component relays arbitrary props, this is unnecessary.
   */
  data: (props, propName, componentName) => {
    const dataPropObj = props[propName];

    if (dataPropObj) {
      const isDataPropValid = Object.keys(dataPropObj).every(dataProp => dataProp.startsWith('data'));

      if (!isDataPropValid) {
        return new Error('Invalid prop ' + propName + ' supplied to ' + componentName + '.' + // eslint-disable-line prefer-template
        " Make sure it's an object with keys that start with 'data-'." + ' Validation failed.');
      }
    }

    return null;
  },

  /**
   * The intended target URL.
   */
  href: _propTypes.default.string,

  /**
   * If true, target="_blank" will be added to the link.
   * Only use this property if you trust the link! https://mathiasbynens.github.io/rel-noopener
   */
  openInNewTab: _propTypes.default.bool,

  /**
   * The text of the link (only used if no children are passed)
   */
  text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * User-defined method for hooking into blur events. Use with
   * onMouseLeave as a keyboard substitute for hover events.
   */
  onBlur: _propTypes.default.func,

  /**
   * A callback to be called when the Link is clicked.
   */
  onClick: _propTypes.default.func,

  /**
   * User-defined method for hooking into focus events. Use with
   * onMouseEnter as a keyboard substitute for hover events.
   */
  onFocus: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseEnter events. Use with the onFocus prop for keyboard a11y purposes.
   */
  onMouseEnter: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseLeave events. Use with the onBlur prop for keyboard a11y purposes.
   */
  onMouseLeave: _propTypes.default.func
};
Link.displayName = 'Link';
var _default = Link;
exports.default = _default;