"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _classnames = _interopRequireDefault(require("classnames"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _listTilePrimary = _interopRequireDefault(require("./partials/list-tile-primary"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * The `ListTile` component is a container to display text or/and links.
 * ListTile can have actions that can be displayed before or/and after the text.
 * Commonly used in the `List` component
 *
 * @example ./DESIGN.md
 */
class ListTile extends _react.default.PureComponent {
  render() {
    const {
      className,
      startAction,
      endAction,
      children,
      href,
      includeEndActionInPrimary,
      tabIndex,
      onBlur,
      onClick,
      onFocus,
      onMouseEnter,
      onMouseLeave
    } = this.props;
    const classes = (0, _classnames.default)('pw-list-tile', {
      'pw--is-anchor': !!href
    }, className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement(_listTilePrimary.default, {
      href: href,
      onBlur: onBlur,
      onClick: onClick,
      onFocus: onFocus,
      onMouseEnter: onMouseEnter,
      onMouseLeave: onMouseLeave,
      tabIndex: tabIndex
    }, startAction && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-list-tile__action"
    }, startAction), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-list-tile__content"
    }, children), includeEndActionInPrimary && endAction && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-list-tile__action"
    }, endAction)), !includeEndActionInPrimary && endAction && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-list-tile__action"
    }, endAction));
  }

}

ListTile.defaultProps = {
  includeEndActionInPrimary: true
};
ListTile.propTypes = {
  /**
   * The content that should be rendered within the ListTile primary
   * section, excluding the action.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * The content that appears at the end of the list tile.
   * Can be a supplementary action
   */
  endAction: _propTypes.default.node,

  /**
   * If provided, the primary part of the ListTile will be rendered
   * as a Link to this URL.
   */
  href: _propTypes.default.string,

  /**
   * Indicates if the endAction should be included inside the primary part.
   * If true, clicking endAction will perform the same action as the primary content
   */
  includeEndActionInPrimary: _propTypes.default.bool,

  /**
   * The content that appears at the start of the list tile. Generally supplementary icon or text
   */
  startAction: _propTypes.default.node,

  /**
   * PROVIDED INTERNALLY: The tabIndex value for this component
   */
  tabIndex: _propTypes.default.oneOfType([_propTypes.default.number, _propTypes.default.string]),

  /**
   * User-defined method for hooking into blur events. Use with
   * onMouseLeave as a keyboard substitute for hover events.
   */
  onBlur: _propTypes.default.func,

  /**
   * Callback for when the primary is clicked. Not called if an href is passed.
   */
  onClick: _propTypes.default.func,

  /**
   * User-defined method for hooking into focus events. Use with
   * onMouseEnter as a keyboard substitute for hover events.
   */
  onFocus: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseEnter events. Use with the onFocus prop for keyboard a11y purposes.
   */
  onMouseEnter: _propTypes.default.func,

  /**
   * User-defined method for hooking into mouseLeave events. Use with the onBlur prop for keyboard a11y purposes.
   */
  onMouseLeave: _propTypes.default.func
};
var _default = ListTile;
exports.default = _default;