"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _testing = require("../../utils/testing");

var _link = _interopRequireDefault(require("../link"));

var _index = _interopRequireDefault(require("./index"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-disable newline-per-chained-call */
test('ListTile renders without errors', () => {
  const wrapper = (0, _testing.mountWithRouter)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.length).toBe(1);
});
test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('pw-list-tile')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name
    }));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
test('renders a Link as primary if an href is present', () => {
  const wrapper = (0, _testing.mountWithRouter)( /*#__PURE__*/_react.default.createElement(_index.default, {
    href: "/mens-handbags"
  }));
  expect(wrapper.find(_link.default).length).toBe(1);
  expect(wrapper.find(_link.default).find('a').hasClass('pw-list-tile__primary')).toBe(true);
  expect(wrapper.find(_link.default).prop('href')).toBe('/mens-handbags');
});
test('does not render a link as primary if no href is present', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  const primary = wrapper.find('.pw-list-tile__primary');
  expect(primary.length).toBe(1);
  expect(primary.type()).toBe('div');
  expect(primary.prop('role')).toBe('button');
});
test('renders the StartAction in the primary container', () => {
  const startAction = /*#__PURE__*/_react.default.createElement("div", {
    className: "test"
  }, "Start Action");

  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    startAction: startAction
  }));
  const start = wrapper.find('.pw-list-tile__primary .test');
  expect(start.length).toBe(1);
  expect(start.text()).toBe('Start Action');
});
test('includeEndActionInPrimary={false} moves endAction to the primary container', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    includeEndActionInPrimary: false,
    endAction: /*#__PURE__*/_react.default.createElement("div", {
      className: "end-action"
    }, "End Action Test")
  }));
  expect(wrapper.find('.pw-list-tile__primary .end-action').length).toBe(0);
});
test('endAction is in the primary container if includeEndActionInPrimary={false} is not set', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    endAction: /*#__PURE__*/_react.default.createElement("div", {
      className: "end-action"
    }, "End Action Test")
  }));
  expect(wrapper.find('.pw-list-tile__primary .end-action').length).toBe(1);
});
test('onMouseEnter fires on hover', () => {
  const mockOnMouseEnter = jest.fn();
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    onMouseEnter: mockOnMouseEnter
  }));
  wrapper.find('.pw-list-tile__primary').simulate('mouseEnter');
  expect(mockOnMouseEnter.mock.calls.length).toBe(1);
});