"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _link = _interopRequireDefault(require("../../link"));

var _a11y = require("../../../utils/a11y");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const ListTilePrimary = ({
  children,
  href,
  tabIndex,
  onBlur,
  onClick,
  onFocus,
  onMouseEnter,
  onMouseLeave
}) => {
  const listTileProps = {
    className: 'pw-list-tile__primary',
    tabIndex
  };
  const actionProps = {
    onBlur,
    onClick,
    onFocus,
    onMouseEnter,
    onMouseLeave
  };
  Object.keys(actionProps).forEach(action => {
    if (actionProps[action] !== undefined) {
      listTileProps[action] = actionProps[action];
    }
  });

  if (href) {
    return /*#__PURE__*/_react.default.createElement(_link.default, (0, _extends2.default)({
      href: href
    }, listTileProps), children);
  } else {
    // Disabling the jsx-a11y/no-static-element-interactions rule because
    // there are some cases where we want the this container to behave as a
    // button, but it also contains a button. Nesting buttons inside buttons
    // in not valid markup.
    //
    // @url: https://developer.mozilla.org/en/docs/Web/HTML/Element/button

    /* eslint-disable jsx-a11y/no-static-element-interactions */
    return /*#__PURE__*/_react.default.createElement("div", (0, _extends2.default)({
      role: "button",
      onKeyUp: (0, _a11y.onKeyUpWrapper)(onClick)
    }, listTileProps), children);
    /* eslint-enable jsx-a11y/no-static-element-interactions */
  }
};

ListTilePrimary.defaultProps = {
  tabIndex: 0
};
ListTilePrimary.propTypes = {
  /**
   * PROVIDED INTERNALLY: the contents of the primary part of the ListTile
   */
  children: _propTypes.default.node,

  /**
   * PROVIDED INTERNALLY: The URL to link to from the primary part
   */
  href: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY: The tabIndex value for this component
   */
  tabIndex: _propTypes.default.oneOfType([_propTypes.default.number, _propTypes.default.string]),

  /**
   * PROVIDED INTERNALLY: User-defined method for hooking into onBlur events. Use with
   * onMouseLeave as a keyboard substitute for hover events.
   */
  onBlur: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY: A callback for when the primary part is clicked.
   */
  onClick: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY: A user-defined method for hooking into focus events.
   * Use with onMouseEnter as a keyboard substitute for hover events.
   */
  onFocus: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY: A user-defined method for hooking into onMouseEnter events.
   * Use with the onFocus prop for keyboard a11y purposes.
   */
  onMouseEnter: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY: A user-defined method for hooking into onMouseLeave events.
   * Use with the onBlur prop for keyboard a11y purposes.
   */
  onMouseLeave: _propTypes.default.func
};
var _default = ListTilePrimary;
exports.default = _default;