"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index.jsx"));

var _index2 = _interopRequireDefault(require("../list-tile/index.jsx"));

var _button = _interopRequireDefault(require("../button"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('List renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: []
  }));
  expect(wrapper.length).toBe(1);
});
test('List renders its children', () => {
  const children = [/*#__PURE__*/_react.default.createElement("p", {
    key: "1"
  }, "Item 1"), /*#__PURE__*/_react.default.createElement("p", {
    key: "2"
  }, "Item 2")];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, children));
  const listChildren = wrapper.find('p');
  expect(listChildren.length).toBe(children.length);
});
test('List renders its children if provided, and then not any items', () => {
  const children = [/*#__PURE__*/_react.default.createElement("p", {
    key: "1"
  }, "Item 1"), /*#__PURE__*/_react.default.createElement("p", {
    key: "2"
  }, "Item 2")];
  const items = [{
    title: 'test'
  }, {
    title: 'test2'
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: items
  }, children));
  const paragraphElements = wrapper.find('p');
  const listElements = wrapper.find(_index2.default);
  expect(paragraphElements.length).toBe(children.length);
  expect(listElements.length).toBe(0);
});
test('List renders a child for each item in the items prop array', () => {
  const items = [{
    title: 'test'
  }, {
    title: 'test2'
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: items
  }));
  const listElements = wrapper.find(_index2.default);
  expect(listElements.length).toBe(items.length);
});
test('List renders its items list as designated by the component prop', () => {
  const items = [{
    title: 'test'
  }, {
    title: 'test2'
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: items,
    component: _button.default
  }));
  const buttonElements = wrapper.find(_button.default);
  expect(buttonElements.length).toBe(items.length);
});
test("List renders its items as designated by that item's own component prop", () => {
  const items = [{
    title: 'test',
    component: _button.default
  }, {
    title: 'test2',
    component: _index2.default
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: items,
    component: _button.default
  }));
  const buttonElements = wrapper.find(_button.default);
  const listElements = wrapper.find(_index2.default);
  expect(buttonElements.length).toBe(1);
  expect(listElements.length).toBe(1);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: []
  }));
  expect(wrapper.hasClass('pw-list')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: []
  }));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      items: [],
      className: name
    }));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
test('the clickHandler is called when an item is clicked, with its index', () => {
  const onClick = jest.fn();
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    items: [{
      title: 'test'
    }],
    clickHandler: onClick
  }));
  wrapper.find('div.pw-list-tile__primary').prop('onClick')();
  expect(onClick).toBeCalledWith(0);
});