"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _browserDetection = require("../../utils/browser-detection");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
// Determine if we are in mobile safari, we need this to apply a scroll fix.
const isIOS = () => {
  const currentBrowser = (0, _browserDetection.detectBrowser)(navigator);
  return currentBrowser.mobile && currentBrowser.name === 'safari';
};
/**
 * `Lockup` is used to stop the user from being able to scroll the page.
 * Normally, it should wrap around your main app container.
 * To lock the page, set the `locked` prop to true.
 * To unlock the page, set `locked` to false.
 *
 * @example ./DESIGN.md
 */


class Lockup extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {};
    this.lock = this.lock.bind(this);
    this.body = null;
  }

  componentDidMount() {
    this.isIOS = isIOS();
    this.body = document.body;

    if (this.props.locked) {
      this.lock();
    }
  }

  componentDidUpdate(prevProps) {
    if (this.props.locked && !prevProps.locked) {
      this.lock();
    } else if (!this.props.locked && prevProps.locked) {
      this.unlock();
    }
  }

  lock() {
    const scrollPosition = window.scrollY;
    let lockStyles;

    if (this.isIOS) {
      // On iOS, we lock the height of the element's body wrapping div as well
      // as do some scrolling magic to make sure forms don't jump the page
      // around when they're focused.
      this.body.style.marginTop = '0';
      this.body.style.marginBottom = '0';
      lockStyles = {
        position: 'relative',
        height: `${window.innerHeight}px`
      };
      this.container.scrollTop = scrollPosition;
    } else {
      // On Chrome, we can get away with fixing the position of the html
      // and moving it up to the equivalent of the scroll position
      // to lock scrolling.
      lockStyles = {
        position: 'fixed',
        top: scrollPosition * -1
      };
    }

    this.setState({
      scrollPosition,
      style: lockStyles
    });
  }

  unlock() {
    if (this.isIOS) {
      this.body.style.marginTop = '';
      this.body.style.marginBottom = '';
    }

    this.setState({
      style: {}
    }, () => {
      window.scrollTo(0, this.state.scrollPosition);
    });
  }

  render() {
    const {
      children,
      className,
      locked
    } = this.props;
    const {
      style
    } = this.state;
    const classes = (0, _classnames.default)('pw-lockup', {
      'pw--is-locked': locked
    }, className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      style: style,
      ref: el => {
        this.container = el;
      }
    }, children);
  }

}

Lockup.propTypes = {
  /**
   * The children to be locked. Usually, you'll wrap Lockup around your app container
   */
  children: _propTypes.default.node.isRequired,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * Indicates whether the children should be locked
   */
  locked: _propTypes.default.bool
};
var _default = Lockup;
exports.default = _default;