"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('Lockup renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  expect(wrapper.length).toBe(1);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  expect(wrapper.hasClass('pw-lockup')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name
    }, "Test"));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
test('renders unlocked by default', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  expect(wrapper.hasClass('pw--is-locked')).toBe(false);
});
test('renders locked with a locked prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    locked: true
  }, "Test"));
  expect(wrapper.hasClass('pw--is-locked')).toBe(true);
});
test('adds the styles in the state to the container div', () => {
  const testStyles = {
    height: '10px',
    overflow: 'hidden'
  };
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  wrapper.setState({
    style: testStyles
  });
  expect(wrapper.prop('style')).toEqual(testStyles);
});
test('locks when the locked prop is updated to true', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
  expect(wrapper.find('.pw-lockup').hasClass('pw--is-locked')).toBe(false);
  expect(wrapper.state('style')).toBe(undefined);
  wrapper.setProps({
    locked: true
  });
  expect(wrapper.find('.pw-lockup').hasClass('pw--is-locked')).toBe(true);
  expect(wrapper.state('style').position).toBe('fixed');
  expect(wrapper.state('style').top).toBe(window.scrollY * -1);
});
test('unlocks when the locked prop is updated to false', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    locked: true
  }, "Test"));
  expect(wrapper.find('.pw-lockup').hasClass('pw--is-locked')).toBe(true);
  expect(wrapper.state('style').position).toBe('fixed');
  expect(wrapper.state('style').top).toBe(window.scrollY * -1);
  wrapper.setProps({
    locked: false
  });
  expect(wrapper.find('.pw-lockup').hasClass('pw--is-locked')).toBe(false);
  expect(wrapper.state('style').position).toBe(undefined);
  expect(wrapper.state('style').top).toBe(undefined);
});
describe('When running on iOS', () => {
  let wrapper;
  let instance;
  beforeEach(() => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, "Test"));
    instance = wrapper.instance();
    instance.isIOS = true; // Reset document styles

    document.body.style.removeProperty('marginTop');
    document.body.style.removeProperty('marginBottom');
  });
  test('locks differently', () => {
    wrapper.setProps({
      locked: true
    });
    expect(wrapper.find('.pw-lockup').hasClass('pw--is-locked')).toBe(true);
    expect(wrapper.state('style').position).toBe('relative');
    expect(instance.container.scrollTop).toBe(window.scrollY);
  });
  test("resets body's vertical margin when unlocked", () => {
    wrapper.setProps({
      locked: true
    });
    wrapper.setProps({
      locked: false
    });
    expect(instance.body.style.marginTop).toBe('0px');
    expect(instance.body.style.marginBottom).toBe('0px');
  });
});