"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _TransitionGroup = _interopRequireDefault(require("react-transition-group/TransitionGroup"));

var _classnames = _interopRequireDefault(require("classnames"));

var _navSlider = _interopRequireDefault(require("../nav-slider"));

var _nav = _interopRequireDefault(require("../nav"));

var _a11y = require("../../utils/a11y");

var _navHeaderDefaultContent = require("./nav-header-default-content");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * Related components:
 *
 * * [Nav](#!/Nav)
 * * [NavHeader](#!/NavHeader)
 * * [NavItem](#!/NavItem)
 * * [NavMenu](#!/NavMenu)
 * * [NavSlider](#!/NavSlider)
 *
 * The `NavHeader` component is responsible for rendering a "header bar" inside a
 * `Nav` component, automatically displaying meta information about the
 * navigation tree such as which navigation node is presently active, as well
 * as displaying UI that can be, for example, used to navigate the tree
 * backwards, or to display buttons that can be used to close a navigation
 * modal.
 *
 * `NavHeader` cannot be used by itself. It must be inserted into a `Nav`
 * component.
 *
 * @example ./DESIGN.md
 */


const NavHeader = (props, context) => {
  const {
    goBack,
    expanded,
    expandedPath
  } = context;
  const {
    startContent,
    endContent,
    onClose,
    className,
    animationProperties,
    onTitleClick
  } = props;
  const classes = (0, _classnames.default)('pw-nav-header', className);
  const sliderClasses = (0, _classnames.default)('pw-nav-header__slider', {
    'pw--has-custom-header-animation': Object.keys(animationProperties).length > 0,
    [animationProperties.className]: animationProperties.className
  });
  const atRoot = expandedPath === '/';

  const onBackClick = e => {
    e.preventDefault();
    goBack();
  };

  const onCloseClick = e => {
    e.preventDefault();
    onClose();
  };

  const start = /*#__PURE__*/_react.default.isValidElement(startContent) ? /*#__PURE__*/_react.default.cloneElement(startContent, {
    atRoot,
    onClick: onBackClick
  }) : startContent;
  const end = /*#__PURE__*/_react.default.isValidElement(endContent) ? /*#__PURE__*/_react.default.cloneElement(endContent, {
    onClick: onCloseClick
  }) : endContent;
  let titleProps = {
    className: 'pw-nav-header__title'
  };

  if (onTitleClick) {
    const wrapTitleClick = () => onTitleClick(expanded.title, expanded.path);

    titleProps = (0, _extends2.default)(titleProps, {
      onClick: wrapTitleClick,
      onKeyUp: (0, _a11y.onKeyUpWrapper)(wrapTitleClick),
      role: 'button',
      tabIndex: '0'
    });
  }

  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-nav-header__actions-start"
  }, start), /*#__PURE__*/_react.default.createElement(_TransitionGroup.default, {
    component: "div",
    className: "pw-nav-header__slider-container"
  }, /*#__PURE__*/_react.default.createElement(_navSlider.default, (0, _extends2.default)({}, animationProperties, {
    className: sliderClasses,
    key: expandedPath
  }), /*#__PURE__*/_react.default.createElement("span", titleProps, expanded && expanded.title || ''))), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-nav-header__actions-end"
  }, end));
};

NavHeader.defaultProps = {
  onClose: noop,
  startContent: /*#__PURE__*/_react.default.createElement(_navHeaderDefaultContent.DefaultStartContent, null),
  animationProperties: {},
  endContent: /*#__PURE__*/_react.default.createElement(_navHeaderDefaultContent.DefaultEndContent, null)
};
NavHeader.propTypes = {
  /**
   * Animation properties for customizing the NavHeader's animation.
   * These properties are passed down to the internal NavSlider
   * component as props.
   */
  animationProperties: _propTypes.default.shape({
    /**
     * Adds values to the `class` attribute to the
     * internal NavSlider.
     */
    className: _propTypes.default.string,

    /**
     * Duration of the animation in millis.
     */
    duration: _propTypes.default.number,

    /**
     * Easing function for the animation.
     */
    easing: _propTypes.default.string,

    /**
     * Id given to the internal NavSlider.
     */
    id: _propTypes.default.string
  }),

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Overrides the content displayed at the end of the header.
   * Defaults to a 'close' button.
   */
  endContent: _propTypes.default.oneOfType([_propTypes.default.element, _propTypes.default.string]),

  /**
   * Overrides the content displayed at the start of the header.
   * Defaults to a 'back' button.
   */
  startContent: _propTypes.default.oneOfType([_propTypes.default.element, _propTypes.default.string]),

  /**
   * A callback invoked when the default close button is pressed.
   */
  onClose: _propTypes.default.func,

  /**
   * A callback invoked when the title is clicked.
   */
  onTitleClick: _propTypes.default.func
};
NavHeader.contextTypes = _nav.default.childContextTypes;
var _default = NavHeader;
exports.default = _default;