"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _listTile = _interopRequireDefault(require("../list-tile"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related components:
 *
 * * [Nav](#!/Nav)
 * * [NavHeader](#!/NavHeader)
 * * [NavItem](#!/NavItem)
 * * [NavMenu](#!/NavMenu)
 * * [NavSlider](#!/NavSlider)
 *
 * By itself, the `NavItem` renders a simple block link that is meant for use
 * in other components. In fact, it is the default component used when the
 * `NavMenu` component renders its list of navigation items.
 *
 * @example ./DESIGN.md
 */
const NavItem = props => {
  const {
    navigate,
    selected,
    title,
    className,
    childIcon,
    beforeContent: beforeContentProp,
    content: contentProp,
    hasChild,
    href
  } = props;

  const onClick = e => {
    e.preventDefault();
    navigate();
  };

  const before = beforeContentProp;
  const content = contentProp || title;
  const after = hasChild ? childIcon : null;
  const classes = (0, _classnames.default)('pw-nav-item', {
    'pw--has-child': hasChild,
    'pw--selected': selected
  }, className);
  return /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement(_listTile.default, {
    className: classes,
    onClick: onClick,
    href: href,
    startAction: before,
    endAction: after,
    includeEndActionInPrimary: true
  }, content));
};

NavItem.defaultProps = {
  navigate:
  /* istanbul ignore next */
  () => null,
  childIcon: '>'
};
NavItem.propTypes = {
  /**
   * Content to go before the main label on the nav item.
   */
  beforeContent: _propTypes.default.node,

  /**
   * Override the default icon for the has-child indicator.
   */
  childIcon: _propTypes.default.node,

  /**
   * Extra classes for the element.
   */
  className: _propTypes.default.string,

  /**
   * Override the main label content on the nav item (default is its title).
   */
  content: _propTypes.default.node,

  /**
   * Indicates whether the item has children.
   */
  hasChild: _propTypes.default.bool,

  /**
   * If provided, element will render as a link.
   */
  href: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY. A callback that is intended to invoke a parent `Nav`
   * component's `onPathChange` (user defined) callback. It can be
   * thought of as an event emitter listened to by the `Nav` component.
   *
   * By itself, `navigate()` does nothing. It is up to the user to
   * define and pass in an `onPathChange` callback to `Nav` that determines
   * what happens when a `MegaMenuItem` is clicked.
   */
  navigate: _propTypes.default.func,

  /**
   * PROVIDED INTERNALLY. Indicates whether the item is currently selected.
   */
  selected: _propTypes.default.bool,

  /**
   * The title of the navigation item.
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = NavItem;
exports.default = _default;