"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-disable newline-per-chained-call */
test('NavItem renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.length).toBe(1);
});
test('it renders a ListTile component', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    content: "title"
  }));
  expect(wrapper.find('ListTile').length).toBe(1);
  expect(wrapper.childAt(0).text()).toBe('title');
});
test('renders extra content correctly', () => {
  const overrideContent = /*#__PURE__*/_react.default.createElement("span", null, "override content");

  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    beforeContent: "before",
    content: overrideContent
  }));
  const primaryWrapper = wrapper.find('.pw-list-tile__primary'); // Check the before content

  expect(primaryWrapper.childAt(0).type()).toBe('div');
  expect(primaryWrapper.childAt(0).text()).toBe('before'); // Check the main content

  expect(primaryWrapper.find('.pw-list-tile__content').children().length).toBe(1);
  expect(primaryWrapper.find('.pw-list-tile__content').childAt(0).type()).toBe('span');
  expect(primaryWrapper.find('.pw-list-tile__content').childAt(0).text()).toBe('override content');
});
test('renders a childIcon if hasChild is true', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    hasChild: true,
    childIcon: ">>"
  }));
  expect(wrapper.find('ListTile').hasClass('pw--has-child')).toBe(true);
  expect(wrapper.find('ListTile').prop('endAction')).toBe('>>');
});
test('does not render a childIcon if hasChild is false', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    hasChild: false,
    childIcon: ">>"
  }));
  const primaryWrapper = wrapper.find('.pw-list-tile__primary');
  expect(wrapper.find('.pw-list-tile').hasClass('pw--has-child')).toBe(false);
  expect(primaryWrapper.children().length).toBe(1);
  expect(primaryWrapper.childAt(0).hasClass('pw-list-tile__content')).toBe(true);
});
test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.find('ListTile').hasClass('pw-nav-item')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  const name = 'name';
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    className: name
  }));
  expect(wrapper.find('ListTile').hasClass(name)).toBe(true);
});
test('invokes the navigate callback on click by default', () => {
  const spy = jest.fn();
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    content: "title",
    navigate: spy
  }));
  wrapper.find('ListTile').simulate('click', new MouseEvent('click'));
  expect(spy.mock.calls.length).toBe(1);
});