"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _navSlider = _interopRequireDefault(require("../nav-slider"));

var _nav = _interopRequireDefault(require("../nav"));

var _navItem = _interopRequireDefault(require("../nav-item"));

var _nonExpandedItems = _interopRequireDefault(require("./non-expanded-items"));

var _TransitionGroup = _interopRequireDefault(require("react-transition-group/TransitionGroup"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related components:
 *
 * * [Nav](#!/Nav)
 * * [NavHeader](#!/NavHeader)
 * * [NavItem](#!/NavItem)
 * * [NavMenu](#!/NavMenu)
 * * [NavSlider](#!/NavSlider)
 *
 * `NavMenu` is responsible for rendering the pages of `NavItem`s as a user
 * traverses the navigation tree (see the [`Nav` component's](#!/Nav) `root`
 * prop).
 *
 * For example, the `NavMenu` could render all of the root level
 * items from which the user could click through into a second level of items,
 * at which point the `NavMenu` would animated out the previous level then
 * animate in the next level.
 *
 * `NavMenu` cannot be used by itself. It must be inserted into a `Nav`
 * component.
 *
 * @example ./DESIGN.md
 */
class NavMenu extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      rerenderNonExpandedItems: false
    };
  }

  render() {
    const {
      className,
      itemFactory,
      animationProperties,
      getRealPath,
      prerender
    } = this.props;
    const {
      selected,
      expanded,
      expandedPath,
      goToPath,
      root
    } = this.context;
    const classes = (0, _classnames.default)('pw-nav-menu', className);
    const sliderClasses = (0, _classnames.default)('pw-nav-menu__slider', {
      'pw--has-custom-menu-animation': Object.keys(animationProperties).length > 0,
      [animationProperties.className]: animationProperties.className
    });
    return /*#__PURE__*/_react.default.createElement(_TransitionGroup.default, {
      component: "div",
      className: classes
    }, /*#__PURE__*/_react.default.createElement(_navSlider.default, (0, _extends2.default)({}, animationProperties, {
      className: sliderClasses,
      key: expandedPath,
      onEnterComplete: () => {
        requestAnimationFrame(() => {
          this.setState({
            rerenderNonExpandedItems: true
          });
        });
      }
    }), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-nav-menu__panel"
    }, (expanded.children || []).map((child, idx) => {
      const props = {
        key: idx,
        navigate: goToPath.bind(undefined, child.path),
        selected: child === selected,
        hasChild: (child.children || []).length > 0,
        title: child.title,
        path: child.path,
        options: child.options
      };
      return itemFactory(child.type, props);
    }))), prerender && /*#__PURE__*/_react.default.createElement(_nonExpandedItems.default, {
      allowRerender: this.state.rerenderNonExpandedItems,
      root: root,
      expanded: expanded,
      getRealPath: getRealPath,
      onRenderComplete: () => {
        this.setState({
          rerenderNonExpandedItems: false
        });
      }
    }));
  }

}

const defaultItemFactory = (type, props) => {
  return /*#__PURE__*/_react.default.createElement(_navItem.default, props);
};

NavMenu.contextTypes = _nav.default.childContextTypes;
NavMenu.defaultProps = {
  itemFactory: defaultItemFactory,
  animationProperties: {},
  getRealPath: path => path,
  prerender: true
};
NavMenu.propTypes = {
  /**
   * Animation properties for customizing the NavMenu's animation.
   * These properties are passed down to the internal NavSlider
   * component as props.
   */
  animationProperties: _propTypes.default.shape({
    /**
     * Adds values to the `class` attribute to the
     * internal NavSlider.
     */
    className: _propTypes.default.string,

    /**
     * Duration of the animation in milliseconds.
     */
    duration: _propTypes.default.number,

    /**
     * Easing function for the animation.
     */
    easing: _propTypes.default.string,

    /**
     * Id given to the internal NavSlider.
     */
    id: _propTypes.default.string
  }),

  /**
   * Extra classes for the element.
   */
  className: _propTypes.default.string,

  /**
   * This function will be passed the path of an item in the navigation
   * It should return the appropriate URL for that path
   * This was included to work around the issue where the Navigation requires unique paths
   * And uses the number of /s in the path to infer which direction to slide when animating
   */
  getRealPath: _propTypes.default.func,

  /**
   * Factory function to render menu items for display.
   */
  itemFactory: _propTypes.default.func,

  /**
   * Determines if the NavMenu should also render parts of the nav
   * that aren't currently expanded
   * This can be helpful for SEO, as it allows all links
   * in the Navigation to be present in the DOM
   */
  prerender: _propTypes.default.bool
};
var _default = NavMenu;
exports.default = _default;