"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _nav = _interopRequireDefault(require("../nav"));

var _static = _interopRequireDefault(require("inline-style-prefixer/static"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * Related components:
 *
 * * [Nav](#!/Nav)
 * * [NavHeader](#!/NavHeader)
 * * [NavItem](#!/NavItem)
 * * [NavMenu](#!/NavMenu)
 * * [NavSlider](#!/NavSlider)
 *
 * `NavSlider` is a helper component for customizing the slide-in and slide-out
 * animations used when traversing items in the [`Nav`](#!/Nav) component.
 * In particular, `NavSlider` can be used to create fully custom versions of the
 * [`NavHeader`](#!/NavHeader) and [`NavMenu`](#!/NavMenu) components.
 *
 * To take full advantage of the `NavSlider`, it should be used with the
 * [`TransitionGroup`](https://github.com/reactjs/react-transition-group)
 * component. See examples of this below.
 */


class NavSlider extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      style: {}
    };
  }

  componentWillEnter(callback) {
    const {
      action
    } = this.context;
    const [start, end] = action === 'descending' ? ['100%', '0'] : ['-100%', '0'];
    this.animate(start, end, this.props.onEnterComplete).then(callback);
  }

  componentWillLeave(callback) {
    const {
      action
    } = this.context;
    const [start, end] = action === 'descending' ? ['0', '-100%'] : ['0', '100%'];
    this.animate(start, end, this.props.onLeaveComplete).then(callback);
  }

  animate(start, end, callback) {
    const {
      duration,
      easing
    } = this.props;

    const frame = () => new Promise(resolve => {
      requestAnimationFrame(resolve);
    });

    const setStartState = () => new Promise(resolve => {
      const state = {
        style: (0, _static.default)({
          transform: `translate(${start}, 0)`,
          transition: `none`,
          position: 'absolute',
          width: '100%'
        }),
        isSliding: true
      };
      this.setState(state, resolve);
    });

    const setEndState = () => new Promise(resolve => {
      const state = {
        style: (0, _static.default)({
          transform: `translate(${end}, 0)`,
          transition: `transform ${duration / 1000}s ${easing}`,
          position: 'absolute',
          width: '100%'
        })
      };
      this.setState(state, resolve);
    });

    const waitForAnimation = () => new Promise(resolve => {
      setTimeout(() => {
        callback();
        resolve();
      }, duration);
    });

    const cleanup = () => {
      this.setState({
        isSliding: false,
        style: {}
      });
    };

    return frame().then(setStartState).then(frame).then(setEndState).then(waitForAnimation).then(cleanup);
  }

  render() {
    const {
      id,
      children,
      className
    } = this.props;
    const {
      style,
      isSliding
    } = this.state;
    const classes = (0, _classnames.default)('pw-nav-slider', {
      'pw--sliding': isSliding
    }, className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      id: id,
      style: style
    }, children);
  }

}

NavSlider.propTypes = {
  /**
   * Child nodes of the `NavSlider`.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Duration of the animation in milliseconds.
   */
  duration: _propTypes.default.number,

  /**
   * Easing function for the animation as a string whose value
   * can be anything the [CSS `transition-timing-function`](https://developer.mozilla.org/en/docs/Web/CSS/transition-timing-function)
   * can take in.
   */
  easing: _propTypes.default.string,

  /**
   * Id given to the root element.
   */
  id: _propTypes.default.string,

  /**
   * Determines if slider is sliding (animating)
   */
  isSliding: _propTypes.default.bool,

  /**
   * This function is called once the enter animation is complete
   */
  onEnterComplete: _propTypes.default.func,

  /**
   * This function is called once the leave animation is complete
   */
  onLeaveComplete: _propTypes.default.func
};
NavSlider.defaultProps = {
  duration: 500,
  easing: 'ease-out',
  onEnterComplete: noop,
  onLeaveComplete: noop
};
NavSlider.contextTypes = _nav.default.childContextTypes;
var _default = NavSlider;
exports.default = _default;