"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
// Mock `requestAnimationFrame` for tests run using jsDOM
global.requestAnimationFrame = global.window.requestAnimationFrame || function (fn) {
  return setTimeout(fn, 0);
};

test('NavSlider renders without errors', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement("h1", null, "content")));
  expect(wrapper.html()).toBe('<div class="pw-nav-slider"><h1>content</h1></div>');
});
test('renders the contents of the className prop if present', () => {
  const name = 'name';
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    className: name
  }, /*#__PURE__*/_react.default.createElement("h1", null, "content")));
  expect(wrapper.html()).toBe('<div class="pw-nav-slider name"><h1>content</h1></div>');
});
test('animates on enter', () => {
  return Promise.all(['descending', 'ascending'].map(action => {
    let resolvePromise;
    const promise = new Promise(resolve => {
      resolvePromise = resolve;
    });
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      onEnterComplete: resolvePromise
    }), {
      context: {
        action
      }
    });
    expect(wrapper.state().style).toEqual({});
    wrapper.instance().componentWillEnter();
    return promise.then(() => {
      const expected = {
        WebkitTransform: 'translate(0, 0)',
        msTransform: 'translate(0, 0)',
        WebkitTransition: '-webkit-transform 0.5s ease-out,transform 0.5s ease-out',
        transform: 'translate(0, 0)',
        transition: '-ms-transform 0.5s ease-out,-webkit-transform 0.5s ease-out,transform 0.5s ease-out',
        width: '100%',
        position: 'absolute'
      };
      expect(wrapper.state().style).toEqual(expected);
    });
  }));
});
test('animates on leave', () => {
  return Promise.all(['descending', 'ascending'].map(action => {
    let resolvePromise;
    const promise = new Promise(resolve => {
      resolvePromise = resolve;
    });
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      onLeaveComplete: resolvePromise
    }), {
      context: {
        action
      }
    });
    expect(wrapper.state().style).toEqual({});
    wrapper.instance().componentWillLeave();
    return promise.then(() => {
      const direction = `${action === 'descending' ? '-' : ''}100%`;
      const expected = {
        WebkitTransform: `translate(${direction}, 0)`,
        msTransform: `translate(${direction}, 0)`,
        WebkitTransition: '-webkit-transform 0.5s ease-out,transform 0.5s ease-out',
        transform: `translate(${direction}, 0)`,
        transition: '-ms-transform 0.5s ease-out,-webkit-transform 0.5s ease-out,transform 0.5s ease-out',
        width: '100%',
        position: 'absolute'
      };
      expect(wrapper.state().style).toEqual(expected);
    });
  }));
});