"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _button = _interopRequireDefault(require("../button"));

const _excluded = ["className", "buttonClassName", "buttonIconName", "buttonIconSize", "buttonTextHide", "buttonTextShow", "hideButtonText", "analyticsName", "showPassword"];

/**
 * An input field to capture passwords.
 * Allows the user to toggle if their password should be visible or masked.
 */
class PasswordInput extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      passwordVisible: this.props.showPassword || false
    };
    this.toggleVisibility = this.toggleVisibility.bind(this);
    this.getChildren = this.getChildren.bind(this);
  }

  toggleVisibility(evt) {
    evt.preventDefault();
    this.setState({
      passwordVisible: !this.state.passwordVisible
    });
  }

  getChildren() {
    const {
      hideButtonText,
      buttonTextHide,
      buttonTextShow
    } = this.props;
    const {
      passwordVisible
    } = this.state; // if hideButtonText is true, do not show buttonText. Otherwise toggle text with passwordVisible.

    if (hideButtonText) {
      return null;
    } else {
      return passwordVisible ? buttonTextHide : buttonTextShow;
    }
  }

  render() {
    /* eslint-disable */
    const _this$props = this.props,
          {
      className,
      buttonClassName,
      buttonIconName,
      buttonIconSize,
      buttonTextHide,
      buttonTextShow,
      hideButtonText,
      analyticsName,
      showPassword
    } = _this$props,
          inputProps = (0, _objectWithoutProperties2.default)(_this$props, _excluded);
    /* eslint-enable */

    const {
      passwordVisible
    } = this.state;
    const classes = (0, _classnames.default)('pw-password-input', className);
    const toggleClasses = (0, _classnames.default)('pw-password-input__toggle', {
      'pw--inactive': !passwordVisible,
      'pw--is-text': !hideButtonText
    }, buttonClassName);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement("input", (0, _extends2.default)({}, inputProps, {
      type: passwordVisible ? 'text' : 'password',
      "data-analytics-name": analyticsName
    })), /*#__PURE__*/_react.default.createElement(_button.default, {
      className: toggleClasses,
      icon: buttonIconName,
      iconSize: buttonIconSize,
      title: passwordVisible ? buttonTextHide : buttonTextShow,
      onClick: this.toggleVisibility,
      "data-analytics-name": "toggle_password_text"
    }, this.getChildren()));
  }

}

PasswordInput.defaultProps = {
  buttonClassName: 'pw--blank',
  buttonTextShow: 'Show Password',
  buttonTextHide: 'Hide Password',
  buttonIconSize: 'large'
};
PasswordInput.propTypes = {
  /**
   * Adds values to the `analytics-name` attribute of the input
   */
  analyticsName: _propTypes.default.string,

  /**
   * Adds values to the `class` attribute of the toggle button
   */
  buttonClassName: _propTypes.default.string,

  /**
   * Identifier for the desired icon within the toggle button
   */
  buttonIconName: _propTypes.default.string,

  /**
   * The size of the icon within the toggle button
   */
  buttonIconSize: _propTypes.default.string,

  /**
   * The text of the toggle button when the password is visible.
   * This text is also used for accessibility.
   */
  buttonTextHide: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The text of the toggle button when the password is masked
   * This text is also used for accessibility.
   */
  buttonTextShow: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * Determines if button has text or not
   */
  hideButtonText: _propTypes.default.bool,

  /**
   * Determines if password is shown
   */
  showPassword: _propTypes.default.bool
};
var _default = PasswordInput;
exports.default = _default;