"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _icon = _interopRequireDefault(require("../icon"));

var _progressStepsStep = _interopRequireDefault(require("./partials/progress-steps-step"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related component:
 *
 * * [ProgressSteps](#!/ProgressSteps)
 *
 * This component is used inside `ProgressSteps` component to display each step.
 *
 * @example ./DESIGN.md
 */
const ProgressStepsItem = props => {
  const {
    className,
    completed,
    current,
    disabled,
    getStepMessage,
    href,
    icon,
    number,
    title,
    total,
    onClick
  } = props;
  const classes = (0, _classnames.default)('pw-progress-steps__item', {
    'pw--current': current,
    'pw--completed': completed,
    'pw--disabled': disabled
  }, className);
  const stepWidth = 100 / total;
  const styles = {
    width: `${stepWidth}%`
  };
  return /*#__PURE__*/_react.default.createElement("li", {
    className: classes,
    style: styles
  }, /*#__PURE__*/_react.default.createElement(_progressStepsStep.default, {
    className: "pw-progress-steps__step",
    href: !disabled ? href : undefined,
    onClick: !disabled ? onClick : undefined
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-progress-steps__label"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-progress-steps__number"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-progress-steps__badge"
  }, icon ? /*#__PURE__*/_react.default.createElement(_icon.default, {
    className: "pw-progress-steps__icon",
    name: icon
  }) : `${number}`)), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-progress-steps__title"
  }, title, /*#__PURE__*/_react.default.createElement("span", {
    className: "u-visually-hidden"
  }, getStepMessage(number, total))))));
};

const defaultGetStepMessage = (stepIndex, total) => `Step ${stepIndex} of ${total}`;

ProgressStepsItem.defaultProps = {
  completed: false,
  getStepMessage: defaultGetStepMessage
};
ProgressStepsItem.propTypes = {
  /**
   * The label for the step.
   */
  title: _propTypes.default.string.isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY: Adds `pw--completed` styling class to completed step.
   */
  completed: _propTypes.default.bool,

  /**
   * Indicates whether the step is the current one.
   */
  current: _propTypes.default.bool,

  /**
   * Indicates whether the step is disabled.
   */
  disabled: _propTypes.default.bool,

  /**
   * This function should return a string (or node) that describes the meaning of each individual
   * "step" of the `ProgressSteps`. This function is run for each step, and is passed the current
   * step index and the total number of steps.
   */
  getStepMessage: _propTypes.default.func,

  /**
   * If provided, the ProgressStepsItem will render as a Link to this URL.
   */
  href: _propTypes.default.string,

  /**
   * Icon to be displayed in the ProgressStepsItem.
   * If omitted, the step number will be displayed instead.
   * For more information about available icons, see the [Icon component](#!/Icon).
   */
  icon: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY: The number of the step. Rendered if no icon is provided.
   */
  number: _propTypes.default.number,

  /**
   * PROVIDED INTERNALLY: Total number of steps in the ProgressSteps.
   */
  total: _propTypes.default.number,

  /**
   * Called when the ProgressStepsItem is clicked on
   */
  onClick: _propTypes.default.func
};
var _default = ProgressStepsItem;
exports.default = _default;