"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related component:
 *
 * * [ProgressStepsItem](#!/ProgressStepsItem)
 *
 * `ProgressStep` component is commonly used in the checkout flow showing the user
 * which step they are currently on and an overview of the amount of steps in this workflow.
 *
 * @example ./DESIGN.md
 */
const ProgressSteps = ({
  className,
  children,
  disableIncompleteSteps
}) => {
  // count of steps
  const total = children.length;
  let completed = true;
  const classes = (0, _classnames.default)('pw-progress-steps', className);
  return /*#__PURE__*/_react.default.createElement("ol", {
    className: classes
  }, _react.default.Children.map(children, (child, index) => {
    if (child.props.current) {
      completed = false;
    }

    const childProps = {
      total,
      number: index + 1,
      key: index,
      completed,
      disabled: disableIncompleteSteps ? !child.props.current && !completed : false
    };
    return /*#__PURE__*/_react.default.cloneElement(child, childProps);
  }));
};

ProgressSteps.defaultProps = {
  disableIncompleteSteps: false
};
ProgressSteps.propTypes = {
  /**
   * The set of steps in the ProgressSteps. To be provided as
   * ProgressStepsItem components.
   */
  children: _propTypes.default.node.isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Indicates if incomplete steps should be disabled
   */
  disableIncompleteSteps: _propTypes.default.bool
};
var _default = ProgressSteps;
exports.default = _default;