"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.aspectPropType = exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _propTypes2 = require("../../utils/prop-types");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const INT_FRAGMENT = '[1-9]\\d*';
const TWO_INT_REGEX = new RegExp(`^${INT_FRAGMENT}:${INT_FRAGMENT}$`);
/**
 * Fluid-width element with an intrinsic width/height ratio. Best used with
 * media elements like `img` or `video`.
 */

const Ratio = ({
  aspect,
  children,
  className,
  height,
  width
}) => {
  let w;
  let h;

  if (aspect) {
    ;
    [w, h] = aspect.split(':');
  } else {
    // these are guaranteed to be defined and positive if proptypes pass
    ;
    [w, h] = [width, height];
  }

  const ratio = h / w * 100;
  const innerStyle = {
    paddingBottom: `${ratio}%`
  };
  const classes = (0, _classnames.default)('pw-ratio', className);
  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ratio__fill",
    style: innerStyle
  }), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-ratio__inner"
  }, children));
};

exports.default = Ratio;
Ratio.defaultProps = {
  style: {},
  height: 1,
  width: 1
};

const aspectPropType = (props, propName) => {
  const aspect = props[propName];

  if (aspect === undefined) {
    // If undefined, assume that the height and width props are being used
    return null;
  }

  const hasTwoIntegers = TWO_INT_REGEX.test(aspect);

  if (!hasTwoIntegers) {
    return new Error('The aspect prop must contain a ":" character between two positive integers, for example 4:3');
  }

  return null;
};

exports.aspectPropType = aspectPropType;
Ratio.propTypes = {
  /**
   * The aspect ratio, which designates the Ratio component's width and
   * height. Written in the format of `x:y`, for example `4:3`.
   */
  aspect: aspectPropType,

  /**
   * The content that will be rendered inside `pw-ratio__inner`.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Designates the Ratio component's height.
   */
  height: _propTypes2.positiveValue,

  /**
   * Designates the Ratio component's width.
   */
  width: _propTypes2.positiveValue
};