"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _image = _interopRequireDefault(require("../image"));

var _rating = _interopRequireDefault(require("../rating"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * `ReviewSummary` component is used to show overall rating and summary.
 * It uses rating and image components.
 *
 * @example ./DESIGN.md
 */
const ReviewSummary = ({
  className,
  imageProps,
  ratingProps,
  summary,
  text
}) => {
  const classes = (0, _classnames.default)('pw-review-summary', className);
  const ratingStarsClasses = (0, _classnames.default)(`pw-review-summary__rating-stars`, ratingProps.className); // alt is in imageProps

  /* eslint-disable jsx-a11y/img-has-alt */

  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review-summary__image"
  }, /*#__PURE__*/_react.default.createElement(_image.default, imageProps)), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review-summary__rating"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review-summary__rating-inner"
  }, /*#__PURE__*/_react.default.createElement(_rating.default, (0, _extends2.default)({}, ratingProps, {
    className: ratingStarsClasses
  })), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review-summary__rating-info"
  }, summary)), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review-summary__recommended"
  }, text)));
};

ReviewSummary.defaultProps = {
  ratingProps: {
    className: 'pw--solid'
  }
};
ReviewSummary.propTypes = {
  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * The properties for Image.
   */
  imageProps: _propTypes.default.object,

  /**
   * The properties for Rating.
   */
  ratingProps: _propTypes.default.object,

  /**
   * Summary of the review that displays the number of given stars over total stars.
   */
  summary: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Content of the review summary.
   */
  text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = ReviewSummary;
exports.default = _default;