"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _rating = _interopRequireDefault(require("../rating"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * `Review` component is used to show title, author's info and descripton.
 * It also uses rating component to display user's satisfaction.
 *
 * @example ./DESIGN.md
 */
const Review = ({
  text,
  className,
  ratingProps,
  info,
  title
}) => {
  const classes = (0, _classnames.default)('pw-review', className);
  const ratingClasses = (0, _classnames.default)(`pw-review__rating`, ratingProps.className);
  return /*#__PURE__*/_react.default.createElement("article", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement("header", {
    className: "pw-review__header"
  }, /*#__PURE__*/_react.default.createElement("h1", {
    className: "pw-review__heading"
  }, title), /*#__PURE__*/_react.default.createElement(_rating.default, (0, _extends2.default)({}, ratingProps, {
    className: ratingClasses
  }))), info && /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-review__author"
  }, info.map((info, idx) => /*#__PURE__*/_react.default.createElement("p", {
    className: "pw-review__author-info",
    key: idx
  }, info))), /*#__PURE__*/_react.default.createElement("p", {
    className: "pw-review__content"
  }, text));
};

Review.defaultProps = {
  ratingProps: {
    className: 'pw--solid'
  }
};
Review.propTypes = {
  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * An array of information related to the author.
   */
  info: _propTypes.default.array,

  /**
   * The properties for Rating.
   */
  ratingProps: _propTypes.default.object,

  /**
   * Content of the review.
   */
  text: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Title of the review.
   */
  title: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
};
var _default = Review;
exports.default = _default;