"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * The ScrollTrigger component allows you to register callbacks for when it is scrolled into or out of view
 */


class ScrollTrigger extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      inView: false
    };
    this.handleScroll = (0, _lodash.default)(this.handleScroll.bind(this), props.throttle);
  }

  componentDidMount() {
    window.addEventListener('scroll', this.handleScroll);
    window.addEventListener('resize', this.handleScroll); // Check if the component has mounted in view

    this.handleScroll();
  }

  handleScroll() {
    const {
      offsetTop,
      offsetBottom,
      onEnter,
      onLeave
    } = this.props;
    let top = 0;
    let bottom = 0;
    let boundingClientRect; // When using enzyme's `shallow`, then `this.element` will be undefined!
    // Check that it exists, before using it...

    if (this.element && this.element.getBoundingClientRect) {
      boundingClientRect = this.element.getBoundingClientRect();
      top = boundingClientRect.top;
      bottom = boundingClientRect.bottom;
    }

    const boundingBoxTop = top + offsetTop;
    const boundingBoxBottom = bottom + offsetBottom;
    const innerHeight = window.innerHeight;
    const inView = boundingBoxTop <= innerHeight && boundingBoxTop >= 0 || boundingBoxBottom <= innerHeight && boundingBoxBottom >= 0;
    const hasChanged = inView !== this.state.inView;

    if (hasChanged) {
      this.setState({
        inView
      });

      if (inView) {
        onEnter();
      } else {
        onLeave();
      }
    }
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this.handleScroll);
    window.removeEventListener('resize', this.handleScroll);
  }

  render() {
    const {
      children,
      className
    } = this.props;
    const classes = (0, _classnames.default)('pw-scroll-trigger', className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      ref: el => {
        this.element = el;
      }
    }, children);
  }

}

ScrollTrigger.defaultProps = {
  offsetBottom: 0,
  offsetTop: 0,
  throttle: 50,
  onEnter: noop,
  onLeave: noop
};
ScrollTrigger.propTypes = {
  /**
   * This children of this component.
   * ScrollTrigger will take their height into account when determining if the component is in view
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element
   */
  className: _propTypes.default.string,

  /**
   * Offsets the "bottom" position of the component.
   * A negative value offsets the bottom above its real position,
   * while a positive value offsets the bottom below its real position
   */
  offsetBottom: _propTypes.default.number,

  /**
   * Offsets the "top" position of the component.
   * A negative value offsets the top above its real position,
   * while a positive value offsets the top below its real position
   */
  offsetTop: _propTypes.default.number,

  /**
   * Determines how frequently the component checks if it has been scrolled into view
   */
  throttle: _propTypes.default.number,

  /**
   * This function is called when the element is scrolled into view
   */
  onEnter: _propTypes.default.func,

  /**
   * This function is called when the element is scrolled out of view
   */
  onLeave: _propTypes.default.func
};
var _default = ScrollTrigger;
exports.default = _default;