"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _button = _interopRequireDefault(require("../button"));

var _icon = _interopRequireDefault(require("../icon"));

var _a11y = require("../../utils/a11y");

var _popoverSuggestions = _interopRequireDefault(require("./partials/popover-suggestions"));

var _takeoverSuggestions = _interopRequireDefault(require("./partials/takeover-suggestions"));

var _searchWrapper = _interopRequireDefault(require("./partials/search-wrapper"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;

const searchId = (() => {
  let i = 0;
  return () => {
    return i++;
  };
})();
/**
 * `Search` component that includes two variants: inline and overlay.
 * This component is commonly used in the header.
 *
 * @example ./DESIGN.md
 */


class Search extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      searchValue: '',
      isActive: false,
      id: `search-${searchId()}`
    };
    this.componentDidUpdate = this.componentDidUpdate.bind(this);
    this.focusInput = this.focusInput.bind(this);
    this.blurInput = this.blurInput.bind(this);
    this.resetInput = this.resetInput.bind(this);
    this.clearInput = this.clearInput.bind(this);
    this.closeSearch = this.closeSearch.bind(this);
    this.submitSearch = this.submitSearch.bind(this);
    this.clickSuggestion = this.clickSuggestion.bind(this);
  }

  componentDidUpdate(prevProps, prevState) {
    if (!prevProps.isOpen && this.props.isOpen) {
      this.focusInput();
    }

    const componentWasOpen = prevProps.isOpen && !this.props.isOpen;
    const componentWasActive = prevState.isActive && !this.state.isActive;

    if (componentWasOpen || componentWasActive && !this.props.allowPageInteractions) {
      this.resetInput();
    }
  }

  resetInput() {
    this.setState({
      searchValue: ''
    });
  }

  clearInput() {
    this.resetInput();
    this.focusInput();
    this.props.onClear();
  }

  focusInput() {
    this.input.focus();
  } // @TODO: check and make sure this doesn't break accessibility


  blurInput() {
    this.input.blur();
  }

  closeSearch(e) {
    if (!this.state.isActive) {
      return; // do nothing if this search isn't even active
    }

    this.setState({
      isActive: false
    });

    if (!this.props.allowPageInteractions) {
      this.resetInput();
    }

    this.props.onClose(e);
    this.blurInput();
  }

  submitSearch(e) {
    this.props.onSubmit(e);
    this.closeSearch();
  }

  clickSuggestion(e) {
    this.props.onClickSuggestion(e);
    this.closeSearch();
  }

  render() {
    const {
      accessibleLabel,
      allowPageInteractions,
      className,
      onChange,
      onClose,
      includeClearButton,
      inputProps,
      isOverlay,
      searchIcon,
      replaceSearchWithClose,
      suggestedTermHeading,
      suggestedProductsHeading,
      termSuggestions,
      productSuggestions,
      onClickSuggestion,
      submitButtonProps,
      closeButtonProps,
      clearButtonProps
    } = this.props;
    const {
      isActive,
      searchValue,
      id
    } = this.state;
    const classes = (0, _classnames.default)('pw-search', {
      'pw--is-active': isActive,
      'pw--is-overlay': isOverlay
    }, className);
    const searchSuggestionClasses = (0, _classnames.default)('pw-search__suggestions', {
      'pw--is-empty': !termSuggestions && !productSuggestions
    });
    const searchIconClasses = (0, _classnames.default)('pw-search__icon', {
      'pw--is-not-clickable': !(isOverlay && replaceSearchWithClose)
    });
    const suggestionsProps = {
      clickSuggestion: this.clickSuggestion,
      onClickSuggestion,
      productSuggestions,
      searchSuggestionClasses,
      suggestedProductsHeading,
      suggestedTermHeading,
      termSuggestions
    };
    const isAllowingInteractionAndHasSuggestions = allowPageInteractions && (productSuggestions || termSuggestions);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      role: "search"
    }, /*#__PURE__*/_react.default.createElement(_searchWrapper.default, {
      onClickOutside: this.closeSearch
    }, /*#__PURE__*/_react.default.createElement("form", {
      className: "pw-search__form",
      onSubmit: this.submitSearch,
      onChange: onChange,
      action: "javascript:void(0)"
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-search__bar"
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: searchIconClasses
    }, replaceSearchWithClose && isOverlay ? /*#__PURE__*/_react.default.createElement(_button.default, (0, _extends2.default)({
      onClick: this.closeSearch
    }, closeButtonProps)) : /*#__PURE__*/_react.default.createElement(_icon.default, {
      className: "pw-search__icon-content",
      name: searchIcon
    })), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-search__field"
    }, /*#__PURE__*/_react.default.createElement("label", {
      htmlFor: id,
      className: "u-visually-hidden"
    }, accessibleLabel), /*#__PURE__*/_react.default.createElement("input", (0, _extends2.default)({
      className: "pw-search__input",
      ref: el => {
        this.input = el;
      },
      id: id,
      value: searchValue,
      onChange: e => this.setState({
        searchValue: e.target.value
      }),
      onFocus: () => this.setState({
        isActive: true
      }),
      type: "search",
      name: "query",
      "data-analytics-name": "search"
    }, inputProps))), includeClearButton && searchValue.length > 0 && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-search__button-clear"
    }, /*#__PURE__*/_react.default.createElement(_button.default, (0, _extends2.default)({
      onClick: this.clearInput
    }, clearButtonProps, {
      "data-analytics-name": "clear_search"
    }))), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-search__button-submit"
    }, /*#__PURE__*/_react.default.createElement(_button.default, (0, _extends2.default)({
      type: "submit",
      disabled: searchValue.trim().length === 0
    }, submitButtonProps, {
      "data-analytics-name": "search"
    }))), isOverlay && !replaceSearchWithClose && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-search__button-close"
    }, /*#__PURE__*/_react.default.createElement(_button.default, (0, _extends2.default)({
      onClick: this.closeSearch
    }, closeButtonProps, {
      "data-analytics-name": "dismiss_search"
    }))))), isAllowingInteractionAndHasSuggestions ? /*#__PURE__*/_react.default.createElement(_popoverSuggestions.default, suggestionsProps) : /*#__PURE__*/_react.default.createElement(_takeoverSuggestions.default, suggestionsProps)), !allowPageInteractions && /*#__PURE__*/_react.default.createElement("div", {
      tabIndex: "-1",
      role: "presentation",
      onKeyUp: (0, _a11y.onKeyUpWrapper)(onClose),
      className: "pw-search__shade",
      onClick: this.closeSearch
    }));
  }

}

Search.defaultProps = {
  accessibleLabel: 'Search',
  allowPageInteractions: false,
  submitButtonProps: {
    text: 'Submit search'
  },
  closeButtonProps: {
    text: 'Close search'
  },
  clearButtonProps: {
    text: 'Clear'
  },
  includeClearButton: true,
  replaceSearchWithClose: false,
  searchIcon: 'search',
  suggestedTermHeading: 'Suggested Search Terms',
  suggestedProductsHeading: 'Suggested Products',
  onClear: noop,
  onClose: noop,
  onSubmit: noop
};
Search.propTypes = {
  /**
   * Adds text as a label for the search input, accessible
   * to screen readers, but hidden to visual users.
   */
  accessibleLabel: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * This prop is only used when `isOverlay` is `false`.
   *
   * If true the search suggestions will behave like the popover component,
   * interactions on the page will not be blocked while suggestions are open,
   * clicking outside the suggestions will close the suggestions popover
   */
  allowPageInteractions: _propTypes.default.bool,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * The properties for clear button.
   */
  clearButtonProps: _propTypes.default.object,

  /**
   * The properties for close button.
   */
  closeButtonProps: _propTypes.default.object,

  /**
   * Indicates if the clear button should be included in the search component
   */
  includeClearButton: _propTypes.default.bool,

  /**
   * The data for the input you wish to render.
   */
  inputProps: _propTypes.default.object,

  /**
   * Is only used when `isOverlay` is `true`. Its comparison in `componentDidUpdate`
   * determines whether to allow or disallow input focus, or input reset. This
   * prop does nothing for an inline search. Note that this is different
   * from isActive.
   */
  isOpen: _propTypes.default.bool,

  /**
   * Controls whether the search component uses an overlay layout or not.
   */
  isOverlay: _propTypes.default.bool,

  /**
   * An array of the search(product) suggestions to be displayed.
   */
  productSuggestions: _propTypes.default.array,

  /**
   * This prop is only used when `isOverlay` is `true`.
   *
   * If true the close button will replace the search icon
   * when the search overlay is open. Clicking the button.
   *
   */
  replaceSearchWithClose: _propTypes.default.bool,

  /**
   * Icon name for search.
   * For more information about available icons, see the [Icon component](#!/Icon).
   */
  searchIcon: _propTypes.default.string,

  /**
   * The properties for submit button.
   */
  submitButtonProps: _propTypes.default.object,

  /**
   * The text for suggested products heading.
   */
  suggestedProductsHeading: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The text for suggested terms heading.
   */
  suggestedTermHeading: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * An array of the search(term) suggestions to be displayed
   */
  termSuggestions: _propTypes.default.array,

  /**
   * The function to be called when the form is changed.
   */
  onChange: _propTypes.default.func,

  /**
   * The function to be called when the clear button is clicked.
   */
  onClear: _propTypes.default.func,

  /**
   * The function to be called when a suggestion is clicked.
   */
  onClickSuggestion: _propTypes.default.func,

  /**
   * The function to be called when the close button is clicked.
   */
  onClose: _propTypes.default.func,

  /**
   * The function to be called after form is submitted.
   */
  onSubmit: _propTypes.default.func
};
var _default = Search;
exports.default = _default;