"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = _interopRequireWildcard(require("./index"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('Select renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.length).toBe(1);
});
test('SelectLabel renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SelectLabel, null));
  expect(wrapper.length).toBe(1);
});
test('SelectOption renders an option element with the passed value and text', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SelectOption, {
    value: "test",
    text: "Test!"
  }));
  expect(wrapper.contains( /*#__PURE__*/_react.default.createElement("option", {
    value: "test"
  }, "Test!"))).toBe(true);
});
test('SelectOption has the value as the default text', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SelectOption, {
    value: "Again"
  }));
  expect(wrapper.contains( /*#__PURE__*/_react.default.createElement("option", {
    value: "Again"
  }, "Again"))).toBe(true);
});
test('SelectLabel renders a label if text is passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SelectLabel, {
    label: "testing"
  }));
  const labels = wrapper.find('label');
  expect(labels.length).toBe(1);
  expect(labels.childAt(0).text()).toBe('testing');
});
test('SelectLabel renders no label if no text is passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SelectLabel, null));
  expect(wrapper.find('label').length).toBe(0);
});
test('Select renders a SelectLabel with the passed value', () => {
  ;
  ['Label', null].forEach(label => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      options: [{
        value: ''
      }],
      label: label
    }));
    expect(wrapper.find(_index.SelectLabel).length).toBe(1);
    expect(wrapper.find(_index.SelectLabel).prop('label')).toBe(label);
  });
});
test('Select renders a select element containing SelectOptions', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    options: [{
      value: 'test'
    }]
  }));
  const selects = wrapper.find('select');
  expect(selects.length).toBe(1);
  expect(selects.find(_index.SelectOption).length).toBe(1);
  expect(selects.find(_index.SelectOption).prop('value')).toBe('test');
});
test('Select renders one SelectOption per passed option, in order', () => {
  const options = [{
    value: 'first'
  }, {
    value: 'second',
    text: '2nd'
  }, {
    value: 'third',
    text: 'third and last'
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    options: options
  }));
  expect(wrapper.find(_index.SelectOption).length).toBe(options.length);
  expect(wrapper.find(_index.SelectOption).reduce((soFar, node, idx) => {
    return soFar && node.prop('value') === options[idx].value && node.prop('text') === options[idx].text;
  }, true)).toBe(true);
});
test('Select correctly shows selected option when passed selectedIndex', () => {
  const options = [{
    value: 'first'
  }, {
    value: 'second',
    text: '2nd'
  }, {
    value: 'third',
    text: 'third and last'
  }];
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    options: options,
    selectedIndex: 2
  }));
  expect(wrapper.find('select').prop('defaultValue')).toBe('third');
});
test('Select renders with name attribute if passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    name: "test-name"
  }));
  expect(wrapper.find('select').prop('name')).toBe('test-name');
});
test('Select renders with id attribute if passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    id: "test-id"
  }));
  expect(wrapper.find('select').prop('id')).toBe('test-id');
});
test('Select renders with multiple attribute if passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    multiple: true
  }));
  expect(wrapper.find('select').prop('multiple')).toBe(true);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('c-select')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      className: name
    }));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});