"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _sheet = _interopRequireDefault(require("../sheet"));

var _shareSheetContent = _interopRequireDefault(require("./partials/share-sheet-content"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined; // A function so that it is testable


const defaultTriggerElement = /*#__PURE__*/_react.default.createElement("button", null, /*#__PURE__*/_react.default.createElement("span", null, "Share\u2026"));

const defaultShareContent = {
  title: null,
  text: null,
  url: null
};

const defaultShareSheetHeader = /*#__PURE__*/_react.default.createElement("h1", {
  className: "pw-share__sheet-header"
}, "Share via");
/**
 * The `Share` component provides a way for Progressive Web Apps
 * to share the URL of the current page.
 *
 * It uses the WebShare API if supported on the current device and falls back
 * to a limited web implementation.
 *
 * @example ./DESIGN.md
 */


class Share extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      isMounted: false,
      webShareSupported: false,
      title: null,
      text: null,
      url: null
    };
    this.onShow = this.onShow.bind(this);
    this.onDismiss = this.onDismiss.bind(this);
  }

  static getDerivedStateFromProps(props, state) {
    const {
      isMounted
    } = state;
    const {
      title,
      text,
      url
    } = props.shareContent || {};
    return {
      webShareSupported: isMounted ? navigator.share !== undefined : false,
      title: title || (isMounted ? document.title : null),
      url: url || (isMounted ? window.location.href : null),
      text: text || null
    };
  }

  componentDidMount() {
    this.setState({
      isMounted: true
    });
  }

  onShow() {
    const {
      title,
      text,
      url,
      webShareSupported
    } = this.state;

    if (webShareSupported) {
      navigator.share({
        title,
        text,
        url
      }).then(this.props.onSuccess).catch(error => {
        this.props.onFail(error);
        console.error('An error has occurred while attempting to share: ', error);
      });
    } else {
      this.props.onShow();
    }
  }

  onDismiss() {
    this.props.onDismiss();
  }

  render() {
    const {
      className,
      coverage,
      duration,
      headerContent,
      open,
      optionsPerCol,
      optionsPerRow,
      triggerElement,
      didDismiss,
      didShow,
      onFail,
      onSuccess,
      willDismiss,
      willShow
    } = this.props;
    const {
      webShareSupported,
      title,
      text,
      url
    } = this.state;
    const classes = (0, _classnames.default)('pw-share', className);
    const triggerClasses = (0, _classnames.default)('pw-share__trigger', triggerElement.props.className); // Bind click handler to the root trigger element

    const triggerComponent = /*#__PURE__*/_react.default.cloneElement(triggerElement, {
      className: triggerClasses,
      onClick: this.onShow
    });

    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, triggerComponent, webShareSupported ? null : /*#__PURE__*/_react.default.createElement(_sheet.default, {
      coverage: coverage,
      duration: duration,
      effect: "slide-bottom",
      headerContent: headerContent,
      open: open,
      onDismiss: this.onDismiss,
      onOpen: didShow,
      onClose: didDismiss,
      onBeforeClose: willDismiss,
      onBeforeOpen: willShow
    }, /*#__PURE__*/_react.default.createElement(_shareSheetContent.default, {
      optionsPerCol: optionsPerCol,
      optionsPerRow: optionsPerRow,
      shareContent: {
        title,
        url,
        text
      },
      onDismiss: this.onDismiss,
      onFail: onFail,
      onSuccess: onSuccess
    })));
  }

}

Share.propTypes = {
  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Coverage defines the amount of the current viewport taken up by the share
   * sheet content.
   */
  coverage: _propTypes.default.string,

  /**
   * User-defined function that is called after the `Share` sheet is dismissed.
   */
  didDismiss: _propTypes.default.func,

  /**
   * User-defined function that is called after the `Share` sheet is shown.
   */
  didShow: _propTypes.default.func,

  /**
   * Defines the time the show/dismiss animation takes to complete.
   */
  duration: _propTypes.default.number,

  /**
   * User-defined header of the sheet
   */
  headerContent: _propTypes.default.element,

  /**
   * Determines whether the Share Sheet is opened or closed.
   */
  open: _propTypes.default.bool,

  /**
   * Defines the number of share options displayed per column on the `Share` sheet.
   */
  optionsPerCol: _propTypes.default.number,

  /**
   * Defines the number of share options displayed per row on the `Share` sheet.
   */
  optionsPerRow: _propTypes.default.number,

  /**
   * Defines what is to be shared by the `Share` component. An object with the following
   * structure: `{title, text, url}`
   */
  shareContent: _propTypes.default.object,

  /**
   * User-defined element that is responsible for triggering the `Share` sheet.
   */
  triggerElement: _propTypes.default.element,

  /**
   * User-defined function that is called before the `Share` sheet is dismissed.
   */
  willDismiss: _propTypes.default.func,

  /**
   * User-defined function that is called before the `Share` sheet is shown.
   */
  willShow: _propTypes.default.func,
  // Callback Props

  /**
   * User-defined function that is called when the user clicks to dismiss the sheet.
   * Use this to change state and re-render the `Share` sheet through props.
   */
  onDismiss: _propTypes.default.func,

  /**
   * User-defined function that is called when the component fails to
   * copy or share the URL. `function(error) {...}`
   */
  onFail: _propTypes.default.func,

  /**
   * User-defined function that is used to trigger the rendering of the sheet.
   * Use this to change state and re-render the `Share` sheet through props.
   */
  onShow: _propTypes.default.func,

  /**
   * User-defined function that is called when the component successfully
   * triggers the share option that a user selected. `function(shareData) {...}`
   */
  onSuccess: _propTypes.default.func
};
Share.defaultProps = {
  coverage: '50%',
  duration: 200,
  open: false,
  optionsPerRow: 4,
  optionsPerCol: 2,
  headerContent: defaultShareSheetHeader,
  shareContent: defaultShareContent,
  triggerElement: defaultTriggerElement,
  didDismiss: noop,
  didShow: noop,
  onDismiss: noop,
  onShow: noop,
  onFail: noop,
  onSuccess: noop,
  willDismiss: noop,
  willShow: noop
};
var _default = Share;
exports.default = _default;