"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _TransitionGroup = _interopRequireDefault(require("react-transition-group/TransitionGroup"));

var _sheetContent = _interopRequireDefault(require("./sheet-content"));

var _propTypes2 = require("../../utils/prop-types");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;

const TRANSITION_GROUP_STYLE = {
  zIndex: 1000,
  position: 'fixed',
  top: 0,
  left: 0,
  // In certain circumstances (see https://github.com/mobify/progressive-web-sdk/pull/1187)
  // we found that the sheet didn't render correctly
  // To fix this, we want to push it onto a new compositing layer
  // Setting willChange pushes it onto the new layer
  willChange: 'transform',
  // Setting the width and height allows the sheet to render inside this new layer
  width: '100%',
  height: '100%',
  // This element is always visible, and it will cover the entire viewport
  // so, disable pointer events on it
  pointerEvents: 'none'
};
/**
 * Sheets are modal containers that animate into view from one side of the screen
 * to reveal more content. Typically, sheets are only displayed as a result of
 * user initiated action, and persist until dismissed. Common uses for sheet
 * include, but not limited to, dialog modals and sidebar navigations.
 *
 * @example ./DESIGN.md
 */

class Sheet extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      portalEl: null
    };
  }

  componentDidMount() {
    const {
      portalId
    } = this.props;
    this.setState({
      portalEl: portalId && typeof document !== 'undefined' ? document.getElementById(portalId) : null
    });
  }

  render() {
    const {
      portalEl
    } = this.state;
    const {
      open,
      prerender,
      children,
      className
    } = this.props;

    const wrapper = /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-sheet__outer-wrapper"
    }, /*#__PURE__*/_react.default.createElement(_TransitionGroup.default, {
      component: "div",
      style: TRANSITION_GROUP_STYLE
    }, open ? /*#__PURE__*/_react.default.createElement(_sheetContent.default, this.props) : null), !open && prerender && /*#__PURE__*/_react.default.createElement("div", {
      className: `pw-sheet__prerendered-children u-visually-hidden ${className}`,
      "aria-hidden": "true"
    }, children));

    return portalEl ? /*#__PURE__*/_reactDom.default.createPortal([wrapper], portalEl) : wrapper;
  }

}

Sheet.propTypes = {
  /**
   * User-defined content of the sheet, can be a combination of markup and/or
   * React components.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Coverage defines the amount of the current viewport taken up by the
   * sheet content.
   */
  coverage: _propTypes2.percentage,

  /**
   * Duration will define the time the animation takes to complete.
   */
  duration: _propTypes.default.number,

  /**
   * Easing function for the animation as a string whose value
   * can be anything the [CSS `transition-timing-function`](https://developer.mozilla.org/en/docs/Web/CSS/transition-timing-function)
   * can take in.
   */
  easing: _propTypes.default.string,

  /**
   * Type will define the entry point of the sheet. See below for details.
   */
  effect: _propTypes.default.oneOf(['slide-top', 'slide-right', 'slide-bottom', 'slide-left', 'modal-center']),

  /**
   * User-defined footer of the sheet.
   */
  footerContent: _propTypes.default.element,

  /**
   * User-defined header of the sheet.
   */
  headerContent: _propTypes.default.element,

  /**
   * Set ID of root element. Generally used to backtrace the source of a
   * sheet.
   */
  id: _propTypes.default.string,

  /**
   * Level of opacity on sheet mask.
   */
  maskOpacity: _propTypes.default.number,

  /**
   * Determines whether the Sheet is opened or closed.
   */
  open: _propTypes.default.bool,

  /**
   * Specifies the id of the DOM element in which the  Sheet will be rendered to.
   * If left undefined, the sheet will be rendered inline.
   */
  portalId: _propTypes.default.string,

  /**
   * This is used by Sheet to prerender the content before openning it.
   */
  prerender: _propTypes.default.bool,

  /**
   * Dictate whether the height of the modal grows to the maximum height as
   * declared by the `coverage` prop, or to shrink down to the height of the
   * modal's content.
   */
  shrinkToContent: _propTypes.default.bool,

  /**
   * User-defined title of the sheet.
   */
  title: _propTypes.default.string,
  // Callback props

  /**
   * User-defined function that is called before the sheet is closed.
   */
  onBeforeClose: _propTypes.default.func,

  /**
   * User-defined function that is called before the sheet is opened.
   */
  onBeforeOpen: _propTypes.default.func,

  /**
   * User-defined function that is called after the sheet is closed.
   */
  onClose: _propTypes.default.func,

  /**
   * User-defined function that is called when the user clicks to dismiss the
   * sheet. Use this to change state and re-render the Sheet through props.
   */
  onDismiss: _propTypes.default.func,

  /**
   * User-defined function that is called after the sheet is opened.
   */
  onOpen: _propTypes.default.func
};
/* istanbul ignore next */

Sheet.defaultProps = {
  coverage: '80%',
  duration: 200,
  onBeforeClose: noop,
  onBeforeOpen: noop,
  onClose: noop,
  onOpen: noop,
  onDismiss: noop,
  maskOpacity: 0.5,
  open: false,
  easing: 'ease-in-out',
  effect: 'slide-left',
  shrinkToContent: false
};
var _default = Sheet;
exports.default = _default;