"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTouchEventObject = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _TransitionGroup = _interopRequireDefault(require("react-transition-group/TransitionGroup"));

var _index = _interopRequireDefault(require("./index"));

var _sheetContent = _interopRequireDefault(require("./sheet-content"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const sheetClassName = '.pw-sheet';
const wrapperClassName = '.pw-sheet__wrapper';
const prerenderClassName = '.pw-sheet__prerendered-children';

const createClientXY = (x, y) => {
  return {
    clientX: x,
    clientY: y,
    screenX: x,
    screenY: y
  };
};

const createTouchEventObject = ({
  x = 0,
  y = 0
}) => {
  return {
    touches: [createClientXY(x, y)]
  };
}; // Mock `requestAnimationFrame` for tests run using jsDOM


exports.createTouchEventObject = createTouchEventObject;

global.requestAnimationFrame = global.window.requestAnimationFrame || function (fn) {
  return setTimeout(fn, 0);
};
/* eslint-disable newline-per-chained-call */


describe('SheetContent', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, _index.default.defaultProps));
    expect(wrapper.length).toBe(1);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, _index.default.defaultProps));
    expect(wrapper.hasClass('pw-sheet')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, _index.default.defaultProps));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, (0, _extends2.default)({}, _index.default.defaultProps, {
        className: name
      })));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('renders the header content if passed', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      headerContent: /*#__PURE__*/_react.default.createElement("div", {
        id: "test"
      }, "Test")
    });

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__inner').childAt(0).hasClass('pw-sheet__header')).toBe(true);
    expect(wrapper.find('.pw-sheet__header').length).toBe(1);
    expect(wrapper.find('.pw-sheet__header > #test').length).toBe(1);
    expect(wrapper.find('#test').text()).toBe('Test');
  });
  test('renders the title if passed', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      title: 'SheetContent Title'
    });

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__header').length).toBe(1);
    expect(wrapper.find('.pw-sheet__header > h1').text()).toBe(props.title);
  });
  test('renders the footer content if passed', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      footerContent: /*#__PURE__*/_react.default.createElement("div", {
        id: "test"
      }, "Test")
    });

    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__inner').childAt(1).hasClass('pw-sheet__footer')).toBe(true);
    expect(wrapper.find('.pw-sheet__footer').length).toBe(1);
    expect(wrapper.find('.pw-sheet__footer > #test').length).toBe(1);
    expect(wrapper.find('#test').text()).toBe('Test');
  });
  test('clicking on the mask calls onDismiss', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      onDismiss: jest.fn()
    });

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__mask').length).toBe(1);
    expect(props.onDismiss).not.toBeCalled();
    wrapper.find('.pw-sheet__mask').simulate('click');
    expect(props.onDismiss).toBeCalled();
  });
  test('clicking directly on the wrapper calls onDismiss', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      onDismiss: jest.fn()
    });

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__wrapper').length).toBe(1);
    expect(props.onDismiss).not.toBeCalled();
    wrapper.find('.pw-sheet__wrapper').simulate('click');
    expect(props.onDismiss).toBeCalled();
  });
  test('clicking on the inner content does not call onDismiss', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      onDismiss: jest.fn()
    });

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    expect(wrapper.find('.pw-sheet__inner').length).toBe(1);
    expect(props.onDismiss).not.toBeCalled();
    wrapper.find('.pw-sheet__inner').simulate('click');
    expect(props.onDismiss).not.toBeCalled();
  });
  test('renders in center of screen if effect is modal-center', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      effect: 'modal-center'
    });

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    let style = wrapper.find(wrapperClassName).prop('style');
    expect(style.top).toBe('10%');
    expect(style.right).toBe('10%');
    expect(style.bottom).toBe('10%');
    expect(style.left).toBe('10%');
    expect(style.transform).toBe('scale(0)');
    wrapper.setState({
      phase: 'enterActive'
    });
    wrapper.update();
    style = wrapper.find(wrapperClassName).prop('style');
    expect(style.transform).toBe('scale(1)');
  });
  test('calls onOpen when component appears', () => {
    const props = _objectSpread(_objectSpread({}, _index.default.defaultProps), {}, {
      onOpen: jest.fn()
    });

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, props));
    wrapper.instance().componentDidAppear();
    expect(props.onOpen).toHaveBeenCalled();
  });
  describe('touch events', () => {
    let wrapper;
    let instance;
    const mockTouchStartEvent = createTouchEventObject({
      x: 10,
      y: 20
    });
    beforeEach(() => {
      wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_sheetContent.default, _index.default.defaultProps));
      instance = wrapper.instance();
      instance.isIOS = true; // Mock the event target DOM node's closest method

      mockTouchStartEvent.target = {
        closest: () => instance.innerEl
      };
    });
    test('should scroll down 1px when touch starts, if scroll is at the top', () => {
      const startingScrollPosition = 0;
      instance.innerEl.scrollTop = startingScrollPosition; // We're at the "top" of the scroll position

      instance.onTouchStart(mockTouchStartEvent);
      expect(instance.innerEl.scrollTop).toBe(startingScrollPosition + 1);
    });
    test('should scroll up 1px when touch starts, if scroll is at the bottom', () => {
      const startingScrollPosition = 10;
      instance.innerEl.scrollTop = startingScrollPosition; // We're at the "bottom" of the scroll position

      instance.onTouchStart(mockTouchStartEvent);
      expect(instance.innerEl.scrollTop).toBe(startingScrollPosition - 1);
    });
    test('should NOT scroll up 1px when touch starts, if on non-iOS', () => {
      instance.isIOS = false; // Same steps as "should scroll down 1px when touch starts"

      const startingScrollPosition = 0;
      instance.innerEl.scrollTop = startingScrollPosition; // We're at the "top" of the scroll position

      instance.onTouchStart(mockTouchStartEvent);
      expect(instance.innerEl.scrollTop).toBe(startingScrollPosition);
    });
    test('should NOT scroll up 1px when touch starts, if on non-iOS', () => {
      instance.isIOS = false; // Same steps as "should scroll up 1px when touch starts"

      const startingScrollPosition = 10;
      instance.innerEl.scrollTop = startingScrollPosition; // We're at the "bottom" of the scroll position

      instance.onTouchStart(mockTouchStartEvent);
      expect(instance.innerEl.scrollTop).toBe(startingScrollPosition);
    });
  });
});
describe('Sheet', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
    expect(wrapper.length).toBe(1);
  });
  test('renders to portal element without errors', () => {
    const portalId = 'sheets-root'; // add a div with #sheets-root id to the global body

    const sheetsRoot = global.document.createElement('div');
    sheetsRoot.setAttribute('id', portalId);
    const body = global.document.querySelector('body');
    body.appendChild(sheetsRoot);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      open: true,
      portalId: portalId
    }));
    expect(wrapper.find(`#${portalId} ${sheetClassName}`).exists());
  });
  test('does not render children when closed', () => {
    const children = /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement("p", null, "This is child"));

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      open: false
    }, children));
    const sheetContents = wrapper.find(sheetClassName);
    expect(sheetContents.length).toBe(0);
    expect(wrapper.contains(children)).toBe(false);
  });
  test('renders children when open', () => {
    const children = /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement("p", null, "This is child"));

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      open: true
    }, children));
    const sheetContents = wrapper.find(sheetClassName);
    expect(sheetContents.length).toBe(1);
    const sheetContent = sheetContents.first();
    expect(sheetContent.contains(children)).toBe(true);
  });
  test('render children when closed and prerender is true', () => {
    const children = /*#__PURE__*/_react.default.createElement("div", null, /*#__PURE__*/_react.default.createElement("p", null, "This is a pre-rendered child"));

    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      prerender: true,
      open: false
    }, children));
    const prerenderedWrapper = wrapper.find(prerenderClassName);
    expect(prerenderedWrapper.length).toBe(1);
    const prerenderedChildren = prerenderedWrapper.first();
    expect(prerenderedChildren.contains(children)).toBe(true);
  });
  test('the element that contains the Sheet creates a new compositing layer', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
    const container = wrapper.find(_TransitionGroup.default);
    const {
      style
    } = container.props();
    expect(style.width).toBe('100%');
    expect(style.height).toBe('100%');
    expect(style.willChange).toBe('transform');
    expect(style.pointerEvents).toBe('none');
  }); // TODO: Re-add the test after May-ple Syrup Release
  // test('supports callbacks signalling opening/closing transitions', () => {
  //     const duration = 50
  //     const timeOut = duration * 3
  //     const onBeforeOpen = jest.fn()
  //     const onOpen = jest.fn()
  //     const onBeforeClose = jest.fn()
  //     const onClose = jest.fn()
  //     const wrapper = mount(
  //         <Sheet
  //             onBeforeOpen={onBeforeOpen}
  //             onOpen={onOpen}
  //             onBeforeClose={onBeforeClose}
  //             onClose={onClose}
  //             open={false}
  //             duration={duration}
  //         />
  //     )
  //     // Sanity check - rendered, but closed initially
  //     expect(onBeforeOpen).not.toBeCalled()
  //     expect(onOpen).not.toBeCalled()
  //     expect(onBeforeClose).not.toBeCalled()
  //     expect(onClose).not.toBeCalled()
  //     const doOpen = () => {
  //         return new Promise((resolve) => {
  //             wrapper.setProps({open: true})
  //             expect(onBeforeOpen).toHaveBeenCalledTimes(1)
  //             expect(onOpen).not.toBeCalled()
  //             setTimeout(resolve, timeOut)
  //         })
  //     }
  //     const afterOpen = () => {
  //         expect(onBeforeOpen).toHaveBeenCalledTimes(1)
  //         expect(onOpen).toHaveBeenCalledTimes(1)
  //         expect(onBeforeClose).not.toBeCalled()
  //         expect(onClose).not.toBeCalled()
  //     }
  //     const doClose = () => {
  //         return new Promise((resolve) => {
  //             wrapper.setProps({open: false})
  //             expect(onBeforeClose).toHaveBeenCalledTimes(1)
  //             expect(onClose).not.toBeCalled()
  //             setTimeout(resolve, timeOut)
  //         })
  //     }
  //     const afterClose = () => {
  //         expect(onBeforeOpen).toHaveBeenCalledTimes(1)
  //         expect(onOpen).toHaveBeenCalledTimes(1)
  //         expect(onBeforeClose).toHaveBeenCalledTimes(1)
  //         expect(onClose).toHaveBeenCalledTimes(1)
  //     }
  //     return doOpen()
  //         .then(afterOpen)
  //         .then(doClose)
  //         .then(afterClose)
  // })
});