"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _skeletonInline = _interopRequireDefault(require("../skeleton-inline"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Related components:
 *
 * * [SkeletonBlock](#!/SkeletonBlock)
 * * [SkeletonInline](#!/SkeletonInline)
 *
 * A skeleton component that mimics lines of text on a page. Internally, it uses `<SkeletonInline />`.
 *
 * @example ./DESIGN.md
 */
const SkeletonText = props => {
  const classes = (0, _classnames.default)('pw-skeleton-text', props.className);
  const lineClasses = (0, _classnames.default)('pw-skeleton-text__line', props.lineClassName);
  const attributes = {
    role: 'presentation',
    className: classes,
    style: props.style
  };
  const skeletons = [];

  for (let i = 0; i < props.lines; i++) {
    skeletons.push( /*#__PURE__*/_react.default.createElement(_skeletonInline.default, {
      key: i,
      type: "div",
      size: props.size,
      width: props.width,
      className: lineClasses
    }));
  }

  return /*#__PURE__*/_react.default.createElement(props.type, attributes, skeletons);
};

SkeletonText.defaultProps = {
  lines: 1,
  style: {},
  type: 'span'
};
SkeletonText.propTypes = {
  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Adds values to the `class` attribute of the text line elements.
   */
  lineClassName: _propTypes.default.string,

  /**
   * Specifies the number of lines in the skeleton.
   */
  lines: _propTypes.default.number,

  /**
  * Specifies the font-size (effectively the height) of the skeleton elements,
  in any valid dimension type.
  */
  size: _propTypes.default.string,

  /**
   * Specifies any inline styles to be applied to the skeleton.
   */
  style: _propTypes.default.object,

  /**
   * Specifies the element type to be constructed.
   */
  type: _propTypes.default.string,

  /**
   * Specifies the width of the skeleton elements' children.
   */
  width: _propTypes.default.string
};
var _default = SkeletonText;
exports.default = _default;