"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.targetPropType = exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * Skip Links are a simple technique that is used to provide a fast navigation
 * option for non-mouse users. For example, users who interact using keyboards,
 * screen readers or other assistive technologies.
 *
 * `SkipLinks` remain hidden until they are focused on by tabbing to it, etc., at
 * which point they become visible.
 *
 * @url: http://webaim.org/techniques/skipnav/
 *
 * @example ./DESIGN.md
 */
const SkipLinks = ({
  className,
  items
}) => {
  const classes = (0, _classnames.default)('pw-skip-links', className);
  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes
  }, items.map(({
    target,
    label
  }, key) => /*#__PURE__*/_react.default.createElement("a", {
    href: target,
    className: "pw-skip-links__anchor",
    key: `skip-link-${target}-${key}`
  }, label)));
};

exports.default = SkipLinks;
SkipLinks.defaultProps = {
  items: []
};

const targetPropType = (props, propName) => {
  const isAlphaNumeric = new RegExp(/^#[\w\-_]+$/).test(props[propName]);

  if (!isAlphaNumeric) {
    return new Error('SkipLinks item target is invalid. Must be an ID selector, starting with "#" followed by alphanumeric characters, dashes and underscores with no spaces.');
  }

  return true;
};

exports.targetPropType = targetPropType;
SkipLinks.propTypes = {
  /**
   * An array of items, each with a `target` and a `label`.
   */
  items: _propTypes.default.arrayOf(_propTypes.default.shape({
    target: targetPropType,
    label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node])
  })).isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string
};