"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _cookieManager = _interopRequireDefault(require("../../utils/cookie-manager"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const cloneChildren = (children, props = {}) => {
  return _react.default.Children.map(children, child => {
    if (typeof child.type === 'function') {
      return /*#__PURE__*/_react.default.cloneElement(child, props);
    } else {
      return /*#__PURE__*/_react.default.cloneElement(child);
    }
  });
};
/**
 * A component that maintains the split variant value in state
 * when given a cookie name
 */


class Split extends _react.Component {
  constructor(props) {
    super(props);
    this.state = {
      splitValue: null
    };
  }

  componentDidMount() {
    this.setState({
      splitValue: this.getCookieValue()
    }, () => {
      if (this.props.splitCookieName) {
        this.cookieSubscriber = _cookieManager.default.subscribe(this.props.splitCookieName, splitValue => {
          if (this.state.splitValue !== splitValue) {
            this.setState({
              splitValue
            });
          }
        });
      }
    });
  }

  componentWillUnmount() {
    if (this.props.splitCookieName) {
      this.cookieSubscriber.unsubscribe();
    }
  }

  getCookieValue() {
    return _cookieManager.default.get(this.props.splitCookieName, this.props.defaultSplitValue);
  }

  render() {
    const {
      children,
      splitCookieName,
      showOnVariant
    } = this.props;

    if (splitCookieName) {
      if (showOnVariant) {
        if (showOnVariant === this.state.splitValue || showOnVariant === 'null' && !this.state.splitValue) {
          return /*#__PURE__*/_react.default.createElement("div", null, cloneChildren(children));
        } else {
          return null;
        }
      } else {
        return /*#__PURE__*/_react.default.createElement("div", null, cloneChildren(children, {
          splitValue: this.state.splitValue
        }));
      }
    } else {
      return children;
    }
  }

}

Split.propTypes = {
  /**
   * Node children
   */
  children: _propTypes.default.node,

  /**
   * Default value of split cookie
   */
  defaultSplitValue: _propTypes.default.string,

  /**
   * Show children only if the split value matches this value or "null"
   */
  showOnVariant: _propTypes.default.string,

  /**
   * Name of split cookie
   */
  splitCookieName: _propTypes.default.string
};
Split.defaultProps = {
  defaultSplitValue: null,
  splitCookieName: null,
  showOnVariant: null
};
var _default = Split;
exports.default = _default;