"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _index = _interopRequireDefault(require("./index"));

var _cookieManager = _interopRequireDefault(require("../../utils/cookie-manager"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
jest.useFakeTimers(); // We cannot expire the cookies in jest. Therefore, let's make sure each test
// is using a new cookie name

const TEST_COOKIENAME = 'test';
let cookieIndex = 0;

const getNewCookieName = () => {
  const newCookieName = `${TEST_COOKIENAME}_${cookieIndex}`;
  cookieIndex++;
  return newCookieName;
};

const SplitValueReader = ({
  splitValue
}) => {
  return /*#__PURE__*/_react.default.createElement("div", {
    className: "splitValue",
    "data-split-value": splitValue
  }, splitValue);
};

SplitValueReader.propTypes = {
  splitValue: _propTypes.default.string
};
let wrapper;
let currentCookieName;
describe('Split', () => {
  beforeEach(() => {
    currentCookieName = getNewCookieName();
  });
  afterEach(() => {
    wrapper.unmount();
  });
  test('default state and props are defined', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName
    }));
    const node = wrapper.instance();
    expect(node.props.splitCookieName).toBe(currentCookieName);
    expect(node.props.defaultSplitValue).toBe(null);
    expect(node.props.showOnVariant).toBe(null);
    expect(node.state.splitValue).toBe(null);
  });
  test('default split value matches splitValue in state when defined', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName,
      defaultSplitValue: "A"
    }));
    const node = wrapper.instance();
    expect(node.props.defaultSplitValue).toBe('A');
    expect(node.state.splitValue).toBe('A');
  });
  test('splitValue updates when cookie manager updates', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName,
      defaultSplitValue: "A"
    }));
    const node = wrapper.instance();
    expect(node.state.splitValue).toBe('A');

    _cookieManager.default.set(currentCookieName, 'B', null, 3000);

    _cookieManager.default.pollForAllCookieUpdate();

    expect(node.state.splitValue).toBe('B');
  });
  test('splitValue updates when cookie manager does polling update', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName,
      defaultSplitValue: "A"
    }));
    const node = wrapper.instance();
    expect(node.state.splitValue).toBe('A');

    _cookieManager.default.pollForAllCookieUpdate();

    _cookieManager.default.set(currentCookieName, 'B', null, 3000);

    jest.runOnlyPendingTimers();
    expect(node.state.splitValue).toBe('B');
  });
  test('renders children if showOnVariant value is null and cookie is not defined', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName,
      showOnVariant: "null"
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "skeleton"
    }, "Skeleton")));
    expect(wrapper.instance().state.splitValue).toBe(null);
    expect(wrapper.find('.skeleton').length).toBe(1);
  });
  test('renders children if showOnVariant value matches cookie value', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName,
      showOnVariant: "A"
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "variantA"
    }, "Variant A")));

    _cookieManager.default.set(currentCookieName, 'A');

    _cookieManager.default.pollForAllCookieUpdate();

    wrapper.update(); // Only to cover the re-render path in unit test

    _cookieManager.default.pollForAllCookieUpdate();

    wrapper.update();
    expect(wrapper.instance().state.splitValue).toBe('A');
    expect(wrapper.find('.variantA').length).toBe(1);
  });
  test('propergate splitValue props to children ', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
      splitCookieName: currentCookieName
    }, /*#__PURE__*/_react.default.createElement(SplitValueReader, null), /*#__PURE__*/_react.default.createElement(SplitValueReader, null)));

    _cookieManager.default.set(currentCookieName, 'B');

    _cookieManager.default.pollForAllCookieUpdate();

    wrapper.update();
    expect(wrapper.find('.splitValue[data-split-value="B"]').length).toBe(2);
  });
  test('renders children if splitCookieName is not defined', () => {
    wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null, /*#__PURE__*/_react.default.createElement("div", {
      className: "child"
    }, "Child")));
    expect(wrapper.find('.child').length).toBe(1);
  });
});