"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _button = _interopRequireDefault(require("../button"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * Related component:
 *
 * * [Swatch](#!/Swatch)
 *
 * This component is used to display items inside `Swatch` component.
 *
 * @example ./DESIGN.md
 */


const SwatchItem = props => {
  const {
    analyticsContent,
    analyticsName,
    children,
    className,
    color,
    disabled,
    label,
    selected,
    value,
    onClick
  } = props;
  const itemClasses = (0, _classnames.default)('pw-swatch__item', {
    'pw--selected': selected
  }, className);
  const chipClasses = (0, _classnames.default)('pw-swatch__chip', {
    'pw--disabled': disabled
  });
  const buttonClasses = (0, _classnames.default)('pw-swatch__button', {
    'pw--active': selected
  });
  return /*#__PURE__*/_react.default.createElement("div", {
    className: itemClasses,
    key: value
  }, /*#__PURE__*/_react.default.createElement(_button.default, {
    className: buttonClasses,
    innerClassName: "pw-swatch__button-inner",
    disabled: disabled,
    onClick: () => onClick(value),
    role: "radio",
    "aria-checked": selected,
    "data-analytics-name": `swatch:${analyticsName}`,
    "data-analytics-content": analyticsContent
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: chipClasses
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-swatch__chip-inner",
    style: {
      backgroundColor: color
    }
  }, children)), label && /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-swatch__outer-label"
  }, label)));
};

SwatchItem.defaultProps = {
  onClick: noop
};
SwatchItem.propTypes = {
  /**
   * Adds values to the `data-analytics-content` attribute of the input
   */
  analyticsContent: _propTypes.default.string,

  /**
   * Adds values to the `data-analytics-name` attribute of the input
   */
  analyticsName: _propTypes.default.string,

  /**
   * The children to be rendered within the chip.
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * If a valid hex or CSS rgb color is provided here,
   * it will be shown as the background-color of the tile.
   */
  color: _propTypes.default.string,

  /**
   * Indicates if this option is disabled.
   */
  disabled: _propTypes.default.bool,

  /**
   * The label to display for the option. This is rendered outside of the chip.
   */
  label: _propTypes.default.node,

  /**
   * PROVIDED INTERNALLY. Indicates if the option is selected.
   */
  selected: _propTypes.default.bool,

  /**
   * The value for the option.
   */
  value: _propTypes.default.string,

  /**
   * PROVIDED INTERNALLY.
   * If you wish to call a function when a SwatchItem is clicked,
   * use the Swatch's onChange prop.
   * This function is called whenever the user selects an option.
   * It is passed the new value.
   */
  onClick: _propTypes.default.func
};
var _default = SwatchItem;
exports.default = _default;