"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _swatchItem = _interopRequireDefault(require("./swatch-item"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * Related component:
 *
 * * [SwatchItem](#!/SwatchItem)
 *
 * `Swatch` allows the user to select from a small list of options.
 *
 * @example ./DESIGN.md
 */


const Swatch = props => {
  const {
    label,
    children,
    value,
    onChange,
    className
  } = props;
  const classes = (0, _classnames.default)('pw-swatch', className);
  let labelStr = label;

  if (typeof labelStr == 'string') {
    labelStr = label.replace(/\s/g, '-');
  }

  const labelId = labelStr;
  return /*#__PURE__*/_react.default.createElement("div", {
    className: classes,
    role: "radiogroup",
    "aria-labelledby": `swatch-label-${labelId}`
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-swatch__label",
    id: `swatch-label-${labelId}`
  }, label), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-swatch__items"
  }, _react.default.Children.map(children, child => {
    if (child && child.type && child.type.name === _swatchItem.default.name) {
      const childValue = child.props.value;
      return /*#__PURE__*/_react.default.cloneElement(child, {
        selected: childValue === value,
        key: childValue,
        onClick: onChange
      });
    } else {
      return child;
    }
  })));
};

Swatch.defaultProps = {
  label: '',
  onChange: noop
};
Swatch.propTypes = {
  /**
   * The options to choose between (Should be SwatchItems).
   */
  children: _propTypes.default.node,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * The label for the Swatch.
   */
  label: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * The selected swatch value.
   */
  value: _propTypes.default.string,

  /**
   * This function is called whenever the user selects a new value.
   * It is passed the new value.
   */
  onChange: _propTypes.default.func
};
var _default = Swatch;
exports.default = _default;