"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = require("./index.js");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
describe('Swatch', () => {
  test('Swatch renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, null));
    expect(wrapper.length).toBe(1);
  });
  /* eslint-disable newline-per-chained-call */

  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Swatch, null));
    expect(wrapper.hasClass('pw-swatch')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Swatch, null));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Swatch, {
        className: name
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('passes the selected prop to the correct child', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, {
      value: "a"
    }, /*#__PURE__*/_react.default.createElement(_index.SwatchItem, {
      value: "a"
    }), /*#__PURE__*/_react.default.createElement(_index.SwatchItem, {
      value: "b"
    }), /*#__PURE__*/_react.default.createElement(_index.SwatchItem, {
      value: "c"
    })));

    const isItemSelected = child => wrapper.find('.pw-swatch__items').childAt(child).props().selected;

    expect(isItemSelected(0)).toBe(true);
    expect(isItemSelected(1)).toBe(false);
    expect(isItemSelected(2)).toBe(false);
    wrapper.setProps({
      value: 'b'
    });
    expect(isItemSelected(0)).toBe(false);
    expect(isItemSelected(1)).toBe(true);
    expect(isItemSelected(2)).toBe(false);
  });
  test('removes spaces from string labels', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, {
      label: "hello world"
    }, /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null)));
    expect(wrapper.find('.pw-swatch__label').prop('id')).toBe('swatch-label-hello-world');
  });
  test('works correctly with non string labels', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, {
      label: 5000
    }, /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null)));
    expect(wrapper.find('.pw-swatch__label').prop('id')).toBe('swatch-label-5000');
  });
  test('passes the selected and onClick props only to SwatchItem children', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, null, /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null), /*#__PURE__*/_react.default.createElement("p", null), /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null)));
    wrapper.find(_index.SwatchItem).forEach(item => {
      expect(item.prop('selected')).not.toBeUndefined();
      expect(item.prop('onClick')).not.toBeUndefined();
    });
    const paragraph = wrapper.find('p');
    expect(paragraph.prop('selected')).toBeUndefined();
    expect(paragraph.prop('onClick')).toBeUndefined();
  });
  test('works correctly with falsy children', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Swatch, null, false, /*#__PURE__*/_react.default.createElement("p", null), undefined));
    expect(wrapper.length).toBe(1);
  });
  test('selecting an option fires the onChange prop', () => {
    const onChange = jest.fn();
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Swatch, {
      onChange: onChange
    }, /*#__PURE__*/_react.default.createElement(_index.SwatchItem, {
      value: "25"
    })));
    expect(onChange).not.toHaveBeenCalled();
    wrapper.find('button.pw-swatch__button').simulate('click');
    expect(onChange).toHaveBeenCalled();
  });
});
describe('SwatchItem', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null));
    expect(wrapper.length).toBe(1);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null));
    expect(wrapper.hasClass('pw-swatch__item')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, {
        className: name
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('if the color prop is passed, it is set as the background color', () => {
    const props = {
      color: '#ff0000'
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null));

    const chip = () => wrapper.find('.pw-swatch__chip-inner');

    expect(chip().props().style.backgroundColor).not.toBe(props.color);
    wrapper.setProps(props);
    expect(chip().props().style.backgroundColor).toBe(props.color);
  });
  test('the label is rendered, if provided', () => {
    const props = {
      label: 'test'
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, null));
    expect(wrapper.find('.pw-swatch__outer-label').length).toBe(0);
    wrapper.setProps(props);
    expect(wrapper.find('.pw-swatch__outer-label').length).toBe(1);
  });
  test('clicking the button fires the onClick props', () => {
    const props = {
      onClick: jest.fn(),
      value: 'test'
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, props));
    expect(props.onClick).not.toHaveBeenCalled();
    wrapper.find('button').simulate('click');
    expect(props.onClick).toHaveBeenCalledWith(props.value);
  });
  test('children are rendered inside the chip', () => {
    const props = {
      children: 'test'
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.SwatchItem, props));
    expect(wrapper.find('.pw-swatch__chip').text()).toBe(props.children);
  });
});