"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

var _a11y = require("../../utils/a11y");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const OVERFLOW_CHECK_INTERVAL = 200;
/**
 * This component is internally used inside `Tab` compoent.
 *
 * @example ./DESIGN.md
 */

class TabsStrip extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      overflowLeft: false,
      overflowRight: false,
      overflowWidth: 0
    };
    this.checkOverflow = (0, _lodash.default)(this.checkOverflow.bind(this), OVERFLOW_CHECK_INTERVAL);
    this.handleScroll = (0, _lodash.default)(this.handleScroll.bind(this), OVERFLOW_CHECK_INTERVAL);
  }

  componentDidMount() {
    if (this.props.isScrollable) {
      // Check if tab links are overflowing and apply styling classes
      this.checkOverflow(); // Re-calculate tab list content width when viewport changes size

      window.addEventListener('resize', this.checkOverflow);

      this._tabStrip.addEventListener('scroll', this.handleScroll, false);
    }
  }

  componentWillUnmount() {
    /* istanbul ignore else */
    if (this.props.isScrollable) {
      // Remove eventListeners if component gets removed
      window.removeEventListener('resize', this.checkOverflow);

      this._tabStrip.removeEventListener('scroll', this.handleScroll, false);
    }
  }

  checkOverflow() {
    const scrollWidth = this._tabStrip.scrollWidth;
    const containerWidth = this._tabStrip.clientWidth;

    if (scrollWidth > containerWidth) {
      this.setState({
        overflowRight: true,
        overflowWidth: scrollWidth - containerWidth
      });
    } else {
      this.setState({
        overflowLeft: false,
        overflowRight: false
      });
    }
  }

  handleScroll() {
    const container = this._tabStrip; // If scroll position is not at the initial position, set overflowLeft to true

    if (container.scrollLeft > 0 && !this.state.overflowLeft) {
      this.setState({
        overflowLeft: true
      });
    } else if (container.scrollLeft === 0) {
      this.setState({
        overflowLeft: false
      });
    } // If scroll position reached all the way to the right, set overflowRight to false


    if (container.scrollLeft < this.state.overflowWidth && !this.state.overflowRight) {
      this.setState({
        overflowRight: true
      });
    } else if (container.scrollLeft === this.state.overflowWidth) {
      this.setState({
        overflowRight: false
      });
    }
  }

  render() {
    const {
      children,
      activeIndex,
      setIndex,
      isScrollable
    } = this.props;
    const classes = (0, _classnames.default)('pw-tabs__strip-container', {
      'pw--is-scrollable': isScrollable,
      'pw--overflow-left': this.state.overflowLeft,
      'pw--overflow-right': this.state.overflowRight
    });
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement("ol", {
      role: "tablist",
      className: "pw-tabs__strip",
      ref: el => {
        this._tabStrip = el;
      }
    }, _react.default.Children.map(children, (item, idx) => {
      const isActive = idx === activeIndex;

      const onClick = () => setIndex(idx);

      const tabClasses = (0, _classnames.default)('pw-tabs__tab', {
        'pw--is-active': isActive
      });
      return /*#__PURE__*/_react.default.createElement("li", {
        className: tabClasses,
        role: "presentation",
        key: idx
      }, /*#__PURE__*/_react.default.createElement("a", {
        className: "pw-tabs__link",
        role: "tab",
        tabIndex: "0",
        "aria-selected": isActive,
        onClick: onClick,
        onKeyUp: (0, _a11y.onKeyUpWrapper)(onClick)
      }, item.props.title));
    })));
  }

}

TabsStrip.propTypes = {
  /**
   * ActiveIndex defines the active tab.
   */
  activeIndex: _propTypes.default.number,

  /**
   * The content of the TabStrip.
   */
  children: _propTypes.default.node,

  /**
   * Indicates whether the strip is scrollable or not.
   */
  isScrollable: _propTypes.default.bool,

  /**
   * This function is called wto change the active tab.
   */
  setIndex: _propTypes.default.func
};
var _default = TabsStrip;
exports.default = _default;