"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _propTypes2 = require("../../utils/prop-types");

var _tabsStrip = _interopRequireDefault(require("./tabs-strip"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const noop = () => undefined;
/**
 * Related component:
 *
 * * [TabsPanel](#!/TabsPanel)
 *
 * This component is used to display tabbed navigation.
 * User can select a tab to view its related content.
 *
 * @example ./DESIGN.md
 */


class Tabs extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      activeTabIndex: props.activeIndex
    }; // Bind class functions to object

    this.setIndex = this.setIndex.bind(this);
  }

  setIndex(index) {
    const {
      activeTabIndex
    } = this.state; // This is an optimization and does not need testing

    /* istanbul ignore if */

    if (index === activeTabIndex) {
      return;
    }

    this.setState({
      activeTabIndex: index
    });
    this.props.onChange(index);
  }

  componentWillReceiveProps(newProps) {
    if (newProps.activeIndex !== this.props.activeIndex) {
      this.setState({
        activeTabIndex: newProps.activeIndex
      });
    }
  }

  render() {
    const {
      activeTabIndex
    } = this.state;
    const {
      children,
      className,
      isScrollable
    } = this.props;
    const classes = (0, _classnames.default)('pw-tabs', className);
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes,
      "data-selected-index": activeTabIndex
    }, /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      activeIndex: activeTabIndex,
      setIndex: this.setIndex,
      isScrollable: isScrollable
    }, children), /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-tabs__panels"
    }, _react.default.Children.map(children, (child, idx) => {
      return /*#__PURE__*/_react.default.cloneElement(child, {
        isActive: activeTabIndex === idx,
        key: idx
      });
    })));
  }

}

Tabs.defaultProps = {
  activeIndex: 0,
  onChange: noop
};
Tabs.propTypes = {
  /**
   * Nodes to be used as tab panels, can be a combination of markup and/or
   * React components. There should be more than 1 child panel.
   */
  children: _propTypes.default.node.isRequired,

  /**
   * ActiveIndex defines the active panel(selected).
   */
  activeIndex: _propTypes2.childIndexProp,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Indicates whether the tab links are scrollable or not.
   */
  isScrollable: _propTypes.default.bool,

  /**
   * This function is called whenever the tab is changed.
   */
  onChange: _propTypes.default.func
};
var _default = Tabs;
exports.default = _default;