"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _index = require("./index.js");

var _tabsStrip = _interopRequireDefault(require("./tabs-strip"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
jest.mock('lodash.throttle');

_lodash.default.mockImplementation(fn => fn);

describe('TabsPanel', () => {
  test('TabsPanel renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null));
    expect(wrapper.length).toBe(1);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null));
    expect(wrapper.hasClass('pw-tabs__panel')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
        className: name
      }));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
});
describe('TabsStrip', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, null));
    expect(wrapper.length).toBe(1);
  });
  test('renders a tab strip with one entry per panel', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab').length).toBe(2);
    const anchors = wrapper.find('.pw-tabs__tab a');
    expect(anchors.at(0).text()).toBe('one');
    expect(anchors.at(1).text()).toBe('two');
  });
  test('selects the activeIndex prop tab by default', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      activeIndex: 1
    }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
  });
  test('adds resize event listener on mount if isScrollable', () => {
    const addEventListener = window.addEventListener;
    window.addEventListener = jest.fn(addEventListener);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    expect(window.addEventListener).toHaveBeenCalledWith('resize', wrapper.instance().checkOverflow);
    window.addEventListener = addEventListener;
  });
  test('removes resize event listener on unmount if isScrollable', () => {
    const removeEventListener = window.removeEventListener;
    window.removeEventListener = jest.fn(removeEventListener);
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    const handler = wrapper.instance().checkOverflow;
    expect(window.removeEventListener).not.toHaveBeenCalledWith('resize', handler);
    wrapper.unmount();
    expect(window.removeEventListener).toHaveBeenCalledWith('resize', handler);
    window.removeEventListener = removeEventListener;
  });
  test('checkOverflow sets no overflow if no overflow is necessary', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.instance()._tabStrip = {
      scrollWidth: 100,
      clientWidth: 150
    };
    wrapper.instance().checkOverflow();
    expect(wrapper.state('overflowLeft')).toBe(false);
    expect(wrapper.state('overflowRight')).toBe(false);
  });
  test('checkOverflow sets overflow width if necessary', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.instance()._tabStrip = {
      scrollWidth: 150,
      clientWidth: 100
    };
    wrapper.instance().checkOverflow();
    expect(wrapper.state('overflowLeft')).toBe(false);
    expect(wrapper.state('overflowRight')).toBe(true);
    expect(wrapper.state('overflowWidth')).toBe(50);
  });
  test('handleScroll sets the left overflow if the scroll position is not at zero', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.instance()._tabStrip = {
      scrollLeft: 5
    };
    wrapper.instance().handleScroll();
    expect(wrapper.state('overflowLeft')).toBe(true);
  });
  test('handleScroll clears the left overflow if the scroll position is at zero', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.setState({
      overflowLeft: true
    });
    wrapper.instance()._tabStrip = {
      scrollLeft: 0
    };
    wrapper.instance().handleScroll();
    expect(wrapper.state('overflowLeft')).toBe(false);
  });
  test('handleScroll sets the right overflow if the scroll position is not at maximum', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.setState({
      overflowWidth: 10,
      overflowLeft: true
    });
    wrapper.instance()._tabStrip = {
      scrollLeft: 5
    };
    wrapper.instance().handleScroll();
    expect(wrapper.state('overflowRight')).toBe(true);
  });
  test('handleScroll clears the right overflow if the scroll position is at maximum', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_tabsStrip.default, {
      isScrollable: true
    }));
    wrapper.setState({
      overflowRight: true,
      overflowWidth: 10
    });
    wrapper.instance()._tabStrip = {
      scrollLeft: 10
    };
    wrapper.instance().handleScroll();
    expect(wrapper.state('overflowRight')).toBe(false);
  });
});
describe('Tabs', () => {
  test('renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test")));
    expect(wrapper.length).toBe(1);
  });
  test('includes the component class name with no className prop', () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Tabs, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test")));
    expect(wrapper.hasClass('pw-tabs')).toBe(true);
  });
  test("does not render an 'undefined' class with no className", () => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Tabs, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test")));
    expect(wrapper.hasClass('undefined')).toBe(false);
  });
  test('renders the contents of the className prop if present', () => {
    ;
    ['test', 'test another'].forEach(name => {
      const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.Tabs, {
        className: name
      }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test")));
      expect(wrapper.hasClass(name)).toBe(true);
    });
  });
  test('has the class pw--is-scrollable if isScrollable prop is set to true', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, {
      isScrollable: true
    }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, null, "Test")));
    expect(wrapper.find('.pw-tabs__strip-container').hasClass('pw--is-scrollable')).toBe(true);
  });
  test('sets the state to the initial activeIndex', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, {
      activeIndex: 1
    }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.state('activeTabIndex')).toBe(1);
  });
  test('changes the active tab when setIndex is called', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('one');
    expect(wrapper.state('activeTabIndex')).toBe(0);
    wrapper.instance().setIndex(1);
    wrapper.update();
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
  });
  test('clicking on the tab link changes the active tab', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, null, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('one');
    expect(wrapper.state('activeTabIndex')).toBe(0);
    wrapper.find('.pw-tabs__tab a').at(1).simulate('click');
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
  });
  test('changes the active tab when the activeIndex prop is changed', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, {
      activeIndex: 0
    }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('one');
    expect(wrapper.state('activeTabIndex')).toBe(0);
    wrapper.setProps({
      activeIndex: 1
    });
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
  });
  test('does not change the active tab when non-activeIndex props are changed', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, {
      activeIndex: 1
    }, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
    wrapper.setProps({
      className: 'test'
    });
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
  });
  test('calls onChange function', () => {
    const props = {
      onChange: jest.fn()
    };
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.Tabs, props, /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "one"
    }, "Test"), /*#__PURE__*/_react.default.createElement(_index.TabsPanel, {
      title: "two"
    }, "Test")));
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('one');
    expect(wrapper.state('activeTabIndex')).toBe(0);
    expect(props.onChange).not.toBeCalled();
    wrapper.find('.pw-tabs__tab a').at(1).simulate('click');
    expect(wrapper.find('.pw-tabs__tab.pw--is-active').text()).toBe('two');
    expect(wrapper.state('activeTabIndex')).toBe(1);
    expect(props.onChange).toBeCalledWith(1);
  });
});