"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _image = _interopRequireDefault(require("../image"));

var _rating = _interopRequireDefault(require("../rating"));

var _tilePrimary = _interopRequireDefault(require("./partials/tile-primary"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/**
 * `Tile` Component is used to display product/category list.
 * It includes 4 variants (row, column, row+full and column+full).
 * It accepts image, product info (title, quantity, ...) and ratings.
 *
 * @example ./DESIGN.md
 */
const Tile = ({
  className,
  imageProps,
  href,
  title,
  options,
  price,
  quantity,
  quantityLabel,
  onClick,
  ratingProps,
  isColumn,
  isFull,
  isSimple
}) => {
  const classes = (0, _classnames.default)('pw-tile', {
    'pw--column': isColumn,
    'pw--full': isFull,
    'pw--simple': isSimple
  }, className);
  return /*#__PURE__*/_react.default.createElement("article", {
    className: classes
  }, /*#__PURE__*/_react.default.createElement(_tilePrimary.default, {
    href: href,
    onClick: onClick
  }, /*#__PURE__*/_react.default.createElement(_image.default, imageProps)), /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-tile__details"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-tile__info"
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "pw-tile__info-inner"
  }, /*#__PURE__*/_react.default.createElement("header", {
    className: "pw-tile__header"
  }, /*#__PURE__*/_react.default.createElement(_tilePrimary.default, {
    className: "pw-tile__title",
    href: href,
    onClick: onClick
  }, title)), options && /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__options`
  }, options.map((option, idx) => /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__option`,
    key: idx
  }, option.label, option.value)))), ratingProps && /*#__PURE__*/_react.default.createElement(_rating.default, (0, _extends2.default)({}, ratingProps, {
    className: (0, _classnames.default)(`pw-tile__rating-stars`, ratingProps.className)
  }))), /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__footer`
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__footer-inner`
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__quantity`
  }, quantityLabel, quantity), /*#__PURE__*/_react.default.createElement("div", {
    className: `pw-tile__price`
  }, price)))));
};

Tile.propTypes = {
  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * If provided, element will render as a link.
   */
  href: _propTypes.default.string,

  /**
   * The properties for Image.
   */
  imageProps: _propTypes.default.object,

  /**
   * Defines if Tile is column.
   */
  isColumn: _propTypes.default.bool,

  /**
   * Defines if Tile is a bigger version.
   */
  isFull: _propTypes.default.bool,

  /**
   * Defines if Tile is a simplified version.
   */
  isSimple: _propTypes.default.bool,

  /**
   * An array of information related to the product.
   */
  options: _propTypes.default.arrayOf(_propTypes.default.shape({
    label: _propTypes.default.string,
    value: _propTypes.default.string
  })),

  /**
   * Price of the product.
   */
  price: _propTypes.default.oneOfType([_propTypes.default.string, _propTypes.default.node]),

  /**
   * Quantity of the product.
   */
  quantity: _propTypes.default.number,

  /**
   * Label for quantity of the product.
   */
  quantityLabel: _propTypes.default.string,

  /**
   * The properties for Rating.
   */
  ratingProps: _propTypes.default.object,

  /**
   * Title of the product.
   */
  title: _propTypes.default.string,

  /**
   * Callback for when the primary is clicked. Not called if an href is passed.
   */
  onClick: _propTypes.default.func
};
var _default = Tile;
exports.default = _default;