"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _reactRouterDom = require("react-router-dom");

var _index = _interopRequireDefault(require("./index.jsx"));

var _rating = _interopRequireDefault(require("../rating"));

var _testing = require("../../utils/testing");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('Tile renders without errors', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.length).toBe(1);
});
/* eslint-disable newline-per-chained-call */

test('includes the component class name with no className prop', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test"
  }));
  expect(wrapper.hasClass('pw-tile')).toBe(true);
});
test("does not render an 'undefined' class with no className", () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test"
  }));
  expect(wrapper.hasClass('undefined')).toBe(false);
});
test('renders the contents of the className prop if present', () => {
  ;
  ['test', 'test another'].forEach(name => {
    const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
      title: "test",
      className: name
    }));
    expect(wrapper.hasClass(name)).toBe(true);
  });
});
test('has "pw--column" class when isColumn is true', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test",
    isColumn: true
  }));
  expect(wrapper.hasClass('pw--column')).toBe(true);
});
test('has "pw--full" class when isFull is true', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test",
    isFull: true
  }));
  expect(wrapper.hasClass('pw--full')).toBe(true);
});
test('has "pw--simple" class when isSimple is true', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test",
    isSimple: true
  }));
  expect(wrapper.hasClass('pw--simple')).toBe(true);
});
test('renders image as primary', () => {
  const wrapper = (0, _testing.mountWithRouter)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test",
    href: "test.html"
  }));
  const primary = wrapper.find(_reactRouterDom.Link).first();
  expect(primary.length).toBe(1);
  expect(primary.prop('to')).toBe('test.html');
});
test('renders title as primary', () => {
  const wrapper = (0, _testing.mountWithRouter)( /*#__PURE__*/_react.default.createElement(_index.default, {
    title: "test",
    href: "test.html"
  }));
  const primary = wrapper.find(_reactRouterDom.Link).last();
  expect(primary.length).toBe(1);
  expect(primary.prop('to')).toBe('test.html');
});
test('does not render primary as a link if no href is present', () => {
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  const primary = wrapper.find('.pw-tile__primary').first();
  expect(primary.type()).toBe('div');
});
test('renders primary as a link if href is present', () => {
  const wrapper = (0, _testing.mountWithRouter)( /*#__PURE__*/_react.default.createElement(_index.default, {
    href: "test.html"
  }));
  const primary = wrapper.find('.pw-tile__primary').first().children();
  expect(primary.type()).toBe(_reactRouterDom.Link);
});
test('If an onClick handler is passed, it is attached to the rendered component', () => {
  const mockOnClick = jest.fn();
  const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_index.default, {
    onClick: mockOnClick
  }));
  const primary = wrapper.find('.pw-tile__primary').first();
  expect(primary.prop('onClick')).toBe(mockOnClick);
  expect(mockOnClick).not.toBeCalled();
  primary.simulate('click');
  expect(mockOnClick).toBeCalled();
  expect(primary.prop('role')).toBe('button');
});
test('renders options if they are passed', () => {
  const options = [{
    label: 'one',
    value: '1'
  }, {
    label: 'two',
    value: '2'
  }];
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    options: options
  }));
  expect(wrapper.find('.pw-tile__options').length).toBe(1);
  expect(wrapper.find('.pw-tile__options .pw-tile__option').length).toBe(options.length);
});
test('renders no Rating if no ratingProps are passed', () => {
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, null));
  expect(wrapper.find(_rating.default).length).toBe(0);
});
test('renders a Rating if ratingProps are passed', () => {
  const ratingProps = {};
  const wrapper = (0, _enzyme.shallow)( /*#__PURE__*/_react.default.createElement(_index.default, {
    ratingProps: ratingProps
  }));
  const rating = wrapper.find(_rating.default);
  expect(rating.length).toBe(1);
  expect(rating.prop('className').includes('pw-tile__rating-stars')).toBe(true);
});