"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _classnames = _interopRequireDefault(require("classnames"));

var _button = _interopRequireDefault(require("../button"));

var _lodash = _interopRequireDefault(require("lodash.throttle"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
const OVERFLOW_CHECK_INTERVAL = 200;
/**
 * `Toggle` component is commonly used to visually 'truncate' its contents
 * if it crosses a preset height threshold.
 * It will show/hide the toggle button if the contents doesn't overflow pass the threshold.
 *
 * @example ./DESIGN.md
 */

class Toggle extends _react.default.Component {
  constructor(props) {
    super(props);
    this.state = {
      expanded: false,
      overflowing: true
    };
    this.toggleContent = this.toggleContent.bind(this);
    this.checkContentHeight = (0, _lodash.default)(this.checkContentHeight.bind(this), OVERFLOW_CHECK_INTERVAL);
    this.removeMaxHeight = this.removeMaxHeight.bind(this);
  }

  componentDidMount() {
    // Check if content's height is greater than the heightThreshold
    this.checkContentHeight(); // Re-calculate content height when viewport changes size

    window.addEventListener('resize', this.checkContentHeight);
  }

  componentWillUnmount() {
    // Remove eventListener if component gets removed
    window.removeEventListener('resize', this.checkContentHeight);
  }

  componentDidUpdate() {
    // Recalc height after component gets updated (For example: image loaded)
    this.checkContentHeight();
  }

  checkContentHeight() {
    const {
      heightThreshold
    } = this.props; // Avoid infinite loop with componentDidUpdate

    if (!this._content || this.state.contentHeight === this._content.clientHeight) {
      return;
    }

    this.setState({
      contentHeight: this._content.clientHeight,
      overflowing: this._content.clientHeight > heightThreshold
    });
  }

  toggleContent(evt) {
    evt.preventDefault();

    if (this.state.expanded) {
      this.setState({
        expanded: false,
        overflowing: true
      });
    } else {
      this.setState({
        expanded: true
      });
    }
  }

  removeMaxHeight(e) {
    if (this.state.expanded && e.target === this._inner) {
      this.setState({
        overflowing: false
      });
    }
  }

  render() {
    const {
      children,
      className,
      heightThreshold,
      expandLabel,
      collapseLabel
    } = this.props;
    const classes = (0, _classnames.default)('pw-toggle', className);
    const innerClasses = (0, _classnames.default)(`pw-toggle__inner`, {
      // Adds fade overlay to indicate content overflow
      'pw--overflow': this.state.overflowing
    });
    const innerStyle = {
      maxHeight: this.state.expanded ? // Expand contents to content height
      this.state.contentHeight : // Cap height to threshold when component is not expanded
      heightThreshold
    };
    const buttonLabel = this.state.expanded ? collapseLabel : expandLabel;
    return /*#__PURE__*/_react.default.createElement("div", {
      className: classes
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: innerClasses,
      style: innerStyle,
      onTransitionEnd: this.removeMaxHeight,
      ref: el => {
        this._inner = el;
      }
    }, /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-toggle__content",
      ref: el => {
        this._content = el;
      }
    }, children)), (this.state.overflowing || this.state.expanded) && /*#__PURE__*/_react.default.createElement("div", {
      className: "pw-toggle__action"
    }, /*#__PURE__*/_react.default.createElement(_button.default, {
      className: "pw-toggle__toggler",
      onClick: this.toggleContent
    }, buttonLabel)));
  }

}

Toggle.defaultProps = {
  expandLabel: 'View More',
  collapseLabel: 'View Less',
  heightThreshold: 100
};
Toggle.propTypes = {
  /**
   * Contents that will be wrapped by the Toggle component.
   */
  children: _propTypes.default.node.isRequired,

  /**
   * Adds values to the `class` attribute of the root element.
   */
  className: _propTypes.default.string,

  /**
   * Label for collapse button.
   */
  collapseLabel: _propTypes.default.node,

  /**
   * Label for expand button.
   */
  expandLabel: _propTypes.default.node,

  /**
   * Max height of the content before it needs to be 'truncated' visually.
   */
  heightThreshold: _propTypes.default.number
};
var _default = Toggle;
exports.default = _default;