"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _enzyme = require("enzyme");

var _react = _interopRequireDefault(require("react"));

var _reactUid = require("react-uid");

var _index = _interopRequireDefault(require("./index.jsx"));

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* eslint-env jest */
describe('withUniqueId higher order component', () => {
  const wrappedComponentName = 'Foo';
  const wrappedClassName = 'mock';
  const fakeText = 'Fake component';

  const mockHtml = /*#__PURE__*/_react.default.createElement("div", {
    className: wrappedClassName
  }, fakeText);

  const mockComponent = () => mockHtml;

  mockComponent.displayName = wrappedComponentName;
  const WrapperComponent = (0, _index.default)(mockComponent);
  test('withUniqueId wrapped component renders without errors', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, null));
    expect(wrapper.length).toBe(1);
  });
  test('Render UID component', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, null));
    expect(wrapper.find('UID').first().length).toBe(1);
  });
  test('Find the UUID inside the component', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_reactUid.UIDReset, null, /*#__PURE__*/_react.default.createElement(_reactUid.UIDFork, null, /*#__PURE__*/_react.default.createElement(WrapperComponent, null))));
    expect(wrapper.find(wrappedComponentName).props().id).toBe('1-1');
  });
  test('Multiple components have different ids', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(_reactUid.UIDReset, null, /*#__PURE__*/_react.default.createElement(_reactUid.UIDFork, null, /*#__PURE__*/_react.default.createElement(WrapperComponent, null), /*#__PURE__*/_react.default.createElement(WrapperComponent, null))));
    const components = wrapper.find(wrappedComponentName);
    expect(components.at(0).props().id).toBe('1-1');
    expect(components.at(1).props().id).toBe('1-2');
  });
  test('If id is defined, it is not overwritten by uid', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, {
      id: "0-0"
    }));
    expect(wrapper.find(wrappedComponentName).props().id).toBe('0-0');
  });
  test('Render children component', () => {
    const wrapper = (0, _enzyme.mount)( /*#__PURE__*/_react.default.createElement(WrapperComponent, null));
    expect(wrapper.find(wrappedComponentName).children().hasClass(wrappedClassName)).toBe(true);
    expect(wrapper.find(wrappedComponentName).children().text()).toBe(fakeText);
  });
});