"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.render = exports.WHITELISTED_INLINE_SCRIPTS = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _path = _interopRequireDefault(require("path"));

var _react = _interopRequireDefault(require("react"));

var _server = _interopRequireDefault(require("react-dom/server"));

var _reactHelmet = _interopRequireDefault(require("react-helmet"));

var _HelmetConstants = require("react-helmet/lib/HelmetConstants");

var _server2 = require("@loadable/server");

var _reactRouterDom = require("react-router-dom");

var _serializeJavascript = _interopRequireDefault(require("serialize-javascript"));

var _utils = require("../universal/utils");

var _deviceContext = _interopRequireDefault(require("../universal/device-context"));

var _ampDocument = _interopRequireDefault(require("../universal/components/_amp-document"));

var _pwaDocument = _interopRequireDefault(require("../universal/components/_pwa-document"));

var _ampApp = _interopRequireDefault(require("../universal/components/_amp-app"));

var _pwaApp = _interopRequireDefault(require("../universal/components/_pwa-app"));

var _throw = _interopRequireDefault(require("../universal/components/throw-404"));

var _appConfig = _interopRequireDefault(require("../universal/components/_app-config"));

var _switch = _interopRequireDefault(require("../universal/components/switch"));

var _routeComponent = require("../universal/components/route-component");

var errors = _interopRequireWildcard(require("../universal/errors"));

var _ssrServer = require("../../utils/ssr-server");

var _ssrShared = require("../../utils/ssr-shared");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const CWD = process.cwd();

const BUNDLES_PATH = _path.default.resolve(CWD, 'build/loadable-stats.json');

const TTI_POLYFILL_SCRIPT = [`!function(){if('PerformanceLongTaskTiming' in window){var g=window.__tti={e:[]};`, `g.o=new PerformanceObserver(function(l){g.e=g.e.concat(l.getEntries())});`, `g.o.observe({entryTypes:['longtask']})}}();`].join('');
const WHITELISTED_INLINE_SCRIPTS = [TTI_POLYFILL_SCRIPT];
/**
 * Convert from thrown Error or String to {message, status} that we need for
 * rendering.
 * @private
 * @param err - Error to be converted
 * @function
 * @return {Object}
 */

exports.WHITELISTED_INLINE_SCRIPTS = WHITELISTED_INLINE_SCRIPTS;

const logAndFormatError = err => {
  if (err instanceof errors.HTTPError) {
    // These are safe to display – we expect end-users to throw them
    return {
      message: err.message,
      status: err.status,
      stack: err.stack
    };
  } else {
    const cause = err.stack || err.toString();
    console.error(cause);
    const safeMessage = 'Internal Server Error';
    return {
      message: safeMessage,
      status: 500,
      stack: err.stack
    };
  }
};
/**
 * This is the main react-rendering function for SSR. It is an Express handler.
 *
 * @param req - Request
 * @param res - Response
 *
 * @function
 *
 * @return {Promise}
 */


const render = (req, res) => {
  // AppConfig.restore *must* come before using getRoutes() or routeComponent()
  // to inject arguments into the wrapped component's getProps methods.
  _appConfig.default.restore(res.locals);

  const routes = (0, _routeComponent.getRoutes)(res.locals);
  const renderConfig = {
    amp: {
      App: (0, _routeComponent.routeComponent)(_ampApp.default, false, res.locals),
      renderer: renderAMP
    },
    pwa: {
      App: (0, _routeComponent.routeComponent)(_pwaApp.default, false, res.locals),
      renderer: renderPWA
    }
  };
  return Promise.resolve().then(() => {
    let ret = {
      route: undefined,
      match: undefined
    };
    routes.some(route => {
      const match = (0, _reactRouterDom.matchPath)(req.path, route);

      if (match) {
        ret = {
          route,
          match
        };
      }

      return !!match;
    });
    return ret;
  }).then(({
    route,
    match
  }) => {
    // TODO: A minor problem here is that if we don't match a route, we
    // can't decide whether to render an AMP or a PWA error page - default
    // to the PWA
    const promises = [Promise.resolve().then(route ? route.component.getPageType : () => 'pwa'), route.component.getComponent()];
    return Promise.all(promises).then(([pageType, component]) => ({
      route,
      match,
      pageType,
      component
    }));
  }).then(({
    route,
    match,
    pageType,
    component
  }) => {
    const {
      App,
      renderer
    } = renderConfig[pageType];

    const initAppState = () => {
      if (component === _throw.default) {
        // Don't init if there was no match
        return Promise.resolve({
          error: new errors.HTTPNotFound('Not found'),
          appState: {}
        });
      } else {
        const {
          params,
          url
        } = match;
        const components = [App, route.component];
        const promises = components.map(c => c.getProps ? c.getProps({
          req,
          res,
          params,
          location: url
        }) : Promise.resolve({}));
        return Promise.resolve().then(() => Promise.all(promises)).then(([appProps, pageProps]) => {
          const appState = {
            appProps,
            pageProps,
            __STATE_MANAGEMENT_LIBRARY: _appConfig.default.freeze(res.locals)
          };
          return {
            error: undefined,
            appState
          };
        }).catch(error => {
          return {
            error: error || new Error(),
            appState: {}
          };
        });
      }
    };

    return Promise.resolve().then(initAppState).then(({
      appState,
      error
    }) => {
      const args = {
        req,
        res,
        routes,
        error: error && logAndFormatError(error),
        appState,
        App
      };
      return Promise.resolve().then(() => renderer(args)).catch(e => renderer(_objectSpread(_objectSpread({}, args), {}, {
        error: logAndFormatError(e)
      }))).then(({
        error,
        html,
        routerContext
      }) => {
        const redirectUrl = routerContext.url;
        const status = error && error.status || res.statusCode;

        if (redirectUrl) {
          res.redirect(302, redirectUrl);
        } else {
          res.status(status).send(html);
        }
      });
    });
  });
};

exports.render = render;

const renderAMP = args => {
  const {
    req,
    res,
    appState,
    routes,
    error,
    App
  } = args;
  const routerContext = {};
  return Promise.resolve().then(() => {
    const appHtml = _server.default.renderToStaticMarkup( /*#__PURE__*/_react.default.createElement(_reactRouterDom.StaticRouter, {
      location: req.path,
      context: routerContext
    }, /*#__PURE__*/_react.default.createElement(_appConfig.default, {
      locals: res.locals
    }, /*#__PURE__*/_react.default.createElement(_switch.default, {
      error: error,
      appState: appState,
      routes: routes,
      App: App
    }))));

    const html = _server.default.renderToStaticMarkup( /*#__PURE__*/_react.default.createElement(_ampDocument.default, {
      head: [/*#__PURE__*/_react.default.createElement("title", {
        key: "title"
      }, "Hello World")],
      html: appHtml,
      beforeBodyEnd: []
    }));

    return {
      error,
      html: ['<!doctype html>', html].join(''),
      routerContext
    };
  });
};

const renderPWA = args => {
  const {
    req,
    res,
    routes,
    appState,
    error,
    App
  } = args;
  return Promise.resolve().then(() => {
    const ssrOnly = ('mobify_server_only' in req.query);
    const prettyPrint = ('mobify_pretty' in req.query);
    const indent = prettyPrint ? 8 : 0;
    const deviceType = (0, _ssrServer.detectDeviceType)(req);
    const routerContext = {};
    let extractor;
    let bundles = [];

    let appJSX = /*#__PURE__*/_react.default.createElement(_reactRouterDom.StaticRouter, {
      location: req.path,
      context: routerContext
    }, /*#__PURE__*/_react.default.createElement(_deviceContext.default.Provider, {
      value: {
        type: deviceType
      }
    }, /*#__PURE__*/_react.default.createElement(_appConfig.default, {
      locals: res.locals
    }, /*#__PURE__*/_react.default.createElement(_switch.default, {
      error: error,
      appState: appState,
      routes: routes,
      App: App
    }))));
    /* istanbul ignore next */


    try {
      extractor = new _server2.ChunkExtractor({
        statsFile: BUNDLES_PATH
      });
      appJSX = extractor.collectChunks(appJSX);
    } catch (e) {// Tests aren't being run through webpack, therefore no chunks or `loadable-stats.json`
      // file is being created. This causes a file read exception. For this
      // reason, swallow the error and carry on when in a test environment.
    } // It's important that we render the App before extracting the script elements,
    // otherwise it won't return the correct chunks.


    const appHtml = _server.default.renderToString(appJSX); // Setting type: 'application/json' stops the browser from executing the code.


    const scriptProps = ssrOnly ? {
      type: 'application/json'
    } : {};
    /* istanbul ignore next */

    if (extractor) {
      // Clone elements with the correct bundle path.
      bundles = extractor.getScriptElements().map(el => /*#__PURE__*/_react.default.cloneElement(el, _objectSpread(_objectSpread(_objectSpread({}, el.props), scriptProps), {}, {
        src: el.props.src && (0, _utils.getAssetUrl)(el.props.src.slice(1))
      })));
    }

    const helmet = _reactHelmet.default.renderStatic(); // Do not include *dynamic*, executable inline scripts – these cause issues with
    // strict CSP headers that customers often want to use. Avoid inline scripts,
    // full-stop, whenever possible.


    return Promise.resolve().then(() => {
      // Each key in `windowGlobals` is expected to be set on the window
      // object, client-side, by code in ssr/browser/main.jsx.
      //
      // Do *not* add to these without a very good reason - globals are a liability.
      const windowGlobals = {
        __DEVICE_TYPE__: deviceType,
        __PRELOADED_STATE__: appState,
        __ERROR__: error,
        // `window.Progressive` has a long history at Mobify and some
        // client-side code depends on it. Maintain its name out of tradition.
        Progressive: getWindowProgressive(req, res)
      };
      const scripts = [/*#__PURE__*/_react.default.createElement("script", {
        id: "mobify-data",
        key: "mobify-data",
        type: "application/json" // Not executable
        ,
        dangerouslySetInnerHTML: {
          __html: (0, _serializeJavascript.default)(windowGlobals, {
            isJSON: true,
            space: indent
          })
        }
      }), ...bundles];

      const helmetHeadTags = _HelmetConstants.VALID_TAG_NAMES.map(tag => helmet[tag] && helmet[tag].toComponent()).filter(tag => tag);

      const html = _server.default.renderToString( /*#__PURE__*/_react.default.createElement(_pwaDocument.default, {
        head: [/*#__PURE__*/_react.default.createElement("script", (0, _extends2.default)({
          id: "performance-metrics",
          key: "performance-metrics",
          dangerouslySetInnerHTML: {
            __html: TTI_POLYFILL_SCRIPT
          }
        }, scriptProps)), ...helmetHeadTags, /*#__PURE__*/_react.default.createElement("link", {
          key: "stylesheet",
          rel: "stylesheet",
          href: (0, _utils.getAssetUrl)('main.css')
        })],
        html: appHtml,
        beforeBodyEnd: scripts,
        htmlAttributes: helmet.htmlAttributes.toComponent(),
        bodyAttributes: helmet.bodyAttributes.toComponent()
      }));

      return {
        error,
        html: ['<!doctype html>', html].join(''),
        routerContext
      };
    });
  });
};

const getWindowProgressive = (req, res) => {
  const options = req.app.options || {};
  return {
    buildOrigin: (0, _utils.getAssetUrl)(''),
    cacheManifest: options.cacheHashManifest || {},
    ssrOptions: {
      // The hostname and origin under which this page is served
      appHostname: options.appHostname,
      appOrigin: options.appOrigin,
      // The id of the bundle being served, as a string,
      // defaulting to 'development' for the local dev server
      bundleId: process.env.BUNDLE_ID || 'development',
      // The id of the deploy as a string, defaulting to '0'
      // for the local dev server
      deployId: process.env.DEPLOY_ID || '0',
      // On a local dev server, the DEPLOY_TARGET environment variable
      // isn't defined by default. Developers may define it if it's
      // used by the UPWA to modify behaviour.
      deployTarget: process.env.DEPLOY_TARGET || 'local',
      proxyConfigs: _ssrShared.proxyConfigs,
      // The request class (undefined by default)
      requestClass: res.locals.requestClass
    }
  };
};