"use strict";

var _a11y = require("./a11y");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('onKeyUpHelper calls the given function if the event key matches the given key or space', () => {
  const mockOnClick = jest.fn();
  (0, _a11y.onKeyUpHelper)({
    keyCode: _a11y.keyMap.enter
  }, mockOnClick, _a11y.keyMap.enter);
  expect(mockOnClick).toHaveBeenCalledTimes(1);
  (0, _a11y.onKeyUpHelper)({
    keyCode: _a11y.keyMap.space
  }, mockOnClick, _a11y.keyMap.enter);
  expect(mockOnClick).toHaveBeenCalledTimes(2);
});
test('onKeyUpHelper defaults to space', () => {
  const mockOnClick = jest.fn();
  (0, _a11y.onKeyUpHelper)({
    keyCode: _a11y.keyMap.space
  }, mockOnClick);
  expect(mockOnClick).toHaveBeenCalledTimes(1);
});
test('onKeyUpHelper does not call the function if the key does not match', () => {
  const mockOnClick = jest.fn();
  (0, _a11y.onKeyUpHelper)({
    keyCode: _a11y.keyMap.tab
  }, mockOnClick, _a11y.keyMap.enter);
  expect(mockOnClick).not.toBeCalled();
});
test('onKeyUpHelper does not break if the onClick is not a function', () => {
  (0, _a11y.onKeyUpHelper)({
    keyCode: _a11y.keyMap.space
  }, null);
});
test('onKeyUpWrapper splits up the arguments to onKeyUpHelper usefully', () => {
  const mockOnClick = jest.fn();
  const wrapper = (0, _a11y.onKeyUpWrapper)(mockOnClick, _a11y.keyMap.enter);
  expect(mockOnClick).not.toBeCalled();
  wrapper({
    keyCode: _a11y.keyMap.enter
  });
  expect(mockOnClick).toHaveBeenCalledTimes(1);
  wrapper({
    keyCode: _a11y.keyMap.space
  });
  expect(mockOnClick).toHaveBeenCalledTimes(2);
  wrapper({
    keyCode: _a11y.keyMap.tab
  });
  expect(mockOnClick).toHaveBeenCalledTimes(2);
});
test('onKeyUpHelper passes the event to the function it calls', () => {
  const mockOnClick = jest.fn();
  const event = {
    keyCode: _a11y.keyMap.space
  };
  (0, _a11y.onKeyUpHelper)(event, mockOnClick);
  expect(mockOnClick).toHaveBeenCalledWith(event);
});