"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.childIndexProp = exports.percentage = exports.positiveNumber = exports.positiveValue = void 0;

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
// Custom PropType Validators
const validator = (valuePredicate, typeDescription) => (props, propName, componentName) => {
  const value = props[propName];

  if (valuePredicate(value, props)) {
    return null;
  }

  return new Error(`Invalid prop ${propName} supplied to ${componentName}. ${propName} should be ${typeDescription}.`);
};

const positiveValue = validator(value => (typeof value === 'number' || typeof value === 'string') && value > 0, 'a number or numeric string greater than 0');
exports.positiveValue = positiveValue;
const positiveNumber = validator(value => typeof value === 'number' && value >= 0, 'a number greater than or equal to 0');
exports.positiveNumber = positiveNumber;
const percentage = validator(value => {
  const numberValue = parseFloat(value);
  return numberValue >= 1 && numberValue <= 100;
}, "a percentage value from '1%' to '100%'");
exports.percentage = percentage;
const childIndexProp = validator((value, props) => {
  const numberValue = parseInt(value, 10);
  return numberValue >= 0 && numberValue < props.children.length;
}, 'a number value from 0 to 1 less than the tab panels count');
exports.childIndexProp = childIndexProp;