"use strict";

var _propTypes = require("./prop-types");

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */

/* Copyright (c) 2020 Mobify Research & Development Inc. All rights reserved. */

/* * *  *  * *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  * */
test('positiveValue returns an Error if the value is a negative number/string', () => {
  ;
  [-1, -2, '-1', '-2'].forEach(value => {
    const result = (0, _propTypes.positiveValue)({
      value
    }, 'value', 'Test');
    expect(result.toString().startsWith('Error:')).toBe(true);
  });
});
test('positiveValue returns an Error if the value is zero or a string with zero', () => {
  ;
  [0, '0', '00'].forEach(value => {
    const result = (0, _propTypes.positiveValue)({
      value
    }, 'value', 'Test');
    expect(result.toString().startsWith('Error:')).toBe(true);
  });
});
test('positiveValue returns null if the value is a positive number/string', () => {
  ;
  [1, 2, '1', '2'].forEach(value => {
    const result = (0, _propTypes.positiveValue)({
      value
    }, 'value', 'Test');
    expect(result).toBeNull();
  });
});
test('positiveNumber returns an Error if the value is a negative number', () => {
  const result = (0, _propTypes.positiveNumber)({
    test: -1
  }, 'test', 'Test');
  expect(result.toString().startsWith('Error:')).toBe(true);
});
test('positiveNumber returns an Error if the value is not a number', () => {
  const result = (0, _propTypes.positiveNumber)({
    test: 'test'
  }, 'test', 'Test');
  expect(result.toString().startsWith('Error:')).toBe(true);
});
test('positiveNumber returns null if the value is a positive number', () => {
  expect((0, _propTypes.positiveNumber)({
    test: 5
  }, 'test', 'Test')).toBe(null);
});
test('percentage should return null for a valid nonzero percentage', () => {
  ;
  [1, 5, 10, 50, 75, 100].forEach(value => {
    expect((0, _propTypes.percentage)({
      value
    }, 'value', 'Test')).toBe(null);
  });
});
test('percentage should return an error for values outside the range', () => {
  ;
  [0, 250, -5].forEach(value => {
    const result = (0, _propTypes.percentage)({
      value
    }, 'value', 'Test');
    expect(result.toString().startsWith('Error:')).toBe(true);
  });
});
test('childIndexProp returns null if the value is a valid index', () => {
  expect((0, _propTypes.childIndexProp)({
    test: 1,
    children: [0, 1]
  }, 'test', 'Test')).toBe(null);
});
test('childIndexProp returns an error if the value is an invalid index', () => {
  const result = (0, _propTypes.childIndexProp)({
    test: 2,
    children: [0]
  }, 'test', 'Test');
  expect(result.toString().startsWith('Error:')).toBe(true);
});