"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _fs = _interopRequireDefault(require("fs"));

var _glob = _interopRequireDefault(require("glob"));

var _path = _interopRequireDefault(require("path"));

var _webpack = _interopRequireDefault(require("webpack"));

var _webpackNotifier = _interopRequireDefault(require("webpack-notifier"));

var _autoprefixer = _interopRequireDefault(require("autoprefixer"));

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

var _terserWebpackPlugin = _interopRequireDefault(require("terser-webpack-plugin"));

var _copyWebpackPlugin = _interopRequireDefault(require("copy-webpack-plugin"));

var _timeFixPlugin = _interopRequireDefault(require("time-fix-plugin"));

var _webpackBundleAnalyzer = require("webpack-bundle-analyzer");

var _webpackPlugin = _interopRequireDefault(require("@loadable/webpack-plugin"));

var _plugins = require("./plugins");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

const root = process.cwd();
const {
  resolve,
  join
} = _path.default;

const pkg = require(resolve(root, 'package.json'));

const nodeModules = resolve(root, 'node_modules');
const appDir = resolve(root, 'app');
const buildDir = resolve(root, 'build');
const production = 'production';
const development = 'development';
const modes = [production, development];
const analyzeBundle = process.env.MOBIFY_ANALYZE === 'true';
const mode = process.env.NODE_ENV === production ? production : development;
const DEBUG = mode !== production && process.env.DEBUG === 'true';
const CI = process.env.CI;
const requestProcessorPath = resolve(appDir, 'request-processor.js');

if (modes.indexOf(mode) < 0) {
  throw new Error(`Mode '${mode}' must be one of '${modes.toString()}'`);
}

const buildMarkerPlugin = new _plugins.BuildMarkerPlugin({
  filename: resolve(root, 'build', 'build.marker')
});
const replacements = [{
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_app-config'),
  newPath: resolve('.', 'app', 'components', '_app-config', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_amp-document'),
  newPath: resolve('.', 'app', 'components', '_amp-document', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_pwa-document'),
  newPath: resolve('.', 'app', 'components', '_pwa-document', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_amp-app'),
  newPath: resolve('.', 'app', 'components', '_amp-app', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_pwa-app'),
  newPath: resolve('.', 'app', 'components', '_pwa-app', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'components', '_error'),
  newPath: resolve('.', 'app', 'components', '_error', 'index.jsx')
}, {
  path: join('progressive-web-sdk', 'dist', 'ssr', 'universal', 'routes'),
  newPath: resolve('.', 'app', 'routes.jsx')
}].filter(({
  newPath
}) => _fs.default.existsSync(newPath));
const moduleReplacementPlugin = (0, _plugins.createModuleReplacementPlugin)({
  replacements
});
const defines = {
  // This is defined as a boolean, not a string
  MESSAGING_ENABLED: `${pkg.messagingEnabled}`,
  WEBPACK_NON_PWA_ENABLED: `${pkg.nonPwaEnabled}`,
  NATIVE_WEBPACK_ASTRO_VERSION: `'0.0.1'`,
  // TODO
  MESSAGING_SITE_ID: `'${pkg.messagingSiteId}'`,
  // This is for internal Mobify test use
  MOBIFY_CONNECTOR_NAME: `'${process.env.MOBIFY_CONNECTOR_NAME}'`,
  // These are defined as string constants
  PROJECT_SLUG: `'${pkg.projectSlug}'`,
  AJS_SLUG: `'${pkg.aJSSlug}'`,
  SITE_NAME: `"${pkg.siteName}"`,
  WEBPACK_PACKAGE_JSON_MOBIFY: `${JSON.stringify(pkg.mobify || {})}`,
  WEBPACK_SSR_ENABLED: pkg.mobify ? `${pkg.mobify.ssrEnabled}` : 'false',
  WEBPACK_SITE_URL: `'${pkg.siteUrl}'`,
  DEBUG,
  WEBPACK_PAGE_NOT_FOUND_URL: `'${(pkg.mobify || {}).pageNotFoundURL || ''}' `,
  NODE_ENV: `'${process.env.NODE_ENV}'`,
  ['global.GENTLY']: false
};
/**
 * We use this trick to make it possible to swap connectors used by the scaffold
 * during development by changing the MOBIFY_CONNECTOR_NAME env var. We don't
 * expect end-users to do this. Note that require('mobify-test-connector') in
 * code will return *different modules* depending on the value of the env var.
 */

const getConnectorPath = () => {
  const name = process.env.MOBIFY_CONNECTOR_NAME || 'demo-connector';
  return _path.default.resolve(nodeModules, '@mobify', 'commerce-integrations', 'dist', 'connectors', name);
};

const minimizer = mode => {
  switch (mode) {
    case production:
      return [new _terserWebpackPlugin.default({
        cache: true,
        parallel: true,
        sourceMap: true
      })];

    case development:
      return [new _terserWebpackPlugin.default({
        cache: true,
        parallel: true,
        sourceMap: true,
        terserOptions: {
          ie8: false,
          mangle: false,
          warnings: false,
          compress: false
        }
      })];

    default:
      throw new Error(`Invalid mode ${mode}`);
  }
};

const cssLoader = [_miniCssExtractPlugin.default.loader, {
  loader: 'css-loader?-autoprefixer',
  options: {
    // Don't use automatic URL transforms
    url: false
  }
}, {
  loader: 'postcss-loader',
  options: {
    plugins: [(0, _autoprefixer.default)({
      // Don't remove outdated prefixes. Speeds up build time.
      remove: false
    })]
  }
}];
const sassLoader = cssLoader.concat(['sass-loader']);
const babelLoader = [{
  loader: 'babel-loader?cacheDirectory',
  options: {
    rootMode: 'upward'
  }
}]; // Avoid compiling server-side only libraries with webpack by setting the
// webpack `externals` configuration. This values originates from the mobify
// configuration object under `externals` in the projects package.json file.

const mobifyConfig = pkg.mobify || {}; // Convert the externals defined in your project with the defualts into an
// object that webpack will understand.

const externals = ['express', ...(mobifyConfig.externals || [])].reduce((acc, lib) => _objectSpread(_objectSpread({}, acc), {}, {
  [lib]: lib
}), {});
const stats = {
  all: false,
  modules: false,
  errors: true,
  warnings: true,
  moduleTrace: true,
  errorDetails: true,
  colors: true,
  assets: false,
  excludeAssets: [/.*img\/.*/, /.*svg\/.*/, /.*json\/.*/, /.*static\/.*/]
};
const common = {
  mode,
  // Reduce amount of output in terminal
  stats,
  // Create source maps for all files
  devtool: 'source-map',
  output: {
    path: buildDir,
    filename: '[name].js',
    chunkFilename: '[name].js' // Support chunking with @loadable/components

  },
  // Tell webpack how to find specific modules
  resolve: {
    symlinks: false,
    extensions: ['.js', '.jsx', '.json'],
    alias: {
      '@loadable/components': resolve(nodeModules, '@loadable/components'),
      '@loadable/server': resolve(nodeModules, '@loadable/server'),
      '@loadable/webpack-plugin': resolve(nodeModules, '@loadable/webpack-plugin'),
      'babel-runtime': resolve(nodeModules, 'babel-runtime'),
      lodash: resolve(nodeModules, 'lodash'),
      'lodash-es': resolve(nodeModules, 'lodash'),
      'lodash._basefor': resolve(nodeModules, 'lodash', '_baseFor'),
      'lodash.escaperegexp': resolve(nodeModules, 'lodash', 'escapeRegExp'),
      'lodash.find': resolve(nodeModules, 'lodash', 'find'),
      'lodash.frompairs': resolve(nodeModules, 'lodash', 'fromPairs'),
      'lodash.isarray': resolve(nodeModules, 'lodash', 'isArray'),
      'lodash.isarguments': resolve(nodeModules, 'lodash', 'isArguments'),
      'lodash.intersection': resolve(nodeModules, 'lodash', 'intersection'),
      'lodash.isplainobject': resolve(nodeModules, 'lodash', 'isPlainObject'),
      'lodash.keys': resolve(nodeModules, 'lodash', 'keys'),
      'lodash.keysin': resolve(nodeModules, 'lodash', 'keysIn'),
      'lodash.mapvalues': resolve(nodeModules, 'lodash', 'mapValues'),
      'lodash.throttle': resolve(nodeModules, 'lodash', 'throttle'),
      react: resolve(nodeModules, 'react'),
      'react-router-dom': resolve(nodeModules, 'react-router-dom'),
      'react-dom': resolve(nodeModules, 'react-dom'),
      'react-helmet': resolve(nodeModules, 'react-helmet'),
      bluebird: resolve(nodeModules, 'bluebird'),
      'mobify-test-connector': getConnectorPath()
    }
  },
  plugins: [new _webpack.default.DefinePlugin(defines), new _webpackNotifier.default({
    title: `Mobify Project: ${pkg.name}`,
    excludeWarnings: true,
    skipFirstNotification: true
  }), new _miniCssExtractPlugin.default({
    filename: '[name].css'
  }), new _copyWebpackPlugin.default({
    patterns: [{
      from: 'app/static/',
      to: 'static/'
    }]
  }), analyzeBundle && new _webpackBundleAnalyzer.BundleAnalyzerPlugin({
    analyzerMode: 'static',
    defaultSizes: 'gzip',
    openAnalyzer: CI !== 'true',
    generateStatsFile: true
  }), mode === development && new _webpack.default.NoEmitOnErrorsPlugin(), // Avoid repeat builds with webpack for files created immediately
  // before starting the server.
  // See - https://github.com/webpack/watchpack/issues/25
  new _timeFixPlugin.default(), buildMarkerPlugin, moduleReplacementPlugin].filter(x => !!x),
  module: {
    rules: [{
      test: /\.js(x?)$/,
      exclude: /node_modules/,
      use: babelLoader
    }, {
      test: /\.svg$/,
      use: 'text-loader'
    }, {
      test: /\.css?$/,
      use: cssLoader
    }, {
      test: /\.scss$/,
      use: sassLoader,
      include: [/node_modules\/progressive-web-sdk/, /app/]
    }, {
      test: /\.html$/,
      exclude: /node_modules/,
      use: {
        loader: 'html-loader'
      }
    }]
  },
  externals
}; // The main PWA entry point gets special treatment for chunking

const main = (0, _extends2.default)({}, common, {
  name: 'pwa-main',
  entry: {
    main: './app/main.jsx'
  },
  optimization: {
    splitChunks: {
      cacheGroups: {
        vendor: {
          // Anything imported from node_modules lands in vendor.js
          test: /node_modules/,
          name: 'vendor',
          chunks: 'all'
        }
      }
    },
    minimizer: minimizer(mode)
  },
  performance: {
    maxEntrypointSize: 905000,
    maxAssetSize: 825000
  },
  plugins: [...common.plugins, new _webpackPlugin.default()]
});
const others = (0, _extends2.default)({}, common, {
  name: 'pwa-others',
  entry: (0, _extends2.default)({
    loader: './app/loader.js',
    worker: './worker/main.js',
    'core-polyfill': 'core-js',
    'fetch-polyfill': 'whatwg-fetch'
  }, _glob.default.sync('./app/translations/*.js').reduce((acc, fileName) => {
    const name = _path.default.basename(fileName).slice(0, -_path.default.extname(fileName).length);

    acc[`translations/${name}`] = fileName;
    return acc;
  }, {})),
  optimization: {
    minimizer: minimizer(mode)
  }
});
/**
 * Configuration for the Express app which is run under Node.
 */

const ssrServerConfig = (0, _extends2.default)({}, {
  name: 'ssr-server',
  mode,
  devtool: 'cheap-source-map',
  // Always use source map, makes debugging the server much easier.
  entry: './app/ssr.js',
  target: 'node',
  output: {
    path: buildDir,
    filename: 'ssr.js',
    libraryTarget: 'commonjs2'
  },
  resolve: {
    extensions: ['.js', '.jsx', '.json'],
    alias: common.resolve.alias
  },
  plugins: [new _webpack.default.DefinePlugin(defines), // Output a single server file for faster Lambda startup
  new _webpack.default.optimize.LimitChunkCountPlugin({
    maxChunks: 1
  }), buildMarkerPlugin, moduleReplacementPlugin],
  externals,
  module: {
    rules: [{
      test: /\.js(x?)$/,
      exclude: /node_modules/,
      use: babelLoader
    }, {
      test: /\.svg$/,
      use: 'text-loader'
    }]
  },
  stats
}, mode === production ? {
  optimization: {
    minimizer: [new _terserWebpackPlugin.default({
      terserOptions: {
        compress: false,
        mangle: false,
        ecma: 6
      }
    })]
  }
} : undefined);
const requestProcessor = (0, _extends2.default)({}, {
  entry: './app/request-processor.js',
  target: 'node',
  mode,
  output: {
    path: resolve(process.cwd(), 'build'),
    filename: 'request-processor.js',
    // Output a CommonJS module for use in Node
    libraryTarget: 'commonjs2'
  },
  module: {
    rules: [{
      test: /\.js$/,
      exclude: /node_modules/,
      use: babelLoader
    }]
  },
  stats
});
const entries = [main, ssrServerConfig, others];

if (_fs.default.existsSync(requestProcessorPath)) {
  entries.push(requestProcessor);
}

module.exports = entries;